/**
 * A locking example as from T. A. Henzinger, R. Jhala, and
 * R. Majumdar, "Lazy Abstraction", in Proceedings of POPL 2002.
 */
public class LockUnlock {
	boolean LOCK;
	boolean _A;
	boolean _ERROR_0;
	boolean _ERROR_1;

    /**
     * Seizes the lock represented by the instance variable LOCK.
     * Calling the lock() function when the lock is already
     * occupied causes _ERROR_0.
     */ 
	void lock() {
		if (!LOCK)
			LOCK = true;
		else
			_ERROR_0 = true;
	} // lock()
	
    /**
     * Releases the lock represented by the instance variable LOCK.
     * Calling the unlock() function when the lock is not occupied
     * causes _ERROR_1.
     */ 
	void unlock() {
		if (LOCK)
			LOCK = false;
		else
			_ERROR_1 = true;
	} // unlock()
	
    /**
     * The core of locking example -- some locking and unlocking actions
     * as in fig. 1 of the paper.
     * We preserved the fact that in the paper the variable new (here 
     * renamed as nnew to avoid clashing with the new keyword) is not 
     * initialized by declaring it as a parameter.
     */
	void example(int nnew) {
		int got_lock;
		int old;
		
		if (_A) {
			do {
				got_lock = 0;
				if (_A) {
					lock();
					got_lock++;
				}
				if (got_lock != 0) {
					unlock();
				}
			} while (_A);
		} 
		
		do {
			lock();
			old = nnew;
			if (_A) {
				unlock();
				nnew++;
			}
		} while (nnew != old);
		unlock();
	} // example()
}
