/*
 TSAFE Prototype: A decision support tool for air traffic controllers
 Copyright (C) 2003  Gregory D. Dennis

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tsafe;

import tsafe.main.*;
import tsafe.engine.*;
import tsafe.data.*;

import java.util.*;

/**
 * This is application that executes part of TSAFE.
 * It is based on class tsafe.main.gui.ConfigConsole.java,
 * however, not important parts were removed.
 */
public class Mytsafe {
       
    public static void main(String args[]) {

        // Make the bounds of the map area.
        // latitude: south pole -90; north pole +90
        // longitude: Greenwich 0; west -180; east +180 
        double minLat = 30;
        double minLon = 0;
        double maxLat = 60;
        double maxLon = 30;
        LatLonBounds bounds = new LatLonBounds(minLat, minLon, maxLat, maxLon);
        System.out.println("Constructed the map bounds.");
       
        // Make the Engine Calculator.
        EngineCalculator calc = new SimpleCalculator(bounds);
        System.out.println("Constructed calculator.");

        // Make the Tsafe Engine.
        TsafeEngine engine = new TsafeEngine(calc);
        System.out.println("Constructed Tsafe engine.");

        // Prepare some flights to check.
        LinkedList flights = new LinkedList();
        
        // double lat, double lon, double alt, long time, double speed, double heading
        // heading: < 0 ; 2pi > 
        FlightTrack track;
        // Route is a list of fixes.
        Route route;
        // double speed, double altitude, Route route
        FlightPlan plan;
        
        track = new FlightTrack(46, 10, 6050, 0, 800, Math.PI/3);
        route = new Route();
        // String id, double lat, double lon
        route.addFix(new Fix("fix0",45.4637,9.1881));  // Milano
        route.addFix(new Fix("fix1",50.0878,14.4205)); // Prague
        route.addFix(new Fix("fix2",49.1911,16.6114)); // Brno
        plan = new FlightPlan(750, 6000, route);
        Flight flight1 = new Flight("FL001: MXP-PRG-BRQ", track, plan);
        flights.add(flight1);    

        track = new FlightTrack(45.4629, 8, 6000, 0, 800, Math.PI);
        route = new Route();
        // String id, double lat, double lon
        route.addFix(new Fix("fix0",45.4640,9.1881));  // Milano
        route.addFix(new Fix("fix1",45.4630,7.6867));  // Torino
        route.addFix(new Fix("fix2",45.4620,9.1881));  // Milano
        plan = new FlightPlan(800, 6000, route);
        Flight flight2 = new Flight("FL002: Mi-To-Mi", track, plan);
        flights.add(flight2);    

        engine.setFlights(flights);

        // Check the flights conformance.
        engine.start();

        return;
    }
    
}
