package muse.algo;

import static muse.bc.Opcodes.*;
import muse.bc.BytecodeDispatcher;
import muse.mem.Operator;
import muse.mem.Type;


public class BytecodeDispatcherAlgorithm extends BytecodeDispatcher<Algorithm> {
    private SEInit seInit = null;
    private SEUnexpected seUnexpected = null;
    private SEAconst_null seAconst_null = null;
    private SEAdd seAdd = null;
    private SEAload seAload = null;
    private SEAnd seAnd = null;
    private SEAnewarray seAnewarray = null;
    private SEArrayLength seArrayLength = null;
    private SEAstore seAstore = null;
    private SEAthrow seAthrow = null;
    private SEBipush seBipush = null;
    private SECheckcast seCheckcast = null;
    private SEConst seConst = null;
    private SEDiv seDiv = null;
    private SEDup_n seDup_n = null;
    private SEDup seDup = null;
    private SEGetfield seGetfield = null;
    private SEGetstatic seGetstatic = null;
    private SEGoto seGoto = null;
    private SEIfacmp seIfacmp = null;
    private SEIfcond seIfcond = null;
    private SEIfnull seIfnull = null;
    private SEIinc seIinc = null;
    private SEInstanceof seInstanceof = null;
    private SEInvokeSpecial seInvokeSpecial = null;
    private SEInvokeStatic seInvokeStatic = null;
    private SEInvokeVirtualInterface seInvokeVirtualInterface = null;
    private SEIUshr seIUshr = null;
    private SEJsr seJsr = null;
    private SELdc seLdc = null;
    private SELoad seLoad = null;
    private SEMonitor seMonitor = null;
    private SEMul seMul = null;
    private SEMultianewarray seMultianewarray = null;
    private SEN2n seN2n = null;
    private SENcmp seNcmp = null;
    private SENeg seNeg = null;
    private SENew seNew = null;
    private SENewarray seNewarray = null;
    private SENop seNop = null;
    private SEOr seOr = null;
    private SEPop sePop = null;
    private SEPutfield sePutfield = null;
    private SEPutstatic sePutstatic = null;
    private SERem seRem = null;
    private SERet seRet = null;
    private SEReturn seReturn = null;
    private SEShl seShl = null;
    private SEShr seShr = null;
    private SESipush seSipush = null;
    private SEStore seStore = null;
    private SESub seSub = null;
    private SESwap seSwap = null;
    private SESwitch seSwitch = null;
    private SEWide seWide = null;
    private SEXor seXor = null;

	private class DispatchStrategy_UNDEFINED implements BytecodeDispatcher.DispatchStrategy<Algorithm> {
		public Algorithm doIt() throws UndefInstructionException {
        	throw new UndefInstructionException("LUSHR");
		}
	}

	private class DispatchStrategy_NOP implements BytecodeDispatcher.DispatchStrategy<SENop> {
		public SENop doIt() {
			if (BytecodeDispatcherAlgorithm.this.seNop == null) {
				BytecodeDispatcherAlgorithm.this.seNop = new SENop();
			}
			return BytecodeDispatcherAlgorithm.this.seNop;
		}
	}
	
	private class DispatchStrategy_XLOAD implements BytecodeDispatcher.DispatchStrategy<SELoad> {
		private boolean def;
		private int index;
		public DispatchStrategy_XLOAD() { this.def = false; }
		public DispatchStrategy_XLOAD(int index) { this.def = true; this.index = index; }
		public SELoad doIt() {
			if (BytecodeDispatcherAlgorithm.this.seLoad == null) {
				BytecodeDispatcherAlgorithm.this.seLoad = new SELoad();
			}
			BytecodeDispatcherAlgorithm.this.seLoad.def = this.def;
			BytecodeDispatcherAlgorithm.this.seLoad.index = this.index;
			return BytecodeDispatcherAlgorithm.this.seLoad;
		}
	}
	
	private class DispatchStrategy_XSTORE implements BytecodeDispatcher.DispatchStrategy<SEStore> {
		private boolean def;
		private int index;
		public DispatchStrategy_XSTORE() { this.def = false; }
		public DispatchStrategy_XSTORE(int index) { this.def = true; this.index = index; }
		public SEStore doIt() {
			if (BytecodeDispatcherAlgorithm.this.seStore == null) {
				BytecodeDispatcherAlgorithm.this.seStore = new SEStore();
			}
			BytecodeDispatcherAlgorithm.this.seStore.def = this.def;
			BytecodeDispatcherAlgorithm.this.seStore.index = this.index;
			return BytecodeDispatcherAlgorithm.this.seStore;
		}
	}
	
	private class DispatchStrategy_BIPUSH implements BytecodeDispatcher.DispatchStrategy<SEBipush> {
		public SEBipush doIt() {
			if (BytecodeDispatcherAlgorithm.this.seBipush == null) {
				BytecodeDispatcherAlgorithm.this.seBipush = new SEBipush();
			}
			return BytecodeDispatcherAlgorithm.this.seBipush;
		}
	}

	private class DispatchStrategy_SIPUSH implements BytecodeDispatcher.DispatchStrategy<SESipush> {
		public SESipush doIt() {
			if (BytecodeDispatcherAlgorithm.this.seSipush == null) {
				BytecodeDispatcherAlgorithm.this.seSipush = new SESipush();
			}
			return BytecodeDispatcherAlgorithm.this.seSipush;
		}
	}

	private class DispatchStrategy_LDCX_Y implements BytecodeDispatcher.DispatchStrategy<SELdc> {
		private boolean def;
		public DispatchStrategy_LDCX_Y(boolean def) {
			this.def = def;
		}
		public SELdc doIt() {
			if (BytecodeDispatcherAlgorithm.this.seLdc == null) {
				BytecodeDispatcherAlgorithm.this.seLdc = new SELdc();
			}
			BytecodeDispatcherAlgorithm.this.seLdc.def = this.def;
			return BytecodeDispatcherAlgorithm.this.seLdc;
		}
	}

	private class DispatchStrategy_ACONST_NULL implements BytecodeDispatcher.DispatchStrategy<SEAconst_null> {
		public SEAconst_null doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAconst_null == null) {
				BytecodeDispatcherAlgorithm.this.seAconst_null = new SEAconst_null();
			}
			return BytecodeDispatcherAlgorithm.this.seAconst_null;
		}
	}

	private class DispatchStrategy_XCONST_Y implements BytecodeDispatcher.DispatchStrategy<SEConst> {
		private int type;
		private int val;
		public DispatchStrategy_XCONST_Y(int type, int val) {
			this.type = type;
			this.val = val;
		}
		public SEConst doIt() {
			if (BytecodeDispatcherAlgorithm.this.seConst == null) {
				BytecodeDispatcherAlgorithm.this.seConst = new SEConst();
			}
			BytecodeDispatcherAlgorithm.this.seConst.type = this.type;
			BytecodeDispatcherAlgorithm.this.seConst.val = this.val;
			return BytecodeDispatcherAlgorithm.this.seConst;
		}
	}

	private class DispatchStrategy_XADD implements BytecodeDispatcher.DispatchStrategy<SEAdd> {
		public SEAdd doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAdd == null) {
				BytecodeDispatcherAlgorithm.this.seAdd = new SEAdd();
			}
			return BytecodeDispatcherAlgorithm.this.seAdd;
		}
	}

	private class DispatchStrategy_XSUB implements BytecodeDispatcher.DispatchStrategy<SESub> {
		public SESub doIt() {
			if (BytecodeDispatcherAlgorithm.this.seSub == null) {
				BytecodeDispatcherAlgorithm.this.seSub = new SESub();
			}
			return BytecodeDispatcherAlgorithm.this.seSub;
		}
	}

	private class DispatchStrategy_XMUL implements BytecodeDispatcher.DispatchStrategy<SEMul> {
		public SEMul doIt() {
			if (BytecodeDispatcherAlgorithm.this.seMul == null) {
				BytecodeDispatcherAlgorithm.this.seMul = new SEMul();
			}
			return BytecodeDispatcherAlgorithm.this.seMul;
		}
	}

	private class DispatchStrategy_XDIV implements BytecodeDispatcher.DispatchStrategy<SEDiv> {
		public SEDiv doIt() {
			if (BytecodeDispatcherAlgorithm.this.seDiv == null) {
				BytecodeDispatcherAlgorithm.this.seDiv = new SEDiv();
			}
			return BytecodeDispatcherAlgorithm.this.seDiv;
		}
	}

	private class DispatchStrategy_XREM implements BytecodeDispatcher.DispatchStrategy<SERem> {
		public SERem doIt() {
			if (BytecodeDispatcherAlgorithm.this.seRem == null) {
				BytecodeDispatcherAlgorithm.this.seRem = new SERem();
			}
			return BytecodeDispatcherAlgorithm.this.seRem;
		}
	}

	private class DispatchStrategy_XNEG implements BytecodeDispatcher.DispatchStrategy<SENeg> {
		public SENeg doIt() {
			if (BytecodeDispatcherAlgorithm.this.seNeg == null) {
				BytecodeDispatcherAlgorithm.this.seNeg = new SENeg();
			}
			return BytecodeDispatcherAlgorithm.this.seNeg;
		}
	}

	private class DispatchStrategy_XSHL implements BytecodeDispatcher.DispatchStrategy<SEShl> {
		public SEShl doIt() {
			if (BytecodeDispatcherAlgorithm.this.seShl == null) {
				BytecodeDispatcherAlgorithm.this.seShl = new SEShl();
			}
			return BytecodeDispatcherAlgorithm.this.seShl;
		}
	}

	private class DispatchStrategy_XSHR implements BytecodeDispatcher.DispatchStrategy<SEShr> {
		public SEShr doIt() {
			if (BytecodeDispatcherAlgorithm.this.seShr == null) {
				BytecodeDispatcherAlgorithm.this.seShr = new SEShr();
			}
			return BytecodeDispatcherAlgorithm.this.seShr;
		}
	}

	private class DispatchStrategy_IUSHR implements BytecodeDispatcher.DispatchStrategy<SEIUshr> {
		public SEIUshr doIt() {
			if (BytecodeDispatcherAlgorithm.this.seIUshr == null) {
				BytecodeDispatcherAlgorithm.this.seIUshr = new SEIUshr();
			}
			return BytecodeDispatcherAlgorithm.this.seIUshr;
		}
	}

	private class DispatchStrategy_XOR implements BytecodeDispatcher.DispatchStrategy<SEOr> {
		public SEOr doIt() {
			if (BytecodeDispatcherAlgorithm.this.seOr == null) {
				BytecodeDispatcherAlgorithm.this.seOr = new SEOr();
			}
			return BytecodeDispatcherAlgorithm.this.seOr;
		}
	}

	private class DispatchStrategy_XAND implements BytecodeDispatcher.DispatchStrategy<SEAnd> {
		public SEAnd doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAnd == null) {
				BytecodeDispatcherAlgorithm.this.seAnd = new SEAnd();
			}
			return BytecodeDispatcherAlgorithm.this.seAnd;
		}
	}

	private class DispatchStrategy_XXOR implements BytecodeDispatcher.DispatchStrategy<SEXor> {
		public SEXor doIt() {
			if (BytecodeDispatcherAlgorithm.this.seXor == null) {
				BytecodeDispatcherAlgorithm.this.seXor = new SEXor();
			}
			return BytecodeDispatcherAlgorithm.this.seXor;
		}
	}

	private class DispatchStrategy_IINC implements BytecodeDispatcher.DispatchStrategy<SEIinc> {
		public SEIinc doIt() {
			if (BytecodeDispatcherAlgorithm.this.seIinc == null) {
				BytecodeDispatcherAlgorithm.this.seIinc = new SEIinc();
			}
			return BytecodeDispatcherAlgorithm.this.seIinc;
		}
	}

	private class DispatchStrategy_XCMPY implements BytecodeDispatcher.DispatchStrategy<SENcmp> {
		public SENcmp doIt() {
			if (BytecodeDispatcherAlgorithm.this.seNcmp == null) {
				BytecodeDispatcherAlgorithm.this.seNcmp = new SENcmp();
			}
			return BytecodeDispatcherAlgorithm.this.seNcmp;
		}
	}

	private class DispatchStrategy_X2Y implements BytecodeDispatcher.DispatchStrategy<SEN2n> {
		private char type;
		private char castType;
		public DispatchStrategy_X2Y(char type, char castType) {
			this.type = type;
			this.castType = castType;
		}
		public SEN2n doIt() {
			if (BytecodeDispatcherAlgorithm.this.seN2n == null) {
				BytecodeDispatcherAlgorithm.this.seN2n = new SEN2n();
			}
			BytecodeDispatcherAlgorithm.this.seN2n.type = this.type;
			BytecodeDispatcherAlgorithm.this.seN2n.castType = this.castType;
			return BytecodeDispatcherAlgorithm.this.seN2n;
		}
	}

	private class DispatchStrategy_NEW implements BytecodeDispatcher.DispatchStrategy<SENew> {
		public SENew doIt() {
			if (BytecodeDispatcherAlgorithm.this.seNew == null) {
				BytecodeDispatcherAlgorithm.this.seNew = new SENew();
			}
			return BytecodeDispatcherAlgorithm.this.seNew;
		}
	}

	private class DispatchStrategy_NEWARRAY implements BytecodeDispatcher.DispatchStrategy<SENewarray> {
		public SENewarray doIt() {
			if (BytecodeDispatcherAlgorithm.this.seNewarray == null) {
				BytecodeDispatcherAlgorithm.this.seNewarray = new SENewarray();
			}
			return BytecodeDispatcherAlgorithm.this.seNewarray;
		}
	}

	private class DispatchStrategy_ANEWARRAY implements BytecodeDispatcher.DispatchStrategy<SEAnewarray> {
		public SEAnewarray doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAnewarray == null) {
				BytecodeDispatcherAlgorithm.this.seAnewarray = new SEAnewarray();
			}
			return BytecodeDispatcherAlgorithm.this.seAnewarray;
		}
	}

	private class DispatchStrategy_MULTIANEWARRAY implements BytecodeDispatcher.DispatchStrategy<SEMultianewarray> {
		public SEMultianewarray doIt() {
			if (BytecodeDispatcherAlgorithm.this.seMultianewarray == null) {
				BytecodeDispatcherAlgorithm.this.seMultianewarray = new SEMultianewarray();
			}
			return BytecodeDispatcherAlgorithm.this.seMultianewarray;
		}
	}

	private class DispatchStrategy_GETFIELD implements BytecodeDispatcher.DispatchStrategy<SEGetfield> {
		public SEGetfield doIt() {
			if (BytecodeDispatcherAlgorithm.this.seGetfield == null) {
				BytecodeDispatcherAlgorithm.this.seGetfield = new SEGetfield();
			}
			return BytecodeDispatcherAlgorithm.this.seGetfield;
		}
	}

	private class DispatchStrategy_PUTFIELD implements BytecodeDispatcher.DispatchStrategy<SEPutfield> {
		public SEPutfield doIt() {
			if (BytecodeDispatcherAlgorithm.this.sePutfield == null) {
				BytecodeDispatcherAlgorithm.this.sePutfield = new SEPutfield();
			}
			return BytecodeDispatcherAlgorithm.this.sePutfield;
		}
	}

	private class DispatchStrategy_GETSTATIC implements BytecodeDispatcher.DispatchStrategy<SEGetstatic> {
		public SEGetstatic doIt() {
			if (BytecodeDispatcherAlgorithm.this.seGetstatic == null) {
				BytecodeDispatcherAlgorithm.this.seGetstatic = new SEGetstatic();
			}
			return BytecodeDispatcherAlgorithm.this.seGetstatic;
		}
	}

	private class DispatchStrategy_PUTSTATIC implements BytecodeDispatcher.DispatchStrategy<SEPutstatic> {
		public SEPutstatic doIt() {
			if (BytecodeDispatcherAlgorithm.this.sePutstatic == null) {
				BytecodeDispatcherAlgorithm.this.sePutstatic = new SEPutstatic();
			}
			return BytecodeDispatcherAlgorithm.this.sePutstatic;
		}
	}

	private class DispatchStrategy_XALOAD implements BytecodeDispatcher.DispatchStrategy<SEAload> {
		public SEAload doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAload == null) {
				BytecodeDispatcherAlgorithm.this.seAload = new SEAload();
			}
			return BytecodeDispatcherAlgorithm.this.seAload;
		}
	}

	private class DispatchStrategy_XASTORE implements BytecodeDispatcher.DispatchStrategy<SEAstore> {
		public SEAstore doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAstore == null) {
				BytecodeDispatcherAlgorithm.this.seAstore = new SEAstore();
			}
			return BytecodeDispatcherAlgorithm.this.seAstore;
		}
	}

	private class DispatchStrategy_ARRAYLENGTH implements BytecodeDispatcher.DispatchStrategy<SEArrayLength> {
		public SEArrayLength doIt() {
			if (BytecodeDispatcherAlgorithm.this.seArrayLength == null) {
				BytecodeDispatcherAlgorithm.this.seArrayLength = new SEArrayLength();
			}
			return BytecodeDispatcherAlgorithm.this.seArrayLength;
		}
	}

	private class DispatchStrategy_INSTANCEOF implements BytecodeDispatcher.DispatchStrategy<SEInstanceof> {
		public SEInstanceof doIt() {
			if (BytecodeDispatcherAlgorithm.this.seInstanceof == null) {
				BytecodeDispatcherAlgorithm.this.seInstanceof = new SEInstanceof();
			}
			return BytecodeDispatcherAlgorithm.this.seInstanceof;
		}
	}

	private class DispatchStrategy_CHECKCAST implements BytecodeDispatcher.DispatchStrategy<SECheckcast> {
		public SECheckcast doIt() {
			if (BytecodeDispatcherAlgorithm.this.seCheckcast == null) {
				BytecodeDispatcherAlgorithm.this.seCheckcast = new SECheckcast();
			}
			return BytecodeDispatcherAlgorithm.this.seCheckcast;
		}
	}

	private class DispatchStrategy_POPX implements BytecodeDispatcher.DispatchStrategy<SEPop> {
		private boolean def;
		public DispatchStrategy_POPX(boolean def) {
			this.def = def;
		}
		public SEPop doIt() {
			if (BytecodeDispatcherAlgorithm.this.sePop == null) {
				BytecodeDispatcherAlgorithm.this.sePop = new SEPop();
			}
			BytecodeDispatcherAlgorithm.this.sePop.def = this.def;
			return BytecodeDispatcherAlgorithm.this.sePop;
		}
	}

	private class DispatchStrategy_DUPX implements BytecodeDispatcher.DispatchStrategy<SEDup> {
		private boolean cat_1;
		public DispatchStrategy_DUPX(boolean cat_1) {
			this.cat_1 = cat_1;
		}
		public SEDup doIt() {
			if (BytecodeDispatcherAlgorithm.this.seDup == null) {
				BytecodeDispatcherAlgorithm.this.seDup = new SEDup();
			}
			BytecodeDispatcherAlgorithm.this.seDup.cat_1 = this.cat_1;
			return BytecodeDispatcherAlgorithm.this.seDup;
		}
	}

	private class DispatchStrategy_DUPX_Y implements BytecodeDispatcher.DispatchStrategy<SEDup_n> {
		private boolean cat_1;
		private boolean x_1;
		public DispatchStrategy_DUPX_Y(boolean cat_1, boolean x_1) {
			this.cat_1 = cat_1;
			this.x_1 = x_1;
		}
		public SEDup_n doIt() {
			if (BytecodeDispatcherAlgorithm.this.seDup_n == null) {
				BytecodeDispatcherAlgorithm.this.seDup_n = new SEDup_n();
			}
			BytecodeDispatcherAlgorithm.this.seDup_n.cat_1 = this.cat_1;
			BytecodeDispatcherAlgorithm.this.seDup_n.x_1 = this.x_1;
			return BytecodeDispatcherAlgorithm.this.seDup_n;
		}
	}

	private class DispatchStrategy_SWAP implements BytecodeDispatcher.DispatchStrategy<SESwap> {
		public SESwap doIt() {
			if (BytecodeDispatcherAlgorithm.this.seSwap == null) {
				BytecodeDispatcherAlgorithm.this.seSwap = new SESwap();
			}
			return BytecodeDispatcherAlgorithm.this.seSwap;
		}
	}

	private class DispatchStrategy_IFX implements BytecodeDispatcher.DispatchStrategy<SEIfcond> {
		private boolean def;
		private String type;
		public DispatchStrategy_IFX(boolean def, String type) {
			this.def = def;
			this.type = type;
		}
		public SEIfcond doIt() {
			if (BytecodeDispatcherAlgorithm.this.seIfcond == null) {
				BytecodeDispatcherAlgorithm.this.seIfcond = new SEIfcond();
			}
			BytecodeDispatcherAlgorithm.this.seIfcond.def = this.def;
			BytecodeDispatcherAlgorithm.this.seIfcond.type = this.type;
			return BytecodeDispatcherAlgorithm.this.seIfcond;
		}
	}

	private class DispatchStrategy_IFXNULL implements BytecodeDispatcher.DispatchStrategy<SEIfnull> {
		private boolean def;
		public DispatchStrategy_IFXNULL(boolean def) {
			this.def = def;
		}
		public SEIfnull doIt() {
			if (BytecodeDispatcherAlgorithm.this.seIfnull == null) {
				BytecodeDispatcherAlgorithm.this.seIfnull = new SEIfnull();
			}
			BytecodeDispatcherAlgorithm.this.seIfnull.def = this.def;
			return BytecodeDispatcherAlgorithm.this.seIfnull;
		}
	}

	private class DispatchStrategy_IF_ACMPX implements BytecodeDispatcher.DispatchStrategy<SEIfacmp> {
		private boolean eq;
		public DispatchStrategy_IF_ACMPX(boolean eq) {
			this.eq = eq;
		}
		public SEIfacmp doIt() {
			if (BytecodeDispatcherAlgorithm.this.seIfacmp == null) {
				BytecodeDispatcherAlgorithm.this.seIfacmp = new SEIfacmp();
			}
			BytecodeDispatcherAlgorithm.this.seIfacmp.eq = this.eq;
			return BytecodeDispatcherAlgorithm.this.seIfacmp;
		}
	}

	private class DispatchStrategy_XSWITCH implements BytecodeDispatcher.DispatchStrategy<SESwitch> {
		private boolean isTableSwitch;
		public DispatchStrategy_XSWITCH(boolean isTableSwitch) {
			this.isTableSwitch = isTableSwitch;
		}
		public SESwitch doIt() {
			if (BytecodeDispatcherAlgorithm.this.seSwitch == null) {
				BytecodeDispatcherAlgorithm.this.seSwitch = new SESwitch();
			}
			BytecodeDispatcherAlgorithm.this.seSwitch.isTableSwitch = this.isTableSwitch;
			return BytecodeDispatcherAlgorithm.this.seSwitch;
		}
	}

	private class DispatchStrategy_GOTOX implements BytecodeDispatcher.DispatchStrategy<SEGoto> {
		private boolean def;
		public DispatchStrategy_GOTOX(boolean def) {
			this.def = def;
		}
		public SEGoto doIt() {
			if (BytecodeDispatcherAlgorithm.this.seGoto == null) {
				BytecodeDispatcherAlgorithm.this.seGoto = new SEGoto();
			}
			BytecodeDispatcherAlgorithm.this.seGoto.def = this.def;
			return BytecodeDispatcherAlgorithm.this.seGoto;
		}
	}

	private class DispatchStrategy_JSRX implements BytecodeDispatcher.DispatchStrategy<SEJsr> {
		private boolean def;
		public DispatchStrategy_JSRX(boolean def) {
			this.def = def;
		}
		public SEJsr doIt() {
			if (BytecodeDispatcherAlgorithm.this.seJsr == null) {
				BytecodeDispatcherAlgorithm.this.seJsr = new SEJsr();
			}
			BytecodeDispatcherAlgorithm.this.seJsr.def = this.def;
			return BytecodeDispatcherAlgorithm.this.seJsr;
		}
	}

	private class DispatchStrategy_RET implements BytecodeDispatcher.DispatchStrategy<SERet> {
		public SERet doIt() {
			if (BytecodeDispatcherAlgorithm.this.seRet == null) {
				BytecodeDispatcherAlgorithm.this.seRet = new SERet();
			}
			return BytecodeDispatcherAlgorithm.this.seRet;
		}
	}

	private class DispatchStrategy_INVOKEVIRTUALINTERFACE implements BytecodeDispatcher.DispatchStrategy<SEInvokeVirtualInterface> {
		private boolean isInterface;
		public DispatchStrategy_INVOKEVIRTUALINTERFACE(boolean isInterface) {
			this.isInterface = isInterface;
		}
		public SEInvokeVirtualInterface doIt() {
			if (BytecodeDispatcherAlgorithm.this.seInvokeVirtualInterface == null) {
				BytecodeDispatcherAlgorithm.this.seInvokeVirtualInterface = new SEInvokeVirtualInterface();
			}
			BytecodeDispatcherAlgorithm.this.seInvokeVirtualInterface.isInterface = this.isInterface;
			return BytecodeDispatcherAlgorithm.this.seInvokeVirtualInterface;
		}
	}

	private class DispatchStrategy_INVOKESPECIAL implements BytecodeDispatcher.DispatchStrategy<SEInvokeSpecial> {
		public SEInvokeSpecial doIt() {
			if (BytecodeDispatcherAlgorithm.this.seInvokeSpecial == null) {
				BytecodeDispatcherAlgorithm.this.seInvokeSpecial = new SEInvokeSpecial();
			}
			return BytecodeDispatcherAlgorithm.this.seInvokeSpecial;
		}
	}

	private class DispatchStrategy_INVOKESTATIC implements BytecodeDispatcher.DispatchStrategy<SEInvokeStatic> {
		public SEInvokeStatic doIt() {
			if (BytecodeDispatcherAlgorithm.this.seInvokeStatic == null) {
				BytecodeDispatcherAlgorithm.this.seInvokeStatic = new SEInvokeStatic();
			}
			return BytecodeDispatcherAlgorithm.this.seInvokeStatic;
		}
	}

	private class DispatchStrategy_XRETURN implements BytecodeDispatcher.DispatchStrategy<SEReturn> {
		private boolean def;
		public DispatchStrategy_XRETURN(boolean def) {
			this.def = def;
		}
		public SEReturn doIt() {
			if (BytecodeDispatcherAlgorithm.this.seReturn == null) {
				BytecodeDispatcherAlgorithm.this.seReturn = new SEReturn();
			}
			BytecodeDispatcherAlgorithm.this.seReturn.def = this.def;
			return BytecodeDispatcherAlgorithm.this.seReturn;
		}
	}

	private class DispatchStrategy_ATHROW implements BytecodeDispatcher.DispatchStrategy<SEAthrow> {
		public SEAthrow doIt() {
			if (BytecodeDispatcherAlgorithm.this.seAthrow == null) {
				BytecodeDispatcherAlgorithm.this.seAthrow = new SEAthrow();
			}
			return BytecodeDispatcherAlgorithm.this.seAthrow;
		}
	}

	private class DispatchStrategy_WIDE implements BytecodeDispatcher.DispatchStrategy<SEWide> {
		public SEWide doIt() {
			if (BytecodeDispatcherAlgorithm.this.seWide == null) {
				BytecodeDispatcherAlgorithm.this.seWide = new SEWide();
			}
			return BytecodeDispatcherAlgorithm.this.seWide;
		}
	}

	private class DispatchStrategy_MONITORX implements BytecodeDispatcher.DispatchStrategy<SEMonitor> {
		public SEMonitor doIt() {
			if (BytecodeDispatcherAlgorithm.this.seMonitor == null) {
				BytecodeDispatcherAlgorithm.this.seMonitor = new SEMonitor();
			}
			return BytecodeDispatcherAlgorithm.this.seMonitor;
		}
	}

	private class DispatchStrategy_UNEXPECTED implements BytecodeDispatcher.DispatchStrategy<SEUnexpected> {
		public SEUnexpected doIt() {
			if (BytecodeDispatcherAlgorithm.this.seUnexpected == null) {
				BytecodeDispatcherAlgorithm.this.seUnexpected = new SEUnexpected();
			}
			return BytecodeDispatcherAlgorithm.this.seUnexpected;
		}
	}

	public BytecodeDispatcherAlgorithm() {
        this
		.setDispatchNonexistentStrategy(new DispatchStrategy_UNDEFINED())
		.setDispatchStrategy(OP_NOP,             new DispatchStrategy_NOP())
		.setDispatchStrategy(OP_ALOAD,           new DispatchStrategy_XLOAD())
		.setDispatchStrategy(OP_DLOAD,           new DispatchStrategy_XLOAD())
		.setDispatchStrategy(OP_FLOAD,           new DispatchStrategy_XLOAD())
		.setDispatchStrategy(OP_ILOAD,           new DispatchStrategy_XLOAD())
		.setDispatchStrategy(OP_LLOAD,           new DispatchStrategy_XLOAD())
		.setDispatchStrategy(OP_ALOAD_0,         new DispatchStrategy_XLOAD(0))
		.setDispatchStrategy(OP_DLOAD_0,         new DispatchStrategy_XLOAD(0))
		.setDispatchStrategy(OP_FLOAD_0,         new DispatchStrategy_XLOAD(0))
		.setDispatchStrategy(OP_ILOAD_0,         new DispatchStrategy_XLOAD(0))
		.setDispatchStrategy(OP_LLOAD_0,         new DispatchStrategy_XLOAD(0))
		.setDispatchStrategy(OP_ALOAD_1,         new DispatchStrategy_XLOAD(1))
		.setDispatchStrategy(OP_DLOAD_1,         new DispatchStrategy_XLOAD(1))
		.setDispatchStrategy(OP_FLOAD_1,         new DispatchStrategy_XLOAD(1))
		.setDispatchStrategy(OP_ILOAD_1,         new DispatchStrategy_XLOAD(1))
		.setDispatchStrategy(OP_LLOAD_1,         new DispatchStrategy_XLOAD(1))
		.setDispatchStrategy(OP_ALOAD_2,         new DispatchStrategy_XLOAD(2))
		.setDispatchStrategy(OP_DLOAD_2,         new DispatchStrategy_XLOAD(2))
		.setDispatchStrategy(OP_FLOAD_2,         new DispatchStrategy_XLOAD(2))
		.setDispatchStrategy(OP_ILOAD_2,         new DispatchStrategy_XLOAD(2))
		.setDispatchStrategy(OP_LLOAD_2,         new DispatchStrategy_XLOAD(2))
		.setDispatchStrategy(OP_ALOAD_3,         new DispatchStrategy_XLOAD(3))
		.setDispatchStrategy(OP_DLOAD_3,         new DispatchStrategy_XLOAD(3))
		.setDispatchStrategy(OP_FLOAD_3,         new DispatchStrategy_XLOAD(3))
		.setDispatchStrategy(OP_ILOAD_3,         new DispatchStrategy_XLOAD(3))
		.setDispatchStrategy(OP_LLOAD_3,         new DispatchStrategy_XLOAD(3))
		.setDispatchStrategy(OP_ASTORE,          new DispatchStrategy_XSTORE())
		.setDispatchStrategy(OP_DSTORE,          new DispatchStrategy_XSTORE())
		.setDispatchStrategy(OP_FSTORE,          new DispatchStrategy_XSTORE())
		.setDispatchStrategy(OP_ISTORE,          new DispatchStrategy_XSTORE())
		.setDispatchStrategy(OP_LSTORE,          new DispatchStrategy_XSTORE())
		.setDispatchStrategy(OP_ASTORE_0,        new DispatchStrategy_XSTORE(0))
		.setDispatchStrategy(OP_DSTORE_0,        new DispatchStrategy_XSTORE(0))
		.setDispatchStrategy(OP_FSTORE_0,        new DispatchStrategy_XSTORE(0))
		.setDispatchStrategy(OP_ISTORE_0,        new DispatchStrategy_XSTORE(0))
		.setDispatchStrategy(OP_LSTORE_0,        new DispatchStrategy_XSTORE(0))
		.setDispatchStrategy(OP_ASTORE_1,        new DispatchStrategy_XSTORE(1))
		.setDispatchStrategy(OP_DSTORE_1,        new DispatchStrategy_XSTORE(1))
		.setDispatchStrategy(OP_FSTORE_1,        new DispatchStrategy_XSTORE(1))
		.setDispatchStrategy(OP_ISTORE_1,        new DispatchStrategy_XSTORE(1))
		.setDispatchStrategy(OP_LSTORE_1,        new DispatchStrategy_XSTORE(1))
		.setDispatchStrategy(OP_ASTORE_2,        new DispatchStrategy_XSTORE(2))
		.setDispatchStrategy(OP_DSTORE_2,        new DispatchStrategy_XSTORE(2))
		.setDispatchStrategy(OP_FSTORE_2,        new DispatchStrategy_XSTORE(2))
		.setDispatchStrategy(OP_ISTORE_2,        new DispatchStrategy_XSTORE(2))
		.setDispatchStrategy(OP_LSTORE_2,        new DispatchStrategy_XSTORE(2))
		.setDispatchStrategy(OP_ASTORE_3,        new DispatchStrategy_XSTORE(3))
		.setDispatchStrategy(OP_DSTORE_3,        new DispatchStrategy_XSTORE(3))
		.setDispatchStrategy(OP_FSTORE_3,        new DispatchStrategy_XSTORE(3))
		.setDispatchStrategy(OP_ISTORE_3,        new DispatchStrategy_XSTORE(3))
		.setDispatchStrategy(OP_LSTORE_3,        new DispatchStrategy_XSTORE(3))
		.setDispatchStrategy(OP_BIPUSH,          new DispatchStrategy_BIPUSH())
		.setDispatchStrategy(OP_SIPUSH,          new DispatchStrategy_SIPUSH())
		.setDispatchStrategy(OP_LDC,             new DispatchStrategy_LDCX_Y(true))
		.setDispatchStrategy(OP_LDC_W,           new DispatchStrategy_LDCX_Y(false))
		.setDispatchStrategy(OP_LDC2_W,          new DispatchStrategy_LDCX_Y(false))
		.setDispatchStrategy(OP_ACONST_NULL,     new DispatchStrategy_ACONST_NULL())
		.setDispatchStrategy(OP_DCONST_0,        new DispatchStrategy_XCONST_Y(Type.DOUBLE, 0))
		.setDispatchStrategy(OP_DCONST_1,        new DispatchStrategy_XCONST_Y(Type.DOUBLE, 1))
		.setDispatchStrategy(OP_FCONST_0,        new DispatchStrategy_XCONST_Y(Type.FLOAT, 0))
		.setDispatchStrategy(OP_FCONST_1,        new DispatchStrategy_XCONST_Y(Type.FLOAT, 1))
		.setDispatchStrategy(OP_FCONST_2,        new DispatchStrategy_XCONST_Y(Type.FLOAT, 2))
		.setDispatchStrategy(OP_ICONST_M1,       new DispatchStrategy_XCONST_Y(Type.INT, -1))
		.setDispatchStrategy(OP_ICONST_0,        new DispatchStrategy_XCONST_Y(Type.INT, 0))
		.setDispatchStrategy(OP_ICONST_1,        new DispatchStrategy_XCONST_Y(Type.INT, 1))
		.setDispatchStrategy(OP_ICONST_2,        new DispatchStrategy_XCONST_Y(Type.INT, 2))
		.setDispatchStrategy(OP_ICONST_3,        new DispatchStrategy_XCONST_Y(Type.INT, 3))
		.setDispatchStrategy(OP_ICONST_4,        new DispatchStrategy_XCONST_Y(Type.INT, 4))
		.setDispatchStrategy(OP_ICONST_5,        new DispatchStrategy_XCONST_Y(Type.INT, 5))
		.setDispatchStrategy(OP_LCONST_0,        new DispatchStrategy_XCONST_Y(Type.LONG, 0))
		.setDispatchStrategy(OP_LCONST_1,        new DispatchStrategy_XCONST_Y(Type.LONG, 1))
		.setDispatchStrategy(OP_DADD,            new DispatchStrategy_XADD())
		.setDispatchStrategy(OP_FADD,            new DispatchStrategy_XADD())
		.setDispatchStrategy(OP_IADD,            new DispatchStrategy_XADD())
		.setDispatchStrategy(OP_LADD,            new DispatchStrategy_XADD())
		.setDispatchStrategy(OP_DSUB,            new DispatchStrategy_XSUB())
		.setDispatchStrategy(OP_FSUB,            new DispatchStrategy_XSUB())
		.setDispatchStrategy(OP_ISUB,            new DispatchStrategy_XSUB())
		.setDispatchStrategy(OP_LSUB,            new DispatchStrategy_XSUB())
		.setDispatchStrategy(OP_DMUL,            new DispatchStrategy_XMUL())
		.setDispatchStrategy(OP_FMUL,            new DispatchStrategy_XMUL())
		.setDispatchStrategy(OP_IMUL,            new DispatchStrategy_XMUL())
		.setDispatchStrategy(OP_LMUL,            new DispatchStrategy_XMUL())
		.setDispatchStrategy(OP_DDIV,            new DispatchStrategy_XDIV())
		.setDispatchStrategy(OP_FDIV,            new DispatchStrategy_XDIV())
		.setDispatchStrategy(OP_IDIV,            new DispatchStrategy_XDIV())
		.setDispatchStrategy(OP_LDIV,            new DispatchStrategy_XDIV())
		.setDispatchStrategy(OP_DREM,            new DispatchStrategy_XREM())
		.setDispatchStrategy(OP_FREM,            new DispatchStrategy_XREM())
		.setDispatchStrategy(OP_IREM,            new DispatchStrategy_XREM())
		.setDispatchStrategy(OP_LREM,            new DispatchStrategy_XREM())
		.setDispatchStrategy(OP_DNEG,            new DispatchStrategy_XNEG())
		.setDispatchStrategy(OP_FNEG,            new DispatchStrategy_XNEG())
		.setDispatchStrategy(OP_INEG,            new DispatchStrategy_XNEG())
		.setDispatchStrategy(OP_LNEG,            new DispatchStrategy_XNEG())
		.setDispatchStrategy(OP_ISHL,            new DispatchStrategy_XSHL())
		.setDispatchStrategy(OP_LSHL,            new DispatchStrategy_XSHL())
		.setDispatchStrategy(OP_ISHR,            new DispatchStrategy_XSHR())
		.setDispatchStrategy(OP_LSHR,            new DispatchStrategy_XSHR())
		.setDispatchStrategy(OP_IUSHR,           new DispatchStrategy_IUSHR())
		//TODO CURRENTLY UNDEFINED LUSHR, should be: .setDispatchStrategy(OP_LUSHR,           new DispatchStrategy_LUSHR())  
		.setDispatchStrategy(OP_IOR,             new DispatchStrategy_XOR())
		.setDispatchStrategy(OP_LOR,             new DispatchStrategy_XOR())
		.setDispatchStrategy(OP_IAND,            new DispatchStrategy_XAND())
		.setDispatchStrategy(OP_LAND,            new DispatchStrategy_XAND())
		.setDispatchStrategy(OP_IXOR,            new DispatchStrategy_XXOR())
		.setDispatchStrategy(OP_LXOR,            new DispatchStrategy_XXOR())
		.setDispatchStrategy(OP_IINC,            new DispatchStrategy_IINC())
		.setDispatchStrategy(OP_DCMPG,           new DispatchStrategy_XCMPY())
		.setDispatchStrategy(OP_DCMPL,           new DispatchStrategy_XCMPY())
		.setDispatchStrategy(OP_FCMPG,           new DispatchStrategy_XCMPY())
		.setDispatchStrategy(OP_FCMPL,           new DispatchStrategy_XCMPY())
		.setDispatchStrategy(OP_LCMP,            new DispatchStrategy_XCMPY())
		.setDispatchStrategy(OP_I2D,             new DispatchStrategy_X2Y(Type.INT, Type.DOUBLE))
		.setDispatchStrategy(OP_I2F,             new DispatchStrategy_X2Y(Type.INT, Type.FLOAT))
		.setDispatchStrategy(OP_I2L,             new DispatchStrategy_X2Y(Type.INT, Type.LONG))
		.setDispatchStrategy(OP_L2D,             new DispatchStrategy_X2Y(Type.LONG, Type.DOUBLE))
		.setDispatchStrategy(OP_L2F,             new DispatchStrategy_X2Y(Type.LONG, Type.FLOAT))
		.setDispatchStrategy(OP_F2D,             new DispatchStrategy_X2Y(Type.FLOAT, Type.DOUBLE))
		.setDispatchStrategy(OP_D2F,             new DispatchStrategy_X2Y(Type.DOUBLE, Type.FLOAT))
		.setDispatchStrategy(OP_D2I,             new DispatchStrategy_X2Y(Type.DOUBLE, Type.INT))
		.setDispatchStrategy(OP_D2L,             new DispatchStrategy_X2Y(Type.DOUBLE, Type.LONG))
		.setDispatchStrategy(OP_F2I,             new DispatchStrategy_X2Y(Type.FLOAT, Type.INT))
		.setDispatchStrategy(OP_F2L,             new DispatchStrategy_X2Y(Type.FLOAT, Type.LONG))
		.setDispatchStrategy(OP_I2B,             new DispatchStrategy_X2Y(Type.INT, Type.BYTE))
		.setDispatchStrategy(OP_I2C,             new DispatchStrategy_X2Y(Type.INT, Type.CHAR))
		.setDispatchStrategy(OP_I2S,             new DispatchStrategy_X2Y(Type.INT, Type.SHORT))
		.setDispatchStrategy(OP_L2I,             new DispatchStrategy_X2Y(Type.LONG, Type.INT))
		.setDispatchStrategy(OP_NEW,             new DispatchStrategy_NEW())
		.setDispatchStrategy(OP_NEWARRAY,        new DispatchStrategy_NEWARRAY())
		.setDispatchStrategy(OP_ANEWARRAY,       new DispatchStrategy_ANEWARRAY())
		.setDispatchStrategy(OP_MULTIANEWARRAY,  new DispatchStrategy_MULTIANEWARRAY())
		.setDispatchStrategy(OP_GETFIELD,        new DispatchStrategy_GETFIELD())
		.setDispatchStrategy(OP_PUTFIELD,        new DispatchStrategy_PUTFIELD())
		.setDispatchStrategy(OP_GETSTATIC,       new DispatchStrategy_GETSTATIC())
		.setDispatchStrategy(OP_PUTSTATIC,       new DispatchStrategy_PUTSTATIC())
		.setDispatchStrategy(OP_AALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_CALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_BALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_DALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_FALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_IALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_LALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_SALOAD,          new DispatchStrategy_XALOAD())
		.setDispatchStrategy(OP_AASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_CASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_BASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_DASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_FASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_IASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_LASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_SASTORE,         new DispatchStrategy_XASTORE())
		.setDispatchStrategy(OP_ARRAYLENGTH,     new DispatchStrategy_ARRAYLENGTH())
		.setDispatchStrategy(OP_INSTANCEOF,      new DispatchStrategy_INSTANCEOF())
		.setDispatchStrategy(OP_CHECKCAST,       new DispatchStrategy_CHECKCAST())
		.setDispatchStrategy(OP_POP,             new DispatchStrategy_POPX(true))
		.setDispatchStrategy(OP_POP2,            new DispatchStrategy_POPX(false))
		.setDispatchStrategy(OP_DUP,             new DispatchStrategy_DUPX(true))
		.setDispatchStrategy(OP_DUP2,            new DispatchStrategy_DUPX(false))
		.setDispatchStrategy(OP_DUP_X1,          new DispatchStrategy_DUPX_Y(true, true))
		.setDispatchStrategy(OP_DUP2_X1,         new DispatchStrategy_DUPX_Y(false, true))
		.setDispatchStrategy(OP_DUP_X2,          new DispatchStrategy_DUPX_Y(true, false))
		.setDispatchStrategy(OP_DUP2_X2,         new DispatchStrategy_DUPX_Y(false, false))
		.setDispatchStrategy(OP_SWAP,            new DispatchStrategy_SWAP())
		.setDispatchStrategy(OP_IFEQ,            new DispatchStrategy_IFX(true, Operator.EQ))
		.setDispatchStrategy(OP_IFGE,            new DispatchStrategy_IFX(true, Operator.GE))
		.setDispatchStrategy(OP_IFGT,            new DispatchStrategy_IFX(true, Operator.GT))
		.setDispatchStrategy(OP_IFLE,            new DispatchStrategy_IFX(true, Operator.LE))
		.setDispatchStrategy(OP_IFLT,            new DispatchStrategy_IFX(true, Operator.LT))
		.setDispatchStrategy(OP_IFNE,            new DispatchStrategy_IFX(true, Operator.NE))
		.setDispatchStrategy(OP_IFNONNULL,       new DispatchStrategy_IFXNULL(false))
		.setDispatchStrategy(OP_IFNULL,          new DispatchStrategy_IFXNULL(true))
		.setDispatchStrategy(OP_IF_ICMPEQ,       new DispatchStrategy_IFX(false, Operator.EQ))
		.setDispatchStrategy(OP_IF_ICMPGE,       new DispatchStrategy_IFX(false, Operator.GE))
		.setDispatchStrategy(OP_IF_ICMPGT,       new DispatchStrategy_IFX(false, Operator.GT))
		.setDispatchStrategy(OP_IF_ICMPLE,       new DispatchStrategy_IFX(false, Operator.LE))
		.setDispatchStrategy(OP_IF_ICMPLT,       new DispatchStrategy_IFX(false, Operator.LT))
		.setDispatchStrategy(OP_IF_ICMPNE,       new DispatchStrategy_IFX(false, Operator.NE))
		.setDispatchStrategy(OP_IF_ACMPEQ,       new DispatchStrategy_IF_ACMPX(true))
		.setDispatchStrategy(OP_IF_ACMPNE,       new DispatchStrategy_IF_ACMPX(false))
		.setDispatchStrategy(OP_TABLESWITCH,     new DispatchStrategy_XSWITCH(true))
		.setDispatchStrategy(OP_LOOKUPSWITCH,    new DispatchStrategy_XSWITCH(false))
		.setDispatchStrategy(OP_GOTO,            new DispatchStrategy_GOTOX(true))
		.setDispatchStrategy(OP_GOTO_W,          new DispatchStrategy_GOTOX(false))
		.setDispatchStrategy(OP_JSR,             new DispatchStrategy_JSRX(true))
		.setDispatchStrategy(OP_JSR_W,           new DispatchStrategy_JSRX(false))
		.setDispatchStrategy(OP_RET,             new DispatchStrategy_RET())
		.setDispatchStrategy(OP_INVOKEINTERFACE, new DispatchStrategy_INVOKEVIRTUALINTERFACE(true))
		.setDispatchStrategy(OP_INVOKEVIRTUAL,   new DispatchStrategy_INVOKEVIRTUALINTERFACE(false))
		.setDispatchStrategy(OP_INVOKESPECIAL,   new DispatchStrategy_INVOKESPECIAL())
		.setDispatchStrategy(OP_INVOKESTATIC,    new DispatchStrategy_INVOKESTATIC())
		.setDispatchStrategy(OP_RETURN,          new DispatchStrategy_XRETURN(true))
		.setDispatchStrategy(OP_ARETURN,         new DispatchStrategy_XRETURN(false))
		.setDispatchStrategy(OP_DRETURN,         new DispatchStrategy_XRETURN(false))
		.setDispatchStrategy(OP_FRETURN,         new DispatchStrategy_XRETURN(false))
		.setDispatchStrategy(OP_IRETURN,         new DispatchStrategy_XRETURN(false))
		.setDispatchStrategy(OP_LRETURN,         new DispatchStrategy_XRETURN(false))
		.setDispatchStrategy(OP_ATHROW,          new DispatchStrategy_ATHROW())
		.setDispatchStrategy(OP_WIDE,            new DispatchStrategy_WIDE())
		.setDispatchStrategy(OP_MONITORENTER,    new DispatchStrategy_MONITORX())
		.setDispatchStrategy(OP_MONITOREXIT,     new DispatchStrategy_MONITORX())
		.setDispatchStrategy(OP_BREAKPOINT,      new DispatchStrategy_NOP())
		.setDispatchStrategy(OP_IMPDEP1,         new DispatchStrategy_NOP())
		.setDispatchStrategy(OP_IMPDEP2,         new DispatchStrategy_NOP())
	;
	
    DispatchStrategy_UNEXPECTED s = new DispatchStrategy_UNEXPECTED();
	this
		.setDispatchStrategy(OP_UNDEF,                     s)
		.setDispatchStrategy(OP_ANEWARRAY_QUICK,           s)
		.setDispatchStrategy(OP_CHECKCAST_QUICK,           s)
		.setDispatchStrategy(OP_GETSTATIC_QUICK,           s)
		.setDispatchStrategy(OP_GETSTATIC2_QUICK,          s)
		.setDispatchStrategy(OP_GETFIELD_QUICK,            s)
		.setDispatchStrategy(OP_GETFIELD_QUICK_W,          s)
		.setDispatchStrategy(OP_GETFIELD2_QUICK,           s)
		.setDispatchStrategy(OP_INSTANCEOF_QUICK,          s)
		.setDispatchStrategy(OP_INVOKEINTERFACE_QUICK,     s)
		.setDispatchStrategy(OP_INVOKENONVIRTUAL_QUICK,    s)
		.setDispatchStrategy(OP_INVOKESTATIC_QUICK,        s)
		.setDispatchStrategy(OP_INVOKESUPER_QUICK,         s)
		.setDispatchStrategy(OP_INVOKEVIRTUAL_QUICK,       s)
		.setDispatchStrategy(OP_INVOKEVIRTUAL_QUICK_W,     s)
		.setDispatchStrategy(OP_INVOKEVIRTUALOBJECT_QUICK, s)
		.setDispatchStrategy(OP_LDC_QUICK,                 s)
		.setDispatchStrategy(OP_LDC_W_QUICK,               s)
		.setDispatchStrategy(OP_LDC2_W_QUICK,              s)
		.setDispatchStrategy(OP_MULTIANEWARRAY_QUICK,      s)
		.setDispatchStrategy(OP_NEW_QUICK,                 s)
		.setDispatchStrategy(OP_PUTFIELD_QUICK,            s)
		.setDispatchStrategy(OP_PUTFIELD_QUICK_W,          s)
		.setDispatchStrategy(OP_PUTFIELD2_QUICK,           s)
		.setDispatchStrategy(OP_PUTSTATIC_QUICK,           s)
		.setDispatchStrategy(OP_PUTSTATIC2_QUICK,          s)
        ;
    }
	
    public SEInit select() {
		if (this.seInit == null) {
			this.seInit = new SEInit();
		}
		return this.seInit;
    }
		
	@Override
	public Algorithm select(byte bytecode) throws UndefInstructionException {
		Algorithm retVal = null;
		try {
			retVal = super.select(bytecode);
		} catch (UndefInstructionException e) {
			throw e;
		} catch (Throwable e) {
			//this should never happen
		}
		return retVal;
	}

}
