package muse.algo;

import muse.bc.Opcodes;
import muse.mem.Operator;
import muse.mem.Type;

/**
 * An object implementing a dispatcher on instruction bytecode.
 * 
 * @author pietro
 */
public class BytecodeDispatcherAlgorithm_OLD {
    public SEInit select() {
        return(new SEInit());
    }
    
    public Algorithm select(byte instruction) throws UndefInstructionException {
    	switch (instruction) {
        //Note: a = address (reference), b = byte, c = char, d = double, 
        //f = float, i = integer, l = long, s = short
        
        case Opcodes.OP_NOP:                    
            return(new SENop());

        //A - LOAD AND STORE INSTRUCTIONS
        //A1 - load a local variable to operand stack
        case Opcodes.OP_ALOAD:
        case Opcodes.OP_DLOAD:
        case Opcodes.OP_FLOAD:
        case Opcodes.OP_ILOAD:
        case Opcodes.OP_LLOAD:
            return(new SELoad());
        case Opcodes.OP_ALOAD_0:
        case Opcodes.OP_DLOAD_0:
        case Opcodes.OP_FLOAD_0:
        case Opcodes.OP_ILOAD_0:
        case Opcodes.OP_LLOAD_0:
            return(new SELoad(0));
        case Opcodes.OP_ALOAD_1:
        case Opcodes.OP_DLOAD_1:
        case Opcodes.OP_FLOAD_1:
        case Opcodes.OP_ILOAD_1:
        case Opcodes.OP_LLOAD_1:
            return(new SELoad(1));
        case Opcodes.OP_ALOAD_2:
        case Opcodes.OP_DLOAD_2:
        case Opcodes.OP_FLOAD_2:
        case Opcodes.OP_ILOAD_2:
        case Opcodes.OP_LLOAD_2:
            return(new SELoad(2));
        case Opcodes.OP_ALOAD_3:
        case Opcodes.OP_DLOAD_3:
        case Opcodes.OP_FLOAD_3:
        case Opcodes.OP_ILOAD_3:
        case Opcodes.OP_LLOAD_3:
            return(new SELoad(3));
            
        //A2 - store a value from the operand stack into a local variable
        case Opcodes.OP_ASTORE:
        case Opcodes.OP_DSTORE:
        case Opcodes.OP_FSTORE:
        case Opcodes.OP_ISTORE:
        case Opcodes.OP_LSTORE:
            return(new SEStore());
        case Opcodes.OP_ASTORE_0:
        case Opcodes.OP_DSTORE_0:
        case Opcodes.OP_FSTORE_0:
        case Opcodes.OP_ISTORE_0:
        case Opcodes.OP_LSTORE_0:
            return(new SEStore(0));
        case Opcodes.OP_ASTORE_1:
        case Opcodes.OP_DSTORE_1:
        case Opcodes.OP_FSTORE_1:
        case Opcodes.OP_ISTORE_1:
        case Opcodes.OP_LSTORE_1:
            return(new SEStore(1));
        case Opcodes.OP_ASTORE_2:
        case Opcodes.OP_DSTORE_2:
        case Opcodes.OP_FSTORE_2:
        case Opcodes.OP_ISTORE_2:
        case Opcodes.OP_LSTORE_2:
            return(new SEStore(2));
        case Opcodes.OP_ASTORE_3:
        case Opcodes.OP_DSTORE_3:
        case Opcodes.OP_FSTORE_3:
        case Opcodes.OP_ISTORE_3:
        case Opcodes.OP_LSTORE_3:
            return(new SEStore(3));
        
        //A3 - load a constant onto the operand stack
        //push byte
        case Opcodes.OP_BIPUSH:
            return(new SEBipush());
        //push short
        case Opcodes.OP_SIPUSH:
            return(new SESipush());
        //push item from constant pool
        case Opcodes.OP_LDC:
            return(new SELdc(true));
        case Opcodes.OP_LDC_W:                             
        case Opcodes.OP_LDC2_W:
            return(new SELdc(false));
        //push null
        case Opcodes.OP_ACONST_NULL:                
            return(new SEAconst_null());
        //push double constant 0, 1
        case Opcodes.OP_DCONST_0:
            return(new SEConst(Type.DOUBLE, 0));
        case Opcodes.OP_DCONST_1:
            return(new SEConst(Type.DOUBLE, 1));
        //push float constant 0, 1, 2
        case Opcodes.OP_FCONST_0:                           
            return(new SEConst(Type.FLOAT, 0));
        case Opcodes.OP_FCONST_1:                           
            return(new SEConst(Type.FLOAT, 1));
        case Opcodes.OP_FCONST_2:                           
            return(new SEConst(Type.FLOAT, 2));
        //push integer constant -1 .. 5
        case Opcodes.OP_ICONST_M1:                          
            return(new SEConst(Type.INT, -1));
        case Opcodes.OP_ICONST_0:                           
            return(new SEConst(Type.INT, 0));
        case Opcodes.OP_ICONST_1:                           
            return(new SEConst(Type.INT, 1));
        case Opcodes.OP_ICONST_2:                           
            return(new SEConst(Type.INT, 2));
        case Opcodes.OP_ICONST_3:                           
            return(new SEConst(Type.INT, 3));
        case Opcodes.OP_ICONST_4:                           
            return(new SEConst(Type.INT, 4));
        case Opcodes.OP_ICONST_5:                           
            return(new SEConst(Type.INT, 5));
        //push long constant 0, 1
        case Opcodes.OP_LCONST_0:                           
            return(new SEConst(Type.LONG, 0));
        case Opcodes.OP_LCONST_1:                           
            return(new SEConst(Type.LONG, 1));

        //B - ARITHMETIC INSTRUCTIONS
        case Opcodes.OP_DADD:
        case Opcodes.OP_FADD:
        case Opcodes.OP_IADD:
        case Opcodes.OP_LADD:                   
            return(new SEAdd());
        case Opcodes.OP_DSUB:
        case Opcodes.OP_FSUB:
        case Opcodes.OP_ISUB:
        case Opcodes.OP_LSUB:                   
            return(new SESub());
        case Opcodes.OP_DMUL:
        case Opcodes.OP_FMUL:
        case Opcodes.OP_IMUL:
        case Opcodes.OP_LMUL:                   
            return(new SEMul());
        case Opcodes.OP_DDIV:
        case Opcodes.OP_FDIV:
        case Opcodes.OP_IDIV:
        case Opcodes.OP_LDIV:                   
            return(new SEDiv());
        case Opcodes.OP_DREM:
        case Opcodes.OP_FREM:
        case Opcodes.OP_IREM:
        case Opcodes.OP_LREM:                               
            return(new SERem());
        case Opcodes.OP_DNEG:
        case Opcodes.OP_FNEG:
        case Opcodes.OP_INEG:
        case Opcodes.OP_LNEG:                               
            return(new SENeg());
        //TODO Non sono sicurissimo che siano le operazioni di shift giuste devo testarle!
        case Opcodes.OP_ISHL:
        case Opcodes.OP_LSHL:                              
            return(new SEShl());
        case Opcodes.OP_ISHR:
        case Opcodes.OP_LSHR:                              
            return(new SEShr());
        case Opcodes.OP_IUSHR:
            return(new SEIUshr());
        case Opcodes.OP_IOR:
        case Opcodes.OP_LOR:                                
            return(new SEOr());
        case Opcodes.OP_IAND:
        case Opcodes.OP_LAND:                               
            return(new SEAnd());
        case Opcodes.OP_IXOR:
        case Opcodes.OP_LXOR:                               
            return(new SEXor());
        case Opcodes.OP_IINC:                              
            return(new SEIinc());
        case Opcodes.OP_DCMPG:                             
        case Opcodes.OP_DCMPL:                             
        case Opcodes.OP_FCMPG:                             
        case Opcodes.OP_FCMPL:                             
        case Opcodes.OP_LCMP:                              
            return(new SENcmp());
            
        //C - TYPE CONVERSION INSTRUCTIONS
        //C1 - Widening
        case Opcodes.OP_I2D:                                
            return(new SEN2n(Type.INT, Type.DOUBLE));
        case Opcodes.OP_I2F:                                
            return(new SEN2n(Type.INT, Type.FLOAT));
        case Opcodes.OP_I2L:                                
            return(new SEN2n(Type.INT, Type.LONG));
        case Opcodes.OP_L2D:                                
            return(new SEN2n(Type.LONG, Type.DOUBLE));
        case Opcodes.OP_L2F:                                
            return(new SEN2n(Type.LONG, Type.FLOAT));
        case Opcodes.OP_F2D:                                
            return(new SEN2n(Type.FLOAT, Type.DOUBLE));

        //C2 - Narrowing
        case Opcodes.OP_D2F:
            return(new SEN2n(Type.DOUBLE, Type.FLOAT));
        case Opcodes.OP_D2I:
            return(new SEN2n(Type.DOUBLE, Type.INT));
        case Opcodes.OP_D2L:
            return(new SEN2n(Type.DOUBLE, Type.LONG));
        case Opcodes.OP_F2I:
            return(new SEN2n(Type.FLOAT, Type.INT));
        case Opcodes.OP_F2L:
            return(new SEN2n(Type.FLOAT, Type.LONG));
        case Opcodes.OP_I2B:
            return(new SEN2n(Type.INT, Type.BYTE));
        case Opcodes.OP_I2C:
            return(new SEN2n(Type.INT, Type.CHAR));
        case Opcodes.OP_I2S:
            return(new SEN2n(Type.INT, Type.SHORT));
        case Opcodes.OP_L2I:
            return(new SEN2n(Type.LONG, Type.INT));
            
        //D - OBJECT CREATION AND MANIPULATION
        //D1 - Create a new class instance
        case Opcodes.OP_NEW:
            return(new SENew());

        //D2 - Create a new array
        case Opcodes.OP_NEWARRAY:
            return(new SENewarray());
        case Opcodes.OP_ANEWARRAY:
            return(new SEAnewarray());
        case Opcodes.OP_MULTIANEWARRAY:
            return(new SEMultianewarray());
            
        //D3 - Access fields of classes and class instances
        case Opcodes.OP_GETFIELD:
            return(new SEGetfield());
        case Opcodes.OP_PUTFIELD:
            return(new SEPutfield());
        case Opcodes.OP_GETSTATIC:
            return(new SEGetstatic());
        case Opcodes.OP_PUTSTATIC:
            return(new SEPutstatic());

        //D4 - Load an array component onto the operand stack
        case Opcodes.OP_AALOAD:
        case Opcodes.OP_CALOAD:
        case Opcodes.OP_BALOAD:
        case Opcodes.OP_DALOAD:
        case Opcodes.OP_FALOAD:
        case Opcodes.OP_IALOAD:
        case Opcodes.OP_LALOAD:
        case Opcodes.OP_SALOAD:                             
            return(new SEAload());

        //D5 - Store a value from the operand stack as an array component
        case Opcodes.OP_AASTORE:
        case Opcodes.OP_CASTORE:
        case Opcodes.OP_BASTORE:
        case Opcodes.OP_DASTORE:
        case Opcodes.OP_FASTORE:
        case Opcodes.OP_IASTORE:
        case Opcodes.OP_LASTORE:
        case Opcodes.OP_SASTORE:
            return(new SEAstore());

        //D6 - Get the length of array
        case Opcodes.OP_ARRAYLENGTH:
            return(new SEArrayLength());

        //D7 - Check the properties of class instances or arrays
        case Opcodes.OP_INSTANCEOF:
            return(new SEInstanceof());
        case Opcodes.OP_CHECKCAST:
            return(new SECheckcast());
            
        //E - OPERAND STACK MANAGEMENT INSTRUCTIONS
        case Opcodes.OP_POP:
            return(new SEPop(true));
        case Opcodes.OP_POP2:
            return(new SEPop(false));
        case Opcodes.OP_DUP:
            return(new SEDup(true));
        case Opcodes.OP_DUP2:
            return(new SEDup(false));
        case Opcodes.OP_DUP_X1:
            return(new SEDup_n(true, true));
        case Opcodes.OP_DUP2_X1:
            return(new SEDup_n(false, true));
        case Opcodes.OP_DUP_X2:
            return(new SEDup_n(true, false));
        case Opcodes.OP_DUP2_X2:
            return(new SEDup_n(false, false));
        case Opcodes.OP_SWAP:
            return(new SESwap());

        //F - CONTROL TRANSFER INSTRUCTIONS
        //F1 - Conditional branch
        //comparison with zero
        case Opcodes.OP_IFEQ:
            return(new SEIfcond(true, Operator.EQ));
        case Opcodes.OP_IFGE:
            return(new SEIfcond(true, Operator.GE));
        case Opcodes.OP_IFGT:
            return(new SEIfcond(true, Operator.GT));
        case Opcodes.OP_IFLE:
            return(new SEIfcond(true, Operator.LE));
        case Opcodes.OP_IFLT:
            return(new SEIfcond(true, Operator.LT));
        case Opcodes.OP_IFNE:
            return(new SEIfcond(true, Operator.NE));
        //comparison with null
        case Opcodes.OP_IFNONNULL:
            return(new SEIfnull(false));
        case Opcodes.OP_IFNULL:
            return(new SEIfnull(true));
        //integer comparison
        case Opcodes.OP_IF_ICMPEQ:
            return(new SEIfcond(false, Operator.EQ));
        case Opcodes.OP_IF_ICMPGE:
            return(new SEIfcond(false, Operator.GE));
        case Opcodes.OP_IF_ICMPGT:
            return(new SEIfcond(false, Operator.GT));
        case Opcodes.OP_IF_ICMPLE:
            return(new SEIfcond(false, Operator.LE));
        case Opcodes.OP_IF_ICMPLT:
            return(new SEIfcond(false, Operator.LT));
        case Opcodes.OP_IF_ICMPNE:
            return(new SEIfcond(false, Operator.NE));
        //reference comparison
        case Opcodes.OP_IF_ACMPEQ:
            return(new SEIfacmp(true));
        case Opcodes.OP_IF_ACMPNE:
            return(new SEIfacmp(false));
            
        //F2 - Compound conditional branch
        case Opcodes.OP_TABLESWITCH:
        	return(new SESwitch(true)); 
        case Opcodes.OP_LOOKUPSWITCH:
        	return(new SESwitch(false)); 
            
        //F3 - Undonditional branch
        case Opcodes.OP_GOTO:
            return(new SEGoto(true));
        case Opcodes.OP_GOTO_W:
            return(new SEGoto(false));
        case Opcodes.OP_JSR:
            return(new SEJsr(true));
        case Opcodes.OP_JSR_W:
            return(new SEJsr(false));
        case Opcodes.OP_RET:
            return(new SERet());
            
        //G - METHOD INVOCATION AND RETURN INSTRUCTIONS
        case Opcodes.OP_INVOKEINTERFACE:
            return(new SEInvokeVirtualInterface(true));
        case Opcodes.OP_INVOKEVIRTUAL:
            return(new SEInvokeVirtualInterface(false));
        case Opcodes.OP_INVOKESPECIAL:
            return(new SEInvokeSpecial());
        case Opcodes.OP_INVOKESTATIC:
            return(new SEInvokeStatic());
        case Opcodes.OP_RETURN:
            return(new SEReturn(true));
        case Opcodes.OP_ARETURN:
        case Opcodes.OP_DRETURN:
        case Opcodes.OP_FRETURN:
        case Opcodes.OP_IRETURN:
        case Opcodes.OP_LRETURN:                   
            return(new SEReturn(false));
        
        //H - THROWING EXCEPTIONS
        case Opcodes.OP_ATHROW:                   
            return(new SEAthrow());

        //I - WIDE PREFIX
        case Opcodes.OP_WIDE:
        	return(new SEWide());

        //J - MONITOR ENTER AND EXIT
        case Opcodes.OP_MONITORENTER:
        case Opcodes.OP_MONITOREXIT:
        	return(new SEMonitor());

        //K - RESERVED
        case Opcodes.OP_UNDEF:
        case Opcodes.OP_BREAKPOINT:
        case Opcodes.OP_IMPDEP1:
        case Opcodes.OP_IMPDEP2:
            return(new SENop()); //graceful behaviour
        
        //L' - TO IMPLEMENT
        case Opcodes.OP_LUSHR:
        	throw new UndefInstructionException("LUSHR");

        //L - UNASSIGNED
        case Opcodes.OP_ANEWARRAY_QUICK:
        case Opcodes.OP_CHECKCAST_QUICK:
        case Opcodes.OP_GETSTATIC_QUICK:
        case Opcodes.OP_GETSTATIC2_QUICK:
        case Opcodes.OP_GETFIELD_QUICK:
        case Opcodes.OP_GETFIELD_QUICK_W:
        case Opcodes.OP_GETFIELD2_QUICK:
        case Opcodes.OP_INSTANCEOF_QUICK:
        case Opcodes.OP_INVOKEINTERFACE_QUICK:
        case Opcodes.OP_INVOKENONVIRTUAL_QUICK:
        case Opcodes.OP_INVOKESTATIC_QUICK:
        case Opcodes.OP_INVOKESUPER_QUICK:
        case Opcodes.OP_INVOKEVIRTUAL_QUICK:
        case Opcodes.OP_INVOKEVIRTUAL_QUICK_W:
        case Opcodes.OP_INVOKEVIRTUALOBJECT_QUICK:
        case Opcodes.OP_LDC_QUICK:
        case Opcodes.OP_LDC_W_QUICK:
        case Opcodes.OP_LDC2_W_QUICK:
        case Opcodes.OP_MULTIANEWARRAY_QUICK:
        case Opcodes.OP_NEW_QUICK:
        case Opcodes.OP_PUTFIELD_QUICK:
        case Opcodes.OP_PUTFIELD_QUICK_W:
        case Opcodes.OP_PUTFIELD2_QUICK:
        case Opcodes.OP_PUTSTATIC_QUICK:
        case Opcodes.OP_PUTSTATIC2_QUICK:

        default:
        	return(new SEUnexpected());
        }
    }
}