package muse.algo;

import muse.bc.Signature;
import muse.mem.State;
import muse.mem.Value;

/**
 * A Strategy whose responsibility is to produce the effects of a 
 * native method invocation.
 * 
 * @author pietro
 */
public interface NativeInvoker {
	/**
	 * Produces the result of a native method invocation.
	 * 
	 * @param state
     *        the <code>State</code> which must be modified.
	 * @param methodSignatureResolved
     *        the resolved <code>SESignature</code> of the method which 
     *        must be invoked.
	 * @param args
     *        an array of <code>Value</code>s which are the arguments 
     *        of the invocation.
	 * @throws CannotInvokeNativeException whenever the preconditions of
	 *         <code>doInvokeNative</code> are violated.
	 */
	void doInvokeNative(State state, Signature methodSignatureResolved, Value[] args) throws CannotInvokeNativeException;
}
