package muse.algo;

import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Type;
import muse.mem.Value;

class SEDup implements Algorithm {
	/** <code>true</code> for dup, <code>false</code> for dup2. */
	boolean cat_1;
	
	//INTERIM
	public SEDup() { }
	/**
	 * Constructor.
	 * @param cat <code>true</code> for dup, <code>false</code> 
	 * for dup2. 
	 */
	public SEDup(boolean cat_1) {
		this.cat_1 = cat_1; 
	}
	
	/**
	 * Implements the true symbolic execution algorithm
	 */
	public void exec(State state, AlgorithmContext ctx) {
		boolean error = false;
		Frame frame = state.getStack().currentFrame();
		Value tmp1 = frame.pop();
		if (this.cat_1) {
			//dup
			if (Type.isCat_1(tmp1.getType())) {
				//dup semantics
				frame.push(tmp1);
				frame.push(tmp1);
			} else {
				//dup: incorrect operand
				error = true;
			}
		} else {
			//dup2
			if (Type.isCat_1(tmp1.getType())) {
				//dup2: we need a second operand 
				Value tmp2 = frame.pop();
				if (Type.isCat_1(tmp2.getType())) {
					//dup2 form 1 semantics
					frame.push(tmp2);
					frame.push(tmp1);
					frame.push(tmp2);
					frame.push(tmp1);
				} else {
					//dup2: incorrect operand 2
					error = true;
				}
			} else {
				//dup2 form 2 semantics 
				frame.push(tmp1);
				frame.push(tmp1);
			}
		}

		try {
			state.incPC();
		} catch (CodeOutOfIndexException e) {
			error = true;
		}

		//common updates
		if (error) {
	    	Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
	    	state.doThrow(myExceRef);
		}
	}
}
