package muse.algo;

import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Type;
import muse.mem.Value;

class SEDup_n implements Algorithm {
	/** <code>true</code> for dup, <code>false</code> for dup2. */
	boolean cat_1;

	/** <code>true</code> for X1, <code>false</code> for X2. */
	boolean x_1;

	//INTERIM
	public SEDup_n () { }
	/**
	 * Constructor. 
	 * @param x_1 <code>true</code> for X1, <code>false</code> for X2.
	 */
	public SEDup_n(boolean cat_1, boolean x_1) {
		this.cat_1 = cat_1;
		this.x_1 = x_1;
	}
	
	/**
	 * Implements the true symbolic execution algorithm
	 */
	public void exec(State state, AlgorithmContext ctx) {
		boolean error = false;
		
		Frame frame = state.getStack().currentFrame();
		Value tmp1 = frame.pop();
		Value tmp2 = frame.pop();
		Value tmp3 = null;
		Value tmp4 = null;
		if (x_1) {
			if (cat_1) {
				//dup_x1: the two operands must be of category 1
				if (Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup_x1 semantics
					frame.push(tmp1);
					frame.push(tmp2);
					frame.push(tmp1);
				} else {
					//dup_x1: incorrect operands
					error = true;
				}
			} else {
				if (Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup2_x1: we need a third operand
					tmp3 = frame.pop();
					if (Type.isCat_1(tmp3.getType())) {
						//dup2_x1 form 1 semantics
						frame.push(tmp2);
						frame.push(tmp1);
						frame.push(tmp3);
						frame.push(tmp2);
						frame.push(tmp1);
					} else {
						//dup2_x1: incorrect operand 3
						error = true;
					}
				} else if (!Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup2_x1 form 2 semantics
					frame.push(tmp1);
					frame.push(tmp2);
					frame.push(tmp1);
				} else {
					//dup2_x1: incorrect operand 2
					error = true;
				}
			}
		} else {
			if (cat_1) {
				//dup_x2
				if (Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup_x2: we need a third operand
					tmp3 = frame.pop();
					if (Type.isCat_1(tmp3.getType())) {
						//dup_x2 form 1 semantics
						frame.push(tmp1);
						frame.push(tmp3);
						frame.push(tmp2);
						frame.push(tmp1);						
					} else {
						//dup_x2: incorrect operand 3
						error = true;
					}
				} else if (Type.isCat_1(tmp1.getType()) && !Type.isCat_1(tmp2.getType())) {
					//dup_x2 form 2 semantics
					frame.push(tmp1);
					frame.push(tmp2);
					frame.push(tmp1);
				} else {
					//dup_x2: incorrect operand 1
					error = true;
				}
			} else {
				//dup2_x2
				if (Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup2_x2: we need a third operand
					tmp3 = frame.pop();
					if (Type.isCat_1(tmp3.getType())) {
						//dup2_x2: we need a fourth operand
						tmp4 = frame.pop();
						if (Type.isCat_1(tmp4.getType())) {
							//dup2_x2 form 1 semantics
							frame.push(tmp2);
							frame.push(tmp1);
							frame.push(tmp4);
							frame.push(tmp3);
							frame.push(tmp2);
							frame.push(tmp1);
						} else {
							//dup2_x2: incorrect operand 4
							error = true;
						}
					} else {
						//dup2_x2 form 3 semantics
						frame.push(tmp2);
						frame.push(tmp1);
						frame.push(tmp3);
						frame.push(tmp2);
						frame.push(tmp1);
					}
				} else if (!Type.isCat_1(tmp1.getType()) && Type.isCat_1(tmp2.getType())) {
					//dup2_x2: we need a third operand
					tmp3 = frame.pop();
					if (Type.isCat_1(tmp1.getType())) {
						//dup2_x2 form 2 semantics
						frame.push(tmp1);
						frame.push(tmp3);
						frame.push(tmp2);
						frame.push(tmp1);
					} else {
						//dup2_x2: incorrect operand 3
						error = true;
					}
				} else if (!Type.isCat_1(tmp1.getType()) && !Type.isCat_1(tmp2.getType())) {
					//dup2_x2 form 4 semantics					
					frame.push(tmp1);
					frame.push(tmp2);
					frame.push(tmp1);
				} else {
					//dup2_x2: incorrect operand 2
					error = true;
				}
			}
		}
		
		try {
			state.incPC();
		} catch (CodeOutOfIndexException e) {
			error = true;
		}

		//common updates
		if (error) {
	    	Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
	    	state.doThrow(myExceRef);
		}
	}
}
