package muse.algo;

import muse.bc.Signature;
import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Heap;
import muse.mem.Instance;
import muse.mem.Reference;
import muse.mem.ReferenceSymbolic;
import muse.mem.State;
import muse.mem.Type;
import muse.mem.Value;

class SEGetfield implements Algorithm {
    public void exec(State state, AlgorithmContext ctx) throws DecisionException, CannotManageStateException, PartialReferenceResolutionException {
        Frame frame = state.getStack().currentFrame();

        //gets the index of the field signature in the current class 
    	//constant pool
    	int index;
        try {
            byte tmp1 = state.getInstruction(1);
            byte tmp2 = state.getInstruction(2);
            index = Util.byteCat(tmp1, tmp2);
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

		//pops the reference to the Instance containing the 
		//field from the operand stack and fetches the Instance 
		//from the state's heap 
        Reference myObjectRef = (Reference) frame.pop();
        if (myObjectRef.isNull()) {
        	//base-level throws NullPointerException 
	    	Reference myExceRef = Util.loadInstance(state, "java/lang/NullPointerException");
			frame.push(myExceRef);
	    	state.doThrow(myExceRef);
	    	return;
        }
        Heap heap = state.getHeap();
        Instance myObject = (Instance) heap.getObject(myObjectRef); 
        
        //gets and resolves the field signature
        Signature fieldSignatureResolved = null;
        boolean failed = false;
		String myExce = null;
		try {
	        //gets the field signature from the current class constant pool
	        String classToOpen = frame.getName().getClassName();    
	        Signature fieldSignature = state.getClassFileInterface().getClassFile(classToOpen).getFieldSignature(index);  
	        
	        //performs resolution
			fieldSignatureResolved = Util.resolveField(state.getClassFileInterface(), fieldSignature);
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}		

		//gets the field's value
        Value tmpValue = myObject.getFieldValue(fieldSignatureResolved);

        //generates all the next states
        if (Type.isUninitializedReference(tmpValue)) {            
            StateGeneratorReference stateGenerator = new StateGeneratorReference(state, ctx.stateTree, ctx.decisionProcedure, 3);
            stateGenerator.generateStatesGetfield((ReferenceSymbolic) tmpValue, index, myObjectRef);
        } else {
            /* TODO the following code is a duplication of the last phase (update) 
             * of StateGeneratorReference.generateStatesGetfield. It is still unclear 
             * how to eliminate such redundancy without a major refactoring.
             */
            frame.push(tmpValue);
            try {
				state.incPC(3);
			} catch (CodeOutOfIndexException e) {
				Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
				frame.push(myExceRef);
				state.doThrow(myExceRef);
			}
        }
    } 
}