package muse.algo;

import muse.bc.AttributeNotFoundException;
import muse.bc.ClassFile;
import muse.bc.FieldNotFoundException;
import muse.bc.Signature;
import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.ConstantPoolString;
import muse.mem.Frame;
import muse.mem.Klass;
import muse.mem.Reference;
import muse.mem.ReferenceSymbolic;
import muse.mem.State;
import muse.mem.Type;
import muse.mem.Value;

class SEGetstatic implements Algorithm {
    public void exec(State state, AlgorithmContext ctx) throws DecisionException, CannotManageStateException, PartialReferenceResolutionException {
        Frame frame = state.getStack().currentFrame();

        //gets the index of the field signature in the current class 
    	//constant pool
        int index;
        try {
	        byte tmp1 = state.getInstruction(1);
	        byte tmp2 = state.getInstruction(2);
	        index = Util.byteCat(tmp1, tmp2);
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

        //gets and resolves the field signature
        Signature fieldSignatureResolved = null;
        boolean failed = false;
		String myExce = null;
		try {
			//gets the field signature from the current class constant pool
	        String currentClassName = frame.getName().getClassName();        
	        Signature fieldSignature = state.getClassFileInterface().getClassFile(currentClassName).getFieldSignature(index);
	        
	        //performs resolution
			fieldSignatureResolved = Util.resolveField(state.getClassFileInterface(), fieldSignature);
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}		
                
        //checks whether the field is constant
        String fieldClassName = fieldSignatureResolved.getClassName();
		ClassFile cf = state.getClassFileInterface().getClassFile(fieldClassName);
		boolean isConstant;
		try {
			isConstant = cf.isFieldFinal(fieldSignatureResolved) && cf.hasFieldConstantValue(fieldSignatureResolved);
		} catch (FieldNotFoundException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}
		
		//gets the field's value, possibly performing class initialization
        Value tmpValue = null;
    	if (isConstant) {
    		try {
				tmpValue = cf.fieldConstantValue(fieldSignatureResolved);
			} catch (FieldNotFoundException e) {
				//nothing to do, should never happen
			} catch (AttributeNotFoundException e) {
				//nothing to do, should never happen
			}
			if (tmpValue instanceof ConstantPoolString) {
				tmpValue = Util.loadStringLiteral(state, tmpValue.toString(), ctx.decisionProcedure);
			}
    	} else if (state.initialized(fieldClassName)) {
            //checks whether a Klass object exists in state.
            //No Klass object means that the class is initialized by 
        	//virtue of the state's path condition (namely by 
        	//its precondition, since in the other cases the Klass 
        	//object must exist). In such case, a Klass object is 
        	//created and filled with symbolic values.
        	Klass k;
        	if (state.hasKlass(fieldClassName)) {
        		k = state.getKlass(fieldClassName);
        	} else {
        		k = ClassInitializer.loadSymbolicKlass(state, fieldClassName, ctx.decisionProcedure);
        		state.setKlass(k, fieldClassName);
       			state.getPathCondition().concatenate(k, fieldClassName);
        	}
            tmpValue = k.getFieldValue(fieldSignatureResolved);
        } else {
        	ClassInitializer.initializeClass(state, fieldClassName, ctx.decisionProcedure);
        	return;
        	//now the execution continues with the class 
        	//initialization code; the current bytecode will 
        	//be reexecuted after that
        }
        
        //generates all the next states
        if (Type.isUninitializedReference(tmpValue)) {            
            StateGeneratorReference stateGenerator = new StateGeneratorReference(state, ctx.stateTree, ctx.decisionProcedure, 3);
            stateGenerator.generateStatesGetstatic((ReferenceSymbolic) tmpValue, index);
        } else {
            /* TODO the following code is a duplication of the last phase (update) 
             * of StateGeneratorReference.generateStatesGetstatic. It is still unclear 
             * how to eliminate such redundancy without a major refactoring.
             */
            frame.push(tmpValue);
            try {
				state.incPC(3);
			} catch (CodeOutOfIndexException e) {
				Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
				frame.push(myExceRef);
				state.doThrow(myExceRef);
			}
        }
    } 
}
