package muse.algo;

import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Primitive;
import muse.mem.Reference;
import muse.mem.Simplex;
import muse.mem.State;

class SEIfcond implements Algorithm {
    boolean def;
    String type;

    //INTERIM
    public SEIfcond() { }
    /**
     * Constructor by default, only initialize the Object
     */
    public SEIfcond(boolean def, String type) {
        this.def = def;
        this.type = type;
    }
    
    /**
     * Implements the true symbolic execution algorithm
     */
    public void exec(State state, AlgorithmContext ctx) throws DecisionException {
        Frame frame = state.getStack().currentFrame();
        
        //gets operands and calculates branch target
        int branchOffset;
        try {
	        byte tmp1 = state.getInstruction(1);
	        byte tmp2 = state.getInstruction(2);
	        branchOffset = Util.byteCatShort(tmp1, tmp2);
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		} //note that now the program counter points to the next instruction

        //takes operands from current frame's operand stack
        Primitive val1 = (Primitive) frame.pop();
        Primitive val2;
        if (def) {
            val2 = (Primitive) new Simplex(0);
        } else {
            //swaps the operands for comparison;
            val2 = val1;
            val1 = (Primitive) frame.pop();
        }
        
		//generates all the states
		StateGeneratorPrimitive g = new StateGeneratorPrimitive(state, ctx.stateTree, ctx.decisionProcedure);
		g.generateStatesIfcond(val1, type, val2, branchOffset);
    }    
}