package muse.algo;

import java.util.HashSet;

import muse.bc.ClassFile;
import muse.bc.ClassFileInterface;
import muse.bc.ClassHierarchy;
import muse.bc.Signature;
import muse.dec.DecisionException;
import muse.dec.DecisionInitException;
import muse.jvm.InitializationException;
import muse.mem.Reference;
import muse.mem.State;


public class SEInit implements Algorithm {
	public void exec(State state, AlgorithmContext ctx) throws DecisionInitException {
		//sets the simplifier with the initial state's path condition
		try {
			ctx.decisionProcedure.pushClause(state.getPathCondition().getCondition());
		} catch (DecisionException e) {
			throw new DecisionInitException("error when pushing precondition");
		}
		
		//puts the initial state in the initializer
		ctx.stateTree.addState(state);
	}
	
	public void exec(Signature sgnMethodSignature, AlgorithmContext ctx) throws DecisionInitException, InitializationException {
		//TODO checks and exception rising

		//builds the state
		State state = new State(new ClassFileInterface(ctx.env, ctx.classFileFactory)); 

		//adds the root object and the root klass to the initial state
		String strClassName = sgnMethodSignature.getClassName();
		Reference hInstance = Util.loadSymbolicInstance(state, strClassName, null, "root");
		try {
			state.setKlass(ClassInitializer.loadSymbolicKlass(state, strClassName, ctx.decisionProcedure), strClassName);
		} catch (DecisionException e) {
			throw new DecisionInitException("error when loading root klass");
		}

		//sets the state's not initialized classes (filters out the 
		//current class' superclasses)
		ClassHierarchy hier = new ClassHierarchy(state.getClassFileInterface());
		HashSet<String> superclasses = new HashSet<String>();
		for (ClassFile cf : hier.superclasses(strClassName)) {
			superclasses.add(cf.getClassName());
		}
		for (String s : ctx.env.getNotInitializedClasses()) {
			if (!superclasses.contains(s)) {
				state.setNotInitialized(s);
			}
		}

		//adds a method frame for the initial method invocation
		//TODO decide whether to perform method resolution and method body lookup as for invoke* bytecodes
		try {
			Util.loadSymbolicFrame(state, sgnMethodSignature, hInstance, true, 0);
		} catch (JavaReifyException e) {
			//TODO decide what to do in this case
			throw new InitializationException("method " + sgnMethodSignature + " does not exist");
		} catch (PleaseDoNativeException e) {
			// TODO decide what to do in this case
			throw new InitializationException("native method " + sgnMethodSignature + " cannot be executed symbolically");
		}

		//completes
		this.exec(state, ctx);
	}
}
