package muse.algo;

import muse.bc.ClassFile;
import muse.bc.Signature;
import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Value;

class SEInvokeStatic implements Algorithm {
	public void exec(State state, AlgorithmContext ctx) throws CannotManageStateException, DecisionException {
		Frame frame = state.getStack().currentFrame();

		//gets index operand from instruction word and 
		//calculates/stores the pointer to the next instruction
		int index;
		try {
			byte tmp1 = state.getInstruction(1);
			byte tmp2 = state.getInstruction(2);
			index = Util.byteCat(tmp1,tmp2);
		} catch (CodeOutOfIndexException e1) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}
		
		//gets the signature of the method to be invoked
		String openClass = frame.getName().getClassName();
		ClassFile classFile = state.getClassFileInterface().getClassFile(openClass);	
		Signature methodSignature = classFile.getMethodSignature(index);

		//initializes the class in case it is not
        if (!state.initialized(openClass)) {
        	ClassInitializer.initializeClass(state, openClass, ctx.decisionProcedure);
        	return;
        	//now the execution continues with the class 
        	//initialization code; the current bytecode will 
        	//be reexecuted after that
        }
        
		//performs method resolution
		//TODO check if resolution for static method performed like that is OK (it is a modified copy of SEInvokeVirtualInterface)  
		boolean failed = false;
		String myExce = null;
		try {
			Signature methodSignatureResolved = Util.resolveMethod(state.getClassFileInterface(), false, methodSignature);
			ClassFile classFileResolved = state.getClassFileInterface().getClassFile(methodSignatureResolved.getClassName());
			if (!classFileResolved.isMethodStatic(methodSignatureResolved)) {
				failed = true;
				myExce = "java/lang/IncompatibleClassChangeError";
			}
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}

		//creates the method frame and sets the parameters on the operand stack
		Value[] args = Util.popMethodCallArgs(state, methodSignature, true);
		try {
			try {
				Util.loadFrameStatic(state, methodSignature, args, 3);
			} catch (PleaseDoNativeException e) {
				//TODO get native invoker from somewhere (perhaps state)
				NativeInvokerPure ni = new NativeInvokerPure(3);
				ni.doInvokeNative(state, methodSignature, args);
			}
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}
	}
}