package muse.algo;

import muse.bc.ClassFile;
import muse.bc.Signature;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Value;

class SEInvokeVirtualInterface implements Algorithm {
	boolean isInterface;
	
	//INTERIM
	public SEInvokeVirtualInterface() {}
	/**
	 * Constructor.
	 */
	public SEInvokeVirtualInterface(boolean isInterface) {
		this.isInterface = isInterface;
	}

	/**
	 * Implements the true symbolic execution algorithm
	 */
	public void exec(State state, AlgorithmContext ctx) throws CannotManageStateException {
		Frame frame = state.getStack().currentFrame();

		//gets index operand from instruction word and 
		//calculates/stores the pointer to the next instruction
		int index;
		try {
			byte tmp1 = state.getInstruction(1);
			byte tmp2 = state.getInstruction(2);
			index = Util.byteCat(tmp1, tmp2);
		} catch(CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}
		
		//gets the signature of the method to be invoked and
		//calculates the program counter offset
		String openClass = frame.getName().getClassName();
		ClassFile classFile = state.getClassFileInterface().getClassFile(openClass);
		int pcOffset;
		Signature methodSignature;
		if (isInterface) {
			methodSignature = classFile.getInterfaceMethodSignature(index);
			pcOffset = 5;
		} else {
			methodSignature = classFile.getMethodSignature(index);
			pcOffset = 3;
		}
		
		//performs method resolution
		Signature methodSignatureResolved = null;
		boolean failed = false;
		String myExce = null;
		try {
			methodSignatureResolved = Util.resolveMethod(state.getClassFileInterface(), isInterface, methodSignature);
			ClassFile classFileResolved = state.getClassFileInterface().getClassFile(methodSignatureResolved.getClassName());
			if (classFileResolved.isMethodStatic(methodSignatureResolved)) {
				failed = true;
				myExce = "java/lang/IncompatibleClassChangeError";
			}
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}

		//creates the method frame and sets the parameters on the operand stack
		Value[] args = Util.popMethodCallArgs(state, methodSignature, false);
		try {
			try {
				Util.loadFrame(false, state, methodSignatureResolved, args, pcOffset);
			} catch (PleaseDoNativeException e) {
				//TODO get native invoker from somewhere (perhaps state)
				NativeInvokerPure ni = new NativeInvokerPure(pcOffset);
				ni.doInvokeNative(state, methodSignature, args);
			}
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}		
	}
}