package muse.algo;

import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.Simplex;
import muse.mem.State;

class SEJsr implements Algorithm {
	/** <code>true</code> for jsr, <code>false</code> for jsr_w. */
	boolean def;

	//INTERIM
	public SEJsr() { }
	/**
	 * Constructor.
	 *
	 * @param def <code>true</code> for jsr, <code>false</code> for jsr_w.
	 */
	public SEJsr(boolean def) {
		this.def = def;
	}
	
	/**
	 * Implements the true symbolic execution algorithm
	 */
	public void exec(State state, AlgorithmContext ctx) {
		Frame frame = state.getStack().currentFrame();
		try {
			int iP = state.getPC();
			byte tmp1 = state.getInstruction(1);
			byte tmp2 = state.getInstruction(2);
			if (def) {
				frame.push(new Simplex(iP + 3));
				state.incPC(Util.byteCatShort(tmp1, tmp2));
			} else {
				byte tmp3 = state.getInstruction(3);
				byte tmp4 = state.getInstruction(4);
				frame.push(new Simplex(iP + 5));
				state.incPC(Util.byteCat(tmp1, tmp2, tmp3, tmp4));
			}
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}
	}
}