package muse.algo;

import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.ConstantPoolString;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Value;

class SELdc implements Algorithm {
	boolean def;

	//INTERIM
	public SELdc() { }
	
	/**
	 * Constructor.
	 * @param def <code>true</code> for ldc, <code>false</code> for
	 *        ldc_w and ldc2_w.
	 */
	public SELdc(boolean def) {
		this.def = def;
	}
	
	/**
	 * Implements the true symbolic execution algorithm
	 * @throws DecisionException 
	 */
	public void exec(State state, AlgorithmContext ctx) throws DecisionException {
		Frame frame = state.getStack().currentFrame();
		int index;
		try {
			byte tmp1 = state.getInstruction(1);
			if (this.def) {
				index = Util.byteCat((byte) 0, tmp1);
			} else {
				byte tmp2 = state.getInstruction(2);
				index = Util.byteCat(tmp1, tmp2);
			}
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

		//gets the current class and pushes the value stored 
		//at index
		String currentClassName = frame.getName().getClassName();
		Value val = state.getClassFileInterface().getClassFile(currentClassName).getValueFromConstantPool(index);
		
		//if the value is a string, creates/fetches the corresponding 
		//Instance in the literal area (the true value) 
		if (val instanceof ConstantPoolString) {
			val = Util.loadStringLiteral(state, val.toString(), ctx.decisionProcedure);
		}

		//pushes the value on the operand stack
		frame.push(val);
		
		try {
			if (this.def) {
				state.incPC(2);
			} else {
				state.incPC(3);
			}
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
		}
	} 
}
