package muse.algo;

import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Reference;
import muse.mem.ReferenceSymbolic;
import muse.mem.State;
import muse.mem.Type;
import muse.mem.Value;

class SELoad implements Algorithm {
    boolean def;
    int index;

    public SELoad() {
        this.def = false;
    }
    
    public SELoad(int param) {
        this.def = true;
        this.index = param;
    }
    
    public void exec(State state, AlgorithmContext ctx) throws DecisionException, CannotManageStateException, PartialReferenceResolutionException {
    	boolean wide = ctx.wide;
    	ctx.wide = false;
        Frame frame = state.getStack().currentFrame();

        //gets the index of the local variable and 
        //calculates the program counter offset for
        //the next instruction
        int pcOffset;
        try {
            if (this.def) {
            	//index already initialized by constructor
            	pcOffset = 1;
            } else if (wide) {
            	byte tmp1 = state.getInstruction(1);
            	byte tmp2 = state.getInstruction(2);
            	this.index = Util.byteCat(tmp1, tmp2);
            	pcOffset = 3;
            } else {
            	this.index = state.getInstruction(1);
            	pcOffset = 2;
            }
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

        //gets the local variable from the current frame
        Value tmpValue = frame.getLocalVariableTable().get(this.index);

        //generates all the next states
        if (Type.isUninitializedReference(tmpValue)) {
            StateGeneratorReference stateGenerator = new StateGeneratorReference(state, ctx.stateTree, ctx.decisionProcedure, pcOffset);
            stateGenerator.generateStatesLoad((ReferenceSymbolic) tmpValue, this.index);
        } else {
            /* TODO the following code is a duplication of the last phase (update) 
             * of StateGeneratorReference.generateStatesLoad. It is still unclear 
             * how to eliminate such redundancy without a major refactoring.
             */
            frame.push(tmpValue);
            try {
				state.incPC(pcOffset);
			} catch (CodeOutOfIndexException e) {
				Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
				frame.push(myExceRef);
				state.doThrow(myExceRef);
			}
        }
    } 
}