package muse.algo;

import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Primitive;
import muse.mem.Reference;
import muse.mem.State;

class SEMultianewarray implements Algorithm {
	public void exec(State state, AlgorithmContext ctx) throws DecisionException, CannotManageStateException, PartialReferenceResolutionException {
		Frame frame = state.getStack().currentFrame();

		//determines the array's type and the number of instantiation 
		//dimensions
		int ndims;
		String arrayType;
		{
			int index;
			try {
				byte tmp1 = state.getInstruction(1);
				byte tmp2 = state.getInstruction(2); 
				index = Util.byteCat(tmp1, tmp2);
				ndims = state.getInstruction(3);
			} catch (CodeOutOfIndexException e) {
				Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
				frame.push(myExceRef);
				state.doThrow(myExceRef);
				return;
			}
			String classToOpen = frame.getName().getClassName();    
			arrayType = state.getClassFileInterface().getClassFile(classToOpen).getClassSignature(index);
		}

		//checks the number of instantiation dimensions
		if (ndims <= 0) { //TODO check that no error arise because of incorrect byte catenation interpreted as negative
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

		//pops the array's dim length values and puts them in param
		Primitive[] length = new Primitive[ndims];
		for (int i = ndims - 1; i >= 0; i--) {
			length[i] = (Primitive) frame.pop();
			//TODO length type check
		}

		//generates the next states
		StateGeneratorArray g = new StateGeneratorArray(state, ctx.stateTree, ctx.decisionProcedure);
		g.generateStatesAnew(arrayType, length, 4);
	}
}