package muse.algo;

import muse.bc.Signature;
import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Klass;
import muse.mem.Reference;
import muse.mem.State;
import muse.mem.Value;

class SEPutstatic implements Algorithm {
    public void exec(State state, AlgorithmContext ctx) throws DecisionException {
		Frame frame = state.getStack().currentFrame();

		//gets the index of the field signature in the current class 
    	//constant pool
		int index;
		try {
			byte tmp1 = state.getInstruction(1);
		    byte tmp2 = state.getInstruction(2);
			index = Util.byteCat(tmp1, tmp2);
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

        //gets and resolves the field signature
        Signature fieldSignatureResolved = null;
        boolean failed = false;
		String myExce = null;
		try {
			//gets the field signature from the current class constant pool
	        String currentClassName = frame.getName().getClassName();        
	        Signature fieldSignature = state.getClassFileInterface().getClassFile(currentClassName).getFieldSignature(index);
	        
	        //performs resolution
			fieldSignatureResolved = Util.resolveField(state.getClassFileInterface(), fieldSignature);
		} catch (JavaReifyException e) {
			failed = true;
			myExce = e.subject();
		} finally {
			if (failed) {
		    	Reference myExceRef = Util.loadInstance(state, myExce);
				frame.push(myExceRef);
		    	state.doThrow(myExceRef);
		    	return;
			}
		}		
                
        //obtains the Klass object containing the field from the state's 
		//static store, possibly performing class intialization (and creating it)
        Klass k = null;
        String fieldClassName = fieldSignatureResolved.getClassName();
        if (state.initialized(fieldClassName)) {
        	//checks whether a Klass object exists in state.
            //No Klass object means that the class is initialized by 
        	//virtue of the state's path condition (namely by 
        	//its precondition, since in the other cases the Klass 
        	//object must exist). In such case, a Klass object is 
        	//created and filled with symbolic values.
        	boolean added = true;
        	if (!state.hasKlass(fieldClassName)) {
        		state.setKlass(ClassInitializer.loadSymbolicKlass(state, fieldClassName, ctx.decisionProcedure), fieldClassName);
        	} else {
        		added = false;
        	}
        	k = state.getKlass(fieldClassName);
        	if (added) {
            	state.getPathCondition().concatenate(k, fieldClassName);
        	}
        } else {
        	ClassInitializer.initializeClass(state, fieldClassName, ctx.decisionProcedure);
        	return;
        	//now the execution continues with the class 
        	//initialization code; the current bytecode will 
        	//be reexecuted after that
        }
        
        //pops the Value from the operand stack
        Value tmpValue = frame.pop();
        
        //sets the field's value
		k.setFieldValue(fieldSignatureResolved, tmpValue);
		
		try {
			state.incPC(3);
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
		}
    } 
 }
