package muse.algo;

import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.State;
import muse.mem.ThreadStack;
import muse.mem.Value;

class SEReturn implements Algorithm {
	/** <code>true</code> iff returning from a method with void return type. */
	boolean def;
	
	//INTERIM
	public SEReturn() {}
	/**
	 * Constructor by default, only initialize the object
	 * @param param <code>true</code> iff the method is void
	 */
	public SEReturn(boolean param) {
		def = param;
	}
	
	/**
	 *Implements the true symbolic execution algorithm
	 */
	public void exec(State state, AlgorithmContext ctx) {
		ThreadStack stack = state.getStack();
		
		if (def) {
			stack.pop();
			if (stack.empty()) {
				state.setStuck();
				return;
			}
		} else {
			Value retValue = stack.currentFrame().pop();
			stack.pop();
			if (stack.empty()) {
				state.setStuckReturn(retValue);
				return;
			}
			Frame frame = stack.currentFrame();
			frame.push(retValue);
		}
		
		try {
			state.useReturnPC();
		} catch (CodeOutOfIndexException e) {
			//TODO this should never happen, throw some internal error exception
		}
	} 
}
