package muse.algo;

import muse.dec.DecisionException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Primitive;
import muse.mem.Reference;
import muse.mem.State;

class SESwitch implements Algorithm {
	boolean isTableSwitch;

	public SESwitch() { }
	public SESwitch(boolean isTable) {
		this.isTableSwitch = isTable;
	}

	public void exec(State state, AlgorithmContext ctx) throws DecisionException {
		Frame frame = state.getStack().currentFrame();

		//pops the index
		Primitive op = (Primitive) frame.pop();
		
		//gets the base address of the jump, the 
		//default offset and the table range
		int deflt, low, high;
		try {
			int ofst = 0;

			//passes the alignment bytes
			do {
				ofst++;
			} while (state.getInstruction(ofst) % 4 != 0);

			//gets the operand bytes; note that the 
			//loop is shorter by 1 to avoid incrementing 
			//the program counter past the instruction
			byte[] t = (isTableSwitch ? new byte[12] : new byte[8]);
			for (int i = 0; i < t.length; i++) {
				t[i] = state.getInstruction(ofst);
				ofst++;
			}
			
			//does calculation
			deflt = Util.byteCat(t[0], t[1], t[2], t[3]);
			if (isTableSwitch) {
				low = Util.byteCat(t[4], t[5], t[6], t[7]);
				high = Util.byteCat(t[8], t[9], t[10], t[11]);
			} else {
				low = 1;
				high = Util.byteCat(t[4], t[5], t[6], t[7]);
			}
		} catch (CodeOutOfIndexException e) {
			Reference myExceRef = Util.loadInstance(state, "java/lang/VerifyError");
			frame.push(myExceRef);
			state.doThrow(myExceRef);
			return;
		}

		//generates all the states
		StateGeneratorPrimitive g = new StateGeneratorPrimitive(state, ctx.stateTree, ctx.decisionProcedure);
		g.generateStatesSwitch(op, this.isTableSwitch, deflt, low, high);		
	}
}