package muse.apps;

import static muse.bc.Opcodes.*;
import muse.bc.BytecodeDispatcher;
import muse.mem.Frame;

public class BytecodeFormatter extends BytecodeDispatcher<BytecodeFormatter.FormatterStrategy> {
	public interface FormatterStrategy {
		String format(Frame f);
	}
	
	/**
	 * A formatter for bytecodes with 0 operands, which just returns their name.
	 * 
	 * @author pietro
	 */
	private class DispatchStrategyFormat0 implements BytecodeDispatcher.DispatchStrategy<FormatterStrategy> {
		private final String text;
		public DispatchStrategyFormat0(String text) { this.text = text; }
		public FormatterStrategy doIt() {
			return new FormatterStrategy() {
				public String format(Frame f) { 
					return DispatchStrategyFormat0.this.text; 
				} 
			};
		}		
	}
	
	public BytecodeFormatter() {
		this
			.setDispatchNonexistentStrategy(new DispatchStrategyFormat0("<INCORRECT>"))
			.setDispatchStrategy(OP_NOP,             new DispatchStrategyFormat0("NOP"))
			.setDispatchStrategy(OP_ALOAD,           new DispatchStrategyFormat0("ALOAD"))
			.setDispatchStrategy(OP_DLOAD,           new DispatchStrategyFormat0("DLOAD"))
			.setDispatchStrategy(OP_FLOAD,           new DispatchStrategyFormat0("FLOAD"))
			.setDispatchStrategy(OP_ILOAD,           new DispatchStrategyFormat0("ILOAD"))
			.setDispatchStrategy(OP_LLOAD,           new DispatchStrategyFormat0("LLOAD"))
			.setDispatchStrategy(OP_ALOAD_0,         new DispatchStrategyFormat0("ALOAD_0"))
			.setDispatchStrategy(OP_DLOAD_0,         new DispatchStrategyFormat0("DLOAD_0"))
			.setDispatchStrategy(OP_FLOAD_0,         new DispatchStrategyFormat0("FLOAD_0"))
			.setDispatchStrategy(OP_ILOAD_0,         new DispatchStrategyFormat0("ILOAD_0"))
			.setDispatchStrategy(OP_LLOAD_0,         new DispatchStrategyFormat0("LLOAD_0"))
			.setDispatchStrategy(OP_ALOAD_1,         new DispatchStrategyFormat0("ALOAD_1"))
			.setDispatchStrategy(OP_DLOAD_1,         new DispatchStrategyFormat0("DLOAD_1"))
			.setDispatchStrategy(OP_FLOAD_1,         new DispatchStrategyFormat0("FLOAD_1"))
			.setDispatchStrategy(OP_ILOAD_1,         new DispatchStrategyFormat0("ILOAD_1"))
			.setDispatchStrategy(OP_LLOAD_1,         new DispatchStrategyFormat0("LLOAD_1"))
			.setDispatchStrategy(OP_ALOAD_2,         new DispatchStrategyFormat0("ALOAD_2"))
			.setDispatchStrategy(OP_DLOAD_2,         new DispatchStrategyFormat0("DLOAD_2"))
			.setDispatchStrategy(OP_FLOAD_2,         new DispatchStrategyFormat0("FLOAD_2"))
			.setDispatchStrategy(OP_ILOAD_2,         new DispatchStrategyFormat0("ILOAD_2"))
			.setDispatchStrategy(OP_LLOAD_2,         new DispatchStrategyFormat0("LLOAD_2"))
			.setDispatchStrategy(OP_ALOAD_3,         new DispatchStrategyFormat0("ALOAD_3"))
			.setDispatchStrategy(OP_DLOAD_3,         new DispatchStrategyFormat0("DLOAD_3"))
			.setDispatchStrategy(OP_FLOAD_3,         new DispatchStrategyFormat0("FLOAD_3"))
			.setDispatchStrategy(OP_ILOAD_3,         new DispatchStrategyFormat0("ILOAD_3"))
			.setDispatchStrategy(OP_LLOAD_3,         new DispatchStrategyFormat0("LLOAD_3"))
			.setDispatchStrategy(OP_ASTORE,          new DispatchStrategyFormat0("ASTORE"))
			.setDispatchStrategy(OP_DSTORE,          new DispatchStrategyFormat0("DSTORE"))
			.setDispatchStrategy(OP_FSTORE,          new DispatchStrategyFormat0("FSTORE"))
			.setDispatchStrategy(OP_ISTORE,          new DispatchStrategyFormat0("ISTORE"))
			.setDispatchStrategy(OP_LSTORE,          new DispatchStrategyFormat0("LSTORE"))
			.setDispatchStrategy(OP_ASTORE_0,        new DispatchStrategyFormat0("ASTORE_0"))
			.setDispatchStrategy(OP_DSTORE_0,        new DispatchStrategyFormat0("DSTORE_0"))
			.setDispatchStrategy(OP_FSTORE_0,        new DispatchStrategyFormat0("FSTORE_0"))
			.setDispatchStrategy(OP_ISTORE_0,        new DispatchStrategyFormat0("ISTORE_0"))
			.setDispatchStrategy(OP_LSTORE_0,        new DispatchStrategyFormat0("LSTORE_0"))
			.setDispatchStrategy(OP_ASTORE_1,        new DispatchStrategyFormat0("ASTORE_1"))
			.setDispatchStrategy(OP_DSTORE_1,        new DispatchStrategyFormat0("DSTORE_1"))
			.setDispatchStrategy(OP_FSTORE_1,        new DispatchStrategyFormat0("FSTORE_1"))
			.setDispatchStrategy(OP_ISTORE_1,        new DispatchStrategyFormat0("ISTORE_1"))
			.setDispatchStrategy(OP_LSTORE_1,        new DispatchStrategyFormat0("LSTORE_1"))
			.setDispatchStrategy(OP_ASTORE_2,        new DispatchStrategyFormat0("ASTORE_2"))
			.setDispatchStrategy(OP_DSTORE_2,        new DispatchStrategyFormat0("DSTORE_2"))
			.setDispatchStrategy(OP_FSTORE_2,        new DispatchStrategyFormat0("FSTORE_2"))
			.setDispatchStrategy(OP_ISTORE_2,        new DispatchStrategyFormat0("ISTORE_2"))
			.setDispatchStrategy(OP_LSTORE_2,        new DispatchStrategyFormat0("LSTORE_2"))
			.setDispatchStrategy(OP_ASTORE_3,        new DispatchStrategyFormat0("ASTORE_3"))
			.setDispatchStrategy(OP_DSTORE_3,        new DispatchStrategyFormat0("DSTORE_3"))
			.setDispatchStrategy(OP_FSTORE_3,        new DispatchStrategyFormat0("FSTORE_3"))
			.setDispatchStrategy(OP_ISTORE_3,        new DispatchStrategyFormat0("ISTORE_3"))
			.setDispatchStrategy(OP_LSTORE_3,        new DispatchStrategyFormat0("LSTORE_3"))
			.setDispatchStrategy(OP_BIPUSH,          new DispatchStrategyFormat0("BIPUSH"))
			.setDispatchStrategy(OP_SIPUSH,          new DispatchStrategyFormat0("SIPUSH"))
			.setDispatchStrategy(OP_LDC,             new DispatchStrategyFormat0("LDC"))
			.setDispatchStrategy(OP_LDC_W,           new DispatchStrategyFormat0("LDC_W"))
			.setDispatchStrategy(OP_LDC2_W,          new DispatchStrategyFormat0("LDC2_W"))
			.setDispatchStrategy(OP_ACONST_NULL,     new DispatchStrategyFormat0("ACONST_NULL"))
			.setDispatchStrategy(OP_DCONST_0,        new DispatchStrategyFormat0("DCONST_0"))
			.setDispatchStrategy(OP_DCONST_1,        new DispatchStrategyFormat0("DCONST_1"))
			.setDispatchStrategy(OP_FCONST_0,        new DispatchStrategyFormat0("FCONST_0"))
			.setDispatchStrategy(OP_FCONST_1,        new DispatchStrategyFormat0("FCONST_1"))
			.setDispatchStrategy(OP_FCONST_2,        new DispatchStrategyFormat0("FCONST_2"))
			.setDispatchStrategy(OP_ICONST_M1,       new DispatchStrategyFormat0("ICONST_M1"))
			.setDispatchStrategy(OP_ICONST_0,        new DispatchStrategyFormat0("ICONST_0"))
			.setDispatchStrategy(OP_ICONST_1,        new DispatchStrategyFormat0("ICONST_1"))
			.setDispatchStrategy(OP_ICONST_2,        new DispatchStrategyFormat0("ICONST_2"))
			.setDispatchStrategy(OP_ICONST_3,        new DispatchStrategyFormat0("ICONST_3"))
			.setDispatchStrategy(OP_ICONST_4,        new DispatchStrategyFormat0("ICONST_4"))
			.setDispatchStrategy(OP_ICONST_5,        new DispatchStrategyFormat0("ICONST_5"))
			.setDispatchStrategy(OP_LCONST_0,        new DispatchStrategyFormat0("LCONST_0"))
			.setDispatchStrategy(OP_LCONST_1,        new DispatchStrategyFormat0("LCONST_1"))
			.setDispatchStrategy(OP_DADD,            new DispatchStrategyFormat0("DADD"))
			.setDispatchStrategy(OP_FADD,            new DispatchStrategyFormat0("FADD"))
			.setDispatchStrategy(OP_IADD,            new DispatchStrategyFormat0("IADD"))
			.setDispatchStrategy(OP_LADD,            new DispatchStrategyFormat0("LADD"))
			.setDispatchStrategy(OP_DSUB,            new DispatchStrategyFormat0("DSUB"))
			.setDispatchStrategy(OP_FSUB,            new DispatchStrategyFormat0("FSUB"))
			.setDispatchStrategy(OP_ISUB,            new DispatchStrategyFormat0("ISUB"))
			.setDispatchStrategy(OP_LSUB,            new DispatchStrategyFormat0("LSUB"))
			.setDispatchStrategy(OP_DMUL,            new DispatchStrategyFormat0("DMUL"))
			.setDispatchStrategy(OP_FMUL,            new DispatchStrategyFormat0("FMUL"))
			.setDispatchStrategy(OP_IMUL,            new DispatchStrategyFormat0("IMUL"))
			.setDispatchStrategy(OP_LMUL,            new DispatchStrategyFormat0("LMUL"))
			.setDispatchStrategy(OP_DDIV,            new DispatchStrategyFormat0("DDIV"))
			.setDispatchStrategy(OP_FDIV,            new DispatchStrategyFormat0("FDIV"))
			.setDispatchStrategy(OP_IDIV,            new DispatchStrategyFormat0("IDIV"))
			.setDispatchStrategy(OP_LDIV,            new DispatchStrategyFormat0("LDIV"))
			.setDispatchStrategy(OP_DREM,            new DispatchStrategyFormat0("DREM"))
			.setDispatchStrategy(OP_FREM,            new DispatchStrategyFormat0("FREM"))
			.setDispatchStrategy(OP_IREM,            new DispatchStrategyFormat0("IREM"))
			.setDispatchStrategy(OP_LREM,            new DispatchStrategyFormat0("LREM"))
			.setDispatchStrategy(OP_DNEG,            new DispatchStrategyFormat0("DNEG"))
			.setDispatchStrategy(OP_FNEG,            new DispatchStrategyFormat0("FNEG"))
			.setDispatchStrategy(OP_INEG,            new DispatchStrategyFormat0("INEG"))
			.setDispatchStrategy(OP_LNEG,            new DispatchStrategyFormat0("LNEG"))
			.setDispatchStrategy(OP_ISHL,            new DispatchStrategyFormat0("ISHL"))
			.setDispatchStrategy(OP_LSHL,            new DispatchStrategyFormat0("LSHL"))
			.setDispatchStrategy(OP_ISHR,            new DispatchStrategyFormat0("ISHR"))
			.setDispatchStrategy(OP_LSHR,            new DispatchStrategyFormat0("LSHR"))
			.setDispatchStrategy(OP_IUSHR,           new DispatchStrategyFormat0("IUSHL"))
			.setDispatchStrategy(OP_LUSHR,           new DispatchStrategyFormat0("LUSHL"))
			.setDispatchStrategy(OP_IOR,             new DispatchStrategyFormat0("IOR"))
			.setDispatchStrategy(OP_LOR,             new DispatchStrategyFormat0("LOR"))
			.setDispatchStrategy(OP_IAND,            new DispatchStrategyFormat0("IAND"))
			.setDispatchStrategy(OP_LAND,            new DispatchStrategyFormat0("LAND"))
			.setDispatchStrategy(OP_IXOR,            new DispatchStrategyFormat0("IXOR"))
			.setDispatchStrategy(OP_LXOR,            new DispatchStrategyFormat0("LXOR"))
			.setDispatchStrategy(OP_IINC,            new DispatchStrategyFormat0("IINC"))
			.setDispatchStrategy(OP_DCMPG,           new DispatchStrategyFormat0("DCMPG"))
			.setDispatchStrategy(OP_DCMPL,           new DispatchStrategyFormat0("DCMPL"))
			.setDispatchStrategy(OP_FCMPG,           new DispatchStrategyFormat0("FCMPG"))
			.setDispatchStrategy(OP_FCMPL,           new DispatchStrategyFormat0("FCMPL"))
			.setDispatchStrategy(OP_LCMP,            new DispatchStrategyFormat0("LCMP"))
			.setDispatchStrategy(OP_I2D,             new DispatchStrategyFormat0("I2D"))
			.setDispatchStrategy(OP_I2F,             new DispatchStrategyFormat0("I2F"))
			.setDispatchStrategy(OP_I2L,             new DispatchStrategyFormat0("I2L"))
			.setDispatchStrategy(OP_L2D,             new DispatchStrategyFormat0("L2D"))
			.setDispatchStrategy(OP_L2F,             new DispatchStrategyFormat0("L2F"))
			.setDispatchStrategy(OP_F2D,             new DispatchStrategyFormat0("F2D"))
			.setDispatchStrategy(OP_D2F,             new DispatchStrategyFormat0("D2F"))
			.setDispatchStrategy(OP_D2I,             new DispatchStrategyFormat0("D2I"))
			.setDispatchStrategy(OP_D2L,             new DispatchStrategyFormat0("D2L"))
			.setDispatchStrategy(OP_F2I,             new DispatchStrategyFormat0("F2I"))
			.setDispatchStrategy(OP_F2L,             new DispatchStrategyFormat0("F2L"))
			.setDispatchStrategy(OP_I2B,             new DispatchStrategyFormat0("I2B"))
			.setDispatchStrategy(OP_I2C,             new DispatchStrategyFormat0("I2C"))
			.setDispatchStrategy(OP_I2S,             new DispatchStrategyFormat0("I2S"))
			.setDispatchStrategy(OP_L2I,             new DispatchStrategyFormat0("L2I"))
			.setDispatchStrategy(OP_NEW,             new DispatchStrategyFormat0("NEW"))
			.setDispatchStrategy(OP_NEWARRAY,        new DispatchStrategyFormat0("NEWARRAY"))
			.setDispatchStrategy(OP_ANEWARRAY,       new DispatchStrategyFormat0("ANEWARRAY"))
			.setDispatchStrategy(OP_MULTIANEWARRAY,  new DispatchStrategyFormat0("MULTIANEWARRAY"))
			.setDispatchStrategy(OP_GETFIELD,        new DispatchStrategyFormat0("GETFIELD"))
			.setDispatchStrategy(OP_PUTFIELD,        new DispatchStrategyFormat0("PUTFIELD"))
			.setDispatchStrategy(OP_GETSTATIC,       new DispatchStrategyFormat0("GETSTATIC"))
			.setDispatchStrategy(OP_PUTSTATIC,       new DispatchStrategyFormat0("PUTSTATIC"))
			.setDispatchStrategy(OP_AALOAD,          new DispatchStrategyFormat0("AALOAD"))
			.setDispatchStrategy(OP_CALOAD,          new DispatchStrategyFormat0("CALOAD"))
			.setDispatchStrategy(OP_BALOAD,          new DispatchStrategyFormat0("BALOAD"))
			.setDispatchStrategy(OP_DALOAD,          new DispatchStrategyFormat0("DALOAD"))
			.setDispatchStrategy(OP_FALOAD,          new DispatchStrategyFormat0("FALOAD"))
			.setDispatchStrategy(OP_IALOAD,          new DispatchStrategyFormat0("IALOAD"))
			.setDispatchStrategy(OP_LALOAD,          new DispatchStrategyFormat0("LALOAD"))
			.setDispatchStrategy(OP_SALOAD,          new DispatchStrategyFormat0("SALOAD"))
			.setDispatchStrategy(OP_AASTORE,         new DispatchStrategyFormat0("AASTORE"))
			.setDispatchStrategy(OP_CASTORE,         new DispatchStrategyFormat0("CASTORE"))
			.setDispatchStrategy(OP_BASTORE,         new DispatchStrategyFormat0("BASTORE"))
			.setDispatchStrategy(OP_DASTORE,         new DispatchStrategyFormat0("DASTORE"))
			.setDispatchStrategy(OP_FASTORE,         new DispatchStrategyFormat0("FASTORE"))
			.setDispatchStrategy(OP_IASTORE,         new DispatchStrategyFormat0("IASTORE"))
			.setDispatchStrategy(OP_LASTORE,         new DispatchStrategyFormat0("LASTORE"))
			.setDispatchStrategy(OP_SASTORE,         new DispatchStrategyFormat0("SASTORE"))
			.setDispatchStrategy(OP_ARRAYLENGTH,     new DispatchStrategyFormat0("ARRAYLENGTH"))
			.setDispatchStrategy(OP_INSTANCEOF,      new DispatchStrategyFormat0("INSTANCEOF"))
			.setDispatchStrategy(OP_CHECKCAST,       new DispatchStrategyFormat0("CHECKCAST"))
			.setDispatchStrategy(OP_POP,             new DispatchStrategyFormat0("POP"))
			.setDispatchStrategy(OP_POP2,            new DispatchStrategyFormat0("POP2"))
			.setDispatchStrategy(OP_DUP,             new DispatchStrategyFormat0("DUP"))
			.setDispatchStrategy(OP_DUP2,            new DispatchStrategyFormat0("DUP2"))
			.setDispatchStrategy(OP_DUP_X1,          new DispatchStrategyFormat0("DUP_X1"))
			.setDispatchStrategy(OP_DUP2_X1,         new DispatchStrategyFormat0("DUP2_X1"))
			.setDispatchStrategy(OP_DUP_X2,          new DispatchStrategyFormat0("DUP_X2"))
			.setDispatchStrategy(OP_DUP2_X2,         new DispatchStrategyFormat0("DUP2_X2"))
			.setDispatchStrategy(OP_SWAP,            new DispatchStrategyFormat0("SWAP"))
			.setDispatchStrategy(OP_IFEQ,            new DispatchStrategyFormat0("IFEQ"))
			.setDispatchStrategy(OP_IFGE,            new DispatchStrategyFormat0("IFGE"))
			.setDispatchStrategy(OP_IFGT,            new DispatchStrategyFormat0("IFGT"))
			.setDispatchStrategy(OP_IFLE,            new DispatchStrategyFormat0("IFLE"))
			.setDispatchStrategy(OP_IFLT,            new DispatchStrategyFormat0("IFLT"))
			.setDispatchStrategy(OP_IFNE,            new DispatchStrategyFormat0("IFNE"))
			.setDispatchStrategy(OP_IFNONNULL,       new DispatchStrategyFormat0("IFNONNULL"))
			.setDispatchStrategy(OP_IFNULL,          new DispatchStrategyFormat0("IFNULL"))
			.setDispatchStrategy(OP_IF_ICMPEQ,       new DispatchStrategyFormat0("IF_ICMPEQ"))
			.setDispatchStrategy(OP_IF_ICMPGE,       new DispatchStrategyFormat0("IF_ICMPGE"))
			.setDispatchStrategy(OP_IF_ICMPGT,       new DispatchStrategyFormat0("IF_ICMPGT"))
			.setDispatchStrategy(OP_IF_ICMPLE,       new DispatchStrategyFormat0("IF_ICMPLE"))
			.setDispatchStrategy(OP_IF_ICMPLT,       new DispatchStrategyFormat0("IF_ICMPLT"))
			.setDispatchStrategy(OP_IF_ICMPNE,       new DispatchStrategyFormat0("IF_ICMPNE"))
			.setDispatchStrategy(OP_IF_ACMPEQ,       new DispatchStrategyFormat0("IF_ACMPEQ"))
			.setDispatchStrategy(OP_IF_ACMPNE,       new DispatchStrategyFormat0("IF_ACMPNE"))
			.setDispatchStrategy(OP_TABLESWITCH,     new DispatchStrategyFormat0("TABLESWITCH"))
			.setDispatchStrategy(OP_LOOKUPSWITCH,    new DispatchStrategyFormat0("LOOKUPSWITCH"))
			.setDispatchStrategy(OP_GOTO,            new DispatchStrategyFormat0("GOTO"))
			.setDispatchStrategy(OP_GOTO_W,          new DispatchStrategyFormat0("GOTO_W"))
			.setDispatchStrategy(OP_JSR,             new DispatchStrategyFormat0("JSR"))
			.setDispatchStrategy(OP_JSR_W,           new DispatchStrategyFormat0("JSR_W"))
			.setDispatchStrategy(OP_RET,             new DispatchStrategyFormat0("RET"))
			.setDispatchStrategy(OP_INVOKEINTERFACE, new DispatchStrategyFormat0("INVOKEINTERFACE"))
			.setDispatchStrategy(OP_INVOKEVIRTUAL,   new DispatchStrategyFormat0("INVOKEVIRTUAL"))
			.setDispatchStrategy(OP_INVOKESPECIAL,   new DispatchStrategyFormat0("INVOKESPECIAL"))
			.setDispatchStrategy(OP_INVOKESTATIC,    new DispatchStrategyFormat0("INVOKESTATIC"))
			.setDispatchStrategy(OP_RETURN,          new DispatchStrategyFormat0("RETURN"))
			.setDispatchStrategy(OP_ARETURN,         new DispatchStrategyFormat0("ARETURN"))
			.setDispatchStrategy(OP_DRETURN,         new DispatchStrategyFormat0("DRETURN"))
			.setDispatchStrategy(OP_FRETURN,         new DispatchStrategyFormat0("FRETURN"))
			.setDispatchStrategy(OP_IRETURN,         new DispatchStrategyFormat0("IRETURN"))
			.setDispatchStrategy(OP_LRETURN,         new DispatchStrategyFormat0("LRETURN"))
			.setDispatchStrategy(OP_ATHROW,          new DispatchStrategyFormat0("ATHROW"))
			.setDispatchStrategy(OP_WIDE,            new DispatchStrategyFormat0("WIDE"))
			.setDispatchStrategy(OP_BREAKPOINT,      new DispatchStrategyFormat0("BREAKPOINT"))
			.setDispatchStrategy(OP_MONITORENTER,    new DispatchStrategyFormat0("MONITORENTER"))
			.setDispatchStrategy(OP_MONITOREXIT,     new DispatchStrategyFormat0("MONITOREXIT"))
			.setDispatchStrategy(OP_IMPDEP1,         new DispatchStrategyFormat0("IMPDEP1"))
			.setDispatchStrategy(OP_IMPDEP2,         new DispatchStrategyFormat0("IMPDEP2"))
		;
		
		DispatchStrategyFormat0 s = new DispatchStrategyFormat0("<RESERVED>");
		this
			.setDispatchStrategy(OP_UNDEF,                     s)
			.setDispatchStrategy(OP_ANEWARRAY_QUICK,           s)
			.setDispatchStrategy(OP_CHECKCAST_QUICK,           s)
			.setDispatchStrategy(OP_GETSTATIC_QUICK,           s)
			.setDispatchStrategy(OP_GETSTATIC2_QUICK,          s)
			.setDispatchStrategy(OP_GETFIELD_QUICK,            s)
			.setDispatchStrategy(OP_GETFIELD_QUICK_W,          s)
			.setDispatchStrategy(OP_GETFIELD2_QUICK,           s)
			.setDispatchStrategy(OP_INSTANCEOF_QUICK,          s)
			.setDispatchStrategy(OP_INVOKEINTERFACE_QUICK,     s)
			.setDispatchStrategy(OP_INVOKENONVIRTUAL_QUICK,    s)
			.setDispatchStrategy(OP_INVOKESTATIC_QUICK,        s)
			.setDispatchStrategy(OP_INVOKESUPER_QUICK,         s)
			.setDispatchStrategy(OP_INVOKEVIRTUAL_QUICK,       s)
			.setDispatchStrategy(OP_INVOKEVIRTUAL_QUICK_W,     s)
			.setDispatchStrategy(OP_INVOKEVIRTUALOBJECT_QUICK, s)
			.setDispatchStrategy(OP_LDC_QUICK,                 s)
			.setDispatchStrategy(OP_LDC_W_QUICK,               s)
			.setDispatchStrategy(OP_LDC2_W_QUICK,              s)
			.setDispatchStrategy(OP_MULTIANEWARRAY_QUICK,      s)
			.setDispatchStrategy(OP_NEW_QUICK,                 s)
			.setDispatchStrategy(OP_PUTFIELD_QUICK,            s)
			.setDispatchStrategy(OP_PUTFIELD_QUICK_W,          s)
			.setDispatchStrategy(OP_PUTFIELD2_QUICK,           s)
			.setDispatchStrategy(OP_PUTSTATIC_QUICK,           s)
			.setDispatchStrategy(OP_PUTSTATIC2_QUICK,          s)
		;
	}
	
	@Override
	public FormatterStrategy select(byte bytecode) {
		FormatterStrategy retVal = null;
		try {
			retVal = super.select(bytecode);
		} catch (Throwable e) {
			//this should never happen
		}
		return retVal;
	}
}
