package muse.apps;

import java.util.Map;

import muse.bc.Signature;
import muse.mem.Heap;
import muse.mem.Instance;
import muse.mem.Klass;
import muse.mem.Objekt;
import muse.mem.Reference;
import muse.mem.ReferenceSymbolic;
import muse.mem.StaticMethodArea;
import muse.mem.Type;


public abstract class StateFormatterGraphviz extends StateFormatterText {
	private static final String nullStyle = "[shape=invtriangle,label=\"\",regular=true]";

	BytecodeFormatter bfmt = new BytecodeFormatter();
	private int nextFreshNode;
	private boolean hasNull;
	private String currentNodePrefix;
	private String currentNodeName;
	private String currentNode;
	private String nullNodeName;
	private String nodes;
	private String edges;
	
	public StateFormatterGraphviz(String[] srcPath) {
		super(srcPath);
	}

	@Override
	public String formatHeap(Heap h) {
		String retVal = "subgraph cluster_heap { label=\"heap\" labeljust=l ";
		this.currentNodePrefix = "H";
		this.nodes = "";
		this.edges = "";
		this.nextFreshNode = 0;
		
		for (Map.Entry<Integer, Objekt> e : h.getObjects().entrySet()) {
			this.hasNull = false;
			this.nullNodeName = "";
			this.currentNodeName = this.currentNodePrefix + e.getKey(); 
			this.currentNode = this.currentNodeName + "[shape=box,label=\"" + e.getKey() + ":";
			this.currentNode += e.getValue().getType();
			this.formatObject(e.getValue());			
			this.currentNode += "\"]";
			this.nodes += currentNode;
		}
		if (this.nodes.equals(""))
			this.nodes += this.currentNodePrefix + "H[style=invis]"; //to force visualization of subgraph without nodes
		retVal += this.nodes + this.edges + "}";
		return retVal;
	}

	public String formatStaticStore(StaticMethodArea a) {
		String retVal = "subgraph cluster_staticstore { label=\"static store\" labeljust=l ";
		this.currentNodePrefix = "S";
		this.nodes = "";
		this.edges = "";

		for (Map.Entry<String, Klass> e : a.getObjects().entrySet()) {
			this.hasNull = false;
			this.nullNodeName = "";
			this.currentNodeName = this.currentNodePrefix + e.getKey(); 
			this.currentNode = this.currentNodeName + "[shape=box,label=\"" + e.getKey() + ":";
			this.currentNode += e.getValue().getType();
			this.formatObject(e.getValue());
		}

		if (this.nodes.equals("")) {
			this.nodes += this.currentNodePrefix + "H[style=invis]"; //to force visualization of subgraph without nodes
		}
		retVal += this.nodes + this.edges + "}";
		return retVal;
	}
	
	@Override
	public String formatObject(Objekt o) {
		if (o instanceof Instance) {
			Instance i = (Instance) o;
			for (Signature sig : i.getFieldSignatures()) {
				if (Type.isArray(sig.getDescriptor()) ||
					Type.isReference(sig.getDescriptor())) {
					Reference r = (Reference) i.getFieldValue(sig);
					ReferenceSymbolic sr = null;
					if (r instanceof ReferenceSymbolic) {
						sr = (ReferenceSymbolic) r;
					}
					if (r.isNull()) {
						if (!this.hasNull) { 
							this.hasNull = true;
							this.nullNodeName = this.currentNodePrefix + "N" + this.nextFreshNode; 
							this.nodes += this.nullNodeName + nullStyle;
							this.nextFreshNode++;
						}
						edges += currentNodeName + "->" + nullNodeName;  
					} else if (sr == null || sr.isInitialized()) {
						edges += currentNodeName + "->" + "H" + r.getHeapPosition(); //target always in heap
					} else {
						String dummyNodeName = this.currentNodePrefix + "I" + this.nextFreshNode;
						nodes += dummyNodeName + "[style=invis]";
						edges += this.currentNodeName + "->" + dummyNodeName;
						this.nextFreshNode++;
					}
					this.edges += "[label=\"" + sig.getName(); 
					if (sr != null) { 
						this.edges += " " + sr.getValue();
					}
					this.edges += "\"]";
				} else if (sig.getDescriptor().charAt(0) == Type.NULLREF) {
					if (!this.hasNull) { 
						this.hasNull = true;
						this.nullNodeName = this.currentNodePrefix + "N" + this.nextFreshNode; 
						this.nodes += this.nullNodeName + nullStyle;
						this.nextFreshNode++;
					}
					this.edges += this.currentNodeName + "->" + this.nullNodeName;
					this.edges += "[label=\"" + sig.getName() + "\"]";
				} else {
					this.currentNode += "\\n" + sig.getName() + " = " + i.getFieldValue(sig);
				}
			}
		} else { //is an array
			//TODO
		}
		return ""; //TODO
	}
}
