package muse.apps.mc;

import java.util.*;

public class Binding implements Cloneable {

/**
 * Maps the name to the atomic proposition.
 */
public HashMap<String, String> propositions;

/**
 * Maps the name of atomic proposition to its evaluation.
 */
public HashMap<String, Boolean> evaluation;

/**
 * New binding is empty - no propositions, no evaluation.
 */
Binding() {
    propositions = new HashMap<String, String>();
    evaluation = new HashMap<String, Boolean>(); 
}

/**
 * Add a new atomic proposition and its name to the binding.
 * 
 * @param  name  identifier of the atomic proposition
 * @param  ap    atomic proposition itself
 */
public void addProposition(String name, String ap) {
    propositions.put(name, ap);
    return;
}

/**
 * Set the evaluation of a given proposition.
 * 
 * @param  name   identifier of the atomic proposition
 * @para   eval   the new value of the atomic proposition 
 */
public void setValue(String name, boolean eval) {
    evaluation.put(name, eval);
    return;
}


public void clearEvaluation() {
    evaluation.clear();
    return;
}

/**
 * Returns current evaluation of the atomic propostion of the given name.
 */
public boolean value(String name) {
    Boolean eval = evaluation.get(name);
    if (eval == null) {
        System.err.println("Evaluation of the atomic proposition "+name+" is not known!");
        return false;
    } else {
        return eval.booleanValue();
    }
}

public boolean contains(String name) {
    return propositions.containsKey(name);
}

public boolean hasValue(String name) {
    return evaluation.containsKey(name);
}

/**
 * Return string view of the bindings.
 */
@Override
public String toString() {
    StringBuffer rs = new StringBuffer();
    Set<String> names = propositions.keySet();
    //String currentName;
    for ( String currentName : names ) {
        String currentProposition = propositions.get(currentName);
        Boolean currentValue = evaluation.get(currentName);
        rs.append(currentName + " = \""+currentProposition+"\" = "+currentValue+"\n");
    }
    return rs.toString();
}

/** Creates the clone of this Binding. */
public Binding clone() {
    Binding cloneBinding = new Binding();
    cloneBinding.propositions = new HashMap<String, String>(this.propositions);
    cloneBinding.evaluation = new HashMap<String, Boolean>(this.evaluation);
	return cloneBinding; 
}

} // Binding
