package muse.apps.mc;

import java.util.*;

/**
 * Condition represents a LTL formula without temporal operators
 * due to temporal operators are expressed in Buchi automata
 * as transitions.
 *  
 * It is represented as a list where terms and operators alternate.
 * Alternation is guaranteed by the restriction on methods.
 * First is always the term (no implicit constructor is allowed).
 */ 
public class Condition {

private LinkedList<Object> termsOpsList = null;

/** Constructor - creates list with one term. */
public Condition(Term t) {
    termsOpsList = new LinkedList<Object>();
    termsOpsList.add((Object) t);
    return;
}
/** Add the operator followed by the term to the list. */
public void add(int op, Term t) {
    termsOpsList.add((Object) new Integer(op));
    termsOpsList.add((Object) t);
    return;
}

/** Convert int to string representation of the binary operator. */
public String operator(int op) {
    String rs = "?";    
    switch(op) {
    case 1: rs="&&"; break;
    case 2: rs="||"; break;
    case 3: rs="->"; break;
    case 4: rs="<->"; break;
    }
    return rs;
}

/** Put all atomic propositions to HashSet propositions.
 * 
 * @param propositions
 */
public void getAtomicPropositions(HashSet<String> propositions) {
	Iterator<?> i = termsOpsList.iterator();
	((Term)i.next()).getAtomicPropositions(propositions);
	while (i.hasNext()) {
		i.next();
		((Term)i.next()).getAtomicPropositions(propositions);
	}
	return;
}

/**
 * Evaluate the condition under current binding.
 * The priorities of the operators are ignored.
 */
public boolean evaluate(Binding binds) {
                                 // We evaluate the first term in a special way.
    Iterator<?> i = termsOpsList.iterator();
    boolean rb = ((Term) i.next()).evaluate(binds);
    int op;
    boolean nextValue;
                                                        // And then other terms.
    while ( i.hasNext() ) {
        op = ((Integer)i.next()).intValue();
        nextValue = ((Term) i.next()).evaluate(binds);
        switch(op) {
        case 1: rb = rb && nextValue; break;
        case 2: rb = rb || nextValue; break;
        case 3: rb = !rb || nextValue; break;
        case 4: rb = rb == nextValue; break;
        } // switch
    } // while
    return rb;
} // evaluate()

/** Return string representation of the Condition. */
public String toString() {
    StringBuffer sb = new StringBuffer();
    Iterator<?> i = termsOpsList.iterator();
    boolean isTerm = true;
    while ( i.hasNext() ) {
       if (isTerm)
           sb.append(i.next().toString());
       else
           sb.append(" "+operator(((Integer)i.next()).intValue())+" ");
       isTerm = !isTerm;
    }
    return sb.toString();
}

} // Condition
