package muse.apps.mc;

import java.util.*;

public class State {

public LinkedList<StateName> name;
public LinkedList<Rule> rules;

/** Create a new state with a given name.
 */
public State(StateName n) {
    name = new LinkedList<StateName>();
    name.add(n);
    rules = new LinkedList<Rule>();
}

public void addName(StateName n) {
	name.add(n);
}

/** Add another rule for the current state.
 */
public void addRule(Rule r) {
    rules.add(r);
    return;
}

/** Put all atomic propositions to HashSet propositions.
 * 
 * @param propositions
 */
public void getAtomicPropositions(HashSet<String> propositions) {
	Iterator<?> i = rules.iterator();
	while (i.hasNext()) {
		((Rule)i.next()).getAtomicPropositions(propositions);
	}
	return;
}

/**
 * Perform a transitions possible from this state in current binding.
 */
public boolean doStep(Binding binds, HashSet<StateName> newStates){
    boolean status = false;
                                                             // Check all rules.
    for ( Rule r : rules ) {
        status = r.doStep(binds, newStates) || status;
    }
    return status;
}

/** Own view of the state.
 */
public String toString() {
    return name.toString();
}

public boolean isInit() {
	for (StateName n : name)
		if (n.isInit())
			return true;
	return false;
}

public boolean isAccept() {
	for (StateName n : name)
		if (n.isAccept())
			return true;
	return false;
}

public boolean isAcceptAll() {
	for (StateName n : name)
		if (n.isAcceptAll())
			return true;
	return false;
}

} // State
