package muse.apps.mc;

import java.util.HashSet;

public class Term {

/** Term types:
 * 1 = atomic proposition
 * 2 = logic value (true/false) 
 * 3 = encapsulated condition
 * 4 = negated term
 */ 
public int       type = 0;
public String    atomicProposition = "";                              // Type 1.
public boolean   logicValue = false;                                  // Type 2.
public Condition condition = null;                                    // Type 3.
public Term      term = null;                                         // Type 4.

/** Constructor for atomic propositions. */
public Term(String ap) {
    type=1;
    atomicProposition = ap;
    return;
}
/** Constructor for logic values. */
public Term(int lv) {
    type=2;
    if (lv==0) 
        logicValue = false;
    else 
        logicValue = true;
    return;
}
/** Constructor for encapsulated condition. */
public Term(Condition c) {
    type=3;
    condition = c;
    return;
}
/** Constructor for nagative term. */
public Term(Term t) {
    type=4;
    term = t;
    return;
}

/** Put all atomic propositions to HashSet propositions.
 * 
 * @param propositions
 */
public void getAtomicPropositions(HashSet<String> propositions) {
    switch(type) {
    case 1: propositions.add((String)atomicProposition);   break;
    case 2: ;                                              break;
    case 3: condition.getAtomicPropositions(propositions); break;
    case 4: term.getAtomicPropositions(propositions);;     break;
    }
	return;
}

/** Evaluate the term under current binding. */
public boolean evaluate(Binding binds) {
    boolean rb = false;
    switch(type) {
    case 1: rb = binds.value(atomicProposition); break;
    case 2: rb = logicValue;                     break;
    case 3: rb = condition.evaluate(binds);      break;
    case 4: rb = !term.evaluate(binds);          break;
    }
    return rb;
}

/** Return string representation of the term.
 */
public String toString() {
    String rs = "?";
    switch(type) {
    case 1: rs=atomicProposition;            break;
    case 2: rs=logicValue ? "1" : "0";       break;
    case 3: rs="("+condition.toString()+")"; break;
    case 4: rs="!"+term.toString();          break;
    }
    return rs;
}

} // Term
