package muse.apps.run;

import java.io.IOException;
import java.io.PrintStream;

public class IO {
	/**
	 * Reads a line from <code>System.in</code>, displaying the 
	 * interaction on a number of <code>PrintStream</code>s.
	 *  
	 * @param ps a <code>PrintStream[]</code>; the state 
     *           will be displayed on all the <code>PrintStream</code>s 
     *           in <code>ps</code>.
	 * @param prompt a <code>String</code> which will be displayed 
	 *               as input prompt.
	 * @return a <code>String</code> representing the input line, 
	 *         without the trailing end line, or <code>null</code> 
	 *         if some <code>IOException</code> arises during 
	 *         input scanning.
	 */
	public static String readln(PrintStream[] ps, String prompt) {
		if (ps != null && prompt != null) {
			IO.print(ps, prompt);
		}
	    String retVal = IO.readln();
	    for (PrintStream p : ps) {
	    	//TODO this avoids duplication of input on System.out, but is not much robust. Find a way to turn off echo on System.out when scanning input and remove the guard.
	    	if (p != null && p != System.out) { 
	    		p.println(retVal);
	    	}
	    }
    	if (retVal == null) { 
    		retVal = "";
    	}

		return retVal;
	}

	/**
	 * Reads a line from <code>System.in</code>; as a side effect, 
	 * the interaction is displayed on <code>System.in</code>.
	 * 
	 * @return
	 */
	private static String readln() { 
	    String result = "";
	    boolean done = false;
	
	    while (!done) {
	    	int charAsInt = -1; //To keep the compiler happy
		    try {
		        charAsInt = System.in.read(); 
		    } catch(IOException e) {
		    	result = null;
		    	break;
		    } 
	    	
	        char nextChar = (char) charAsInt;
	        if (nextChar == '\n') {
	           done = true;
	        } else if (nextChar == '\r') {
	        	; //discard
	        } else { 
	           result += nextChar;
	        }
	    }
	    
	    return result;
	}

	public static void println(PrintStream[] ps, String txt) {
		for (PrintStream p : ps) {
			if (p != null) {
				p.println(txt);
			}
		}
	}

	public static void println(PrintStream[] ps) {
		for (PrintStream p : ps) {
			if (p != null) {
				p.println();
			}
		}
	}

	public static void printException(PrintStream[] ps, Throwable e) {
		for (PrintStream p : ps) {
			if (p != null) {
				e.printStackTrace(p);
			}
		}
	}

	public static void print(PrintStream[] ps, String txt) {
		for (PrintStream p : ps) {
			if (p != null) {
				p.print(txt);
			}
		}
	}
}
