package muse.bc;

import java.util.ArrayList;
import java.util.List;

import muse.mem.Value;


public class ClassFileArray extends ClassFile {
	//TODO by now clone is treated as a native method; implement it.
	private static int METHOD_CLONE_INDEX = 99;
	private static String METHOD_CLONE_NAME = "clone";
	private static String METHOD_CLONE_SIGNATURE = "()Ljava/lang/Object;";
	
	private String className;
	
	public ClassFileArray(String className) { this.className = className; }

	@Override
	public String getClassName() {
		return this.className;
	}

	@Override
	public String getClassSignature(int classRef) throws InvalidIndexException {
		throw new InvalidIndexException();
	}

	@Override
	public int getCodeLength(Signature methodSignature)
			throws MethodNotFoundException, MethodCodeNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException(); //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public ExceptionTable getExceptionTable(Signature methodSignature)
			throws MethodNotFoundException, MethodCodeNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException(); //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public Signature getFieldSignature(int fieldRef)
			throws InvalidIndexException {
		throw new InvalidIndexException();
	}

	@Override
	public Signature[] getFieldsNonStatic() {
		return new Signature[0];
	}

	@Override
	public Signature[] getFieldsStatic() {
		return new Signature[0];
	}

	@Override
	public Signature getInterfaceMethodSignature(int methodRef)
			throws InvalidIndexException {
		throw new InvalidIndexException();
	}

	@Override
	public int getLocalVariableLength(Signature methodSignature)
			throws MethodNotFoundException, MethodCodeNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException(); //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public LocalVariableTable getLocalVariableTable(
			Signature methodSignature) throws MethodNotFoundException,
			MethodCodeNotFoundException, LocalVariableInfoNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException(); //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public byte[] getMethodCodeBySignature(Signature methodSignature)
			throws MethodNotFoundException, MethodCodeNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException(); //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public Signature getMethodSignature(int methodRef)
			throws InvalidIndexException {
		if (methodRef == METHOD_CLONE_INDEX)
			return new Signature(this.className, METHOD_CLONE_SIGNATURE, METHOD_CLONE_NAME);
		throw new InvalidIndexException();
	}

	@Override
	public String getSuperClassName() {
		return "java/lang/Object";
	}

	@Override
	public List<String> getSuperInterfaceNames() {
		List<String> retVal = new ArrayList<String>(2);
		retVal.add("java/lang/Cloneable");
		retVal.add("java/io/Serializable");
		return retVal;
	}

	@Override
	public Value getValueFromConstantPool(int index)
			throws InvalidIndexException {
		throw new InvalidIndexException();
	}

	@Override
	public boolean hasMethodDeclaration(Signature methodSignature) {
		if (isMethodClone(methodSignature))
			return true;
		return false;
	}

	@Override
	public boolean hasMethodImplementation(Signature methodSignature) {
		if (isMethodClone(methodSignature))
			return true;
		return false;
	}

	@Override
	public boolean isAbstract() {
		return false;
	}

	@Override
	public boolean isInterface() {
		return false;
	}

	@Override
	public boolean isMethodAbstract(Signature methodSignature)
			throws MethodNotFoundException {
		if (isMethodClone(methodSignature))
			return false;
		throw new MethodNotFoundException();
	}

	@Override
	public boolean isMethodNative(Signature methodSignature)
			throws MethodNotFoundException {
		if (isMethodClone(methodSignature))
			return true; //TODO return code
		throw new MethodNotFoundException();
	}

	@Override
	public boolean isMethodStatic(Signature methodSignature)
			throws MethodNotFoundException {
		if (isMethodClone(methodSignature))
			return false;
		throw new MethodNotFoundException();
	}

	@Override
	public boolean isPublic() {
		return true;
	}

	@Override
	public boolean isSuperInvoke() {
		//TODO
		return false;
	}

	@Override
	public boolean hasFieldDeclaration(Signature fieldSignature) {
		return false;
	}
	
	private boolean isMethodClone(Signature methodSignature) {
		return (methodSignature.getName().equals(METHOD_CLONE_NAME) &&
		methodSignature.getDescriptor().equals(METHOD_CLONE_SIGNATURE));
	}

	@Override
	public LineNumberTable getLineNumberTable(Signature methodSignature) throws MethodNotFoundException, MethodCodeNotFoundException, LocalVariableInfoNotFoundException {
		if (isMethodClone(methodSignature))
			throw new MethodCodeNotFoundException();
		throw new MethodNotFoundException();
	}

	@Override
	public Value fieldConstantValue(Signature fieldSignature) throws FieldNotFoundException, AttributeNotFoundException {
		throw new FieldNotFoundException();
	}

	@Override
	public boolean hasFieldConstantValue(Signature fieldSignature) throws FieldNotFoundException {
		throw new FieldNotFoundException();
	}

	@Override
	public boolean isFieldFinal(Signature fieldSignature) throws FieldNotFoundException {
		throw new FieldNotFoundException();
	}
}
