package muse.bc;

import java.util.Map;
import java.util.HashMap;

import muse.jvm.Environment;


/**
 * Class that permits to retrieve information from a number of classfiles. 
 * We take assumption that there are not two classfiles with same name.
 */ 
public class ClassFileInterface {
	private Environment env;
	private ClassFileFactory f;
    private Map<String, ClassFile> jcrArray;
    
    public ClassFileInterface(Environment env, ClassFileFactory f) {
    	this.env = env;
    	this.f = f;
    }
    
    public Environment getEnvironment() {
    	return this.env;
    }
    
    /**
     * Given a class name returns the correspondent classFile.
     * @param className the searched class.
     * @return the classFile structure of the correspondent class.
     * @throws ClassFileNotFoundException when the class file cannot be 
     * found in the classpath.
     */
    public ClassFile getClassFile(String className) {
    	ClassFile retVal = null;
        //lazily initializes jcrArray
        if (jcrArray == null) {
            jcrArray = new HashMap<String, ClassFile>();
        }
        
        //if the class file is not already in jcrArray, adds it
        if (!jcrArray.containsKey(className)) {
    		ClassFile tempCF = this.f.newClassFile(className);
    		jcrArray.put(className, tempCF);
        }
        
        retVal = jcrArray.get(className);
        if (retVal == null) {
        	throw new ClassFileNotFoundException();
        }
        return retVal; 
    }
}