package muse.dec;

import java.util.SortedSet;
import java.util.Stack;

import muse.algo.Util;
import muse.mem.Expression;
import muse.mem.Primitive;
import muse.mem.Simplex;
import muse.tree.DecisionAlternativeBoolean;
import muse.tree.DecisionAlternativeComparison;
import muse.tree.DecisionAlternativeSwitch;


/**
 * Class implementing a "no assumption" simplifier, i.e., a 
 * simplifier that considers all clauses as satisfiable.
 * 
 * @author pietro
 *
 */
public class DecisionProcedureAlwSat extends DecisionProcedure {
	private Stack<Primitive> cstack;

	public DecisionProcedureAlwSat() {
		this.cstack = new Stack<Primitive>();
	}

	@Override
	public void pushClause(Primitive c) {
		long start = System.currentTimeMillis();
		this.cstack.push(c);
        this.time += System.currentTimeMillis() - start;
	}

	@Override
	public void resetClause()  {
		long start = System.currentTimeMillis();
		this.cstack.clear();
        this.time += System.currentTimeMillis() - start;
	}

	@Override
	public Primitive getAccumulatedClause() {
		long start = System.currentTimeMillis();
		Primitive retval = null;
		
		if (this.cstack.empty()) {
			return new Simplex(true);
		}

		boolean first = true;
		for (Primitive c : this.cstack) {
			if (first) {
				retval = c;
			} else {
				retval = retval.and(c);
			}
			first = false;
		}
        this.time += System.currentTimeMillis() - start;
		return retval;
	}

	@Override
	public boolean isSat(Expression exp) {
		return true;
	}

	@Override
	public void simplifyDecision(Expression exp, SortedSet<DecisionAlternativeBoolean> upd) throws DecisionException {
		long start = System.currentTimeMillis();
        upd.add(DecisionAlternativeBoolean.TT);
        upd.add(DecisionAlternativeBoolean.FF);
        this.time += System.currentTimeMillis() - start;
	}

	@Override
	public void simplifyComparison(Primitive val1, Primitive val2, SortedSet<DecisionAlternativeComparison> upd) throws DecisionException {
		long start = System.currentTimeMillis();
        upd.add(DecisionAlternativeComparison.GT);
        upd.add(DecisionAlternativeComparison.EQ);
        upd.add(DecisionAlternativeComparison.LT);
        this.time += System.currentTimeMillis() - start;
	}

	@Override
	public void simplifySwitch(Primitive val, boolean isTableSwitch, int low, int high, int tableStart, int tableEnd, byte[] code, SortedSet<DecisionAlternativeSwitch> upd) throws DecisionException {
		long start = System.currentTimeMillis();
		byte[] t = new byte[4];
        if (isTableSwitch) {
			for (int i = low; i < high; i++) {
				upd.add(new DecisionAlternativeSwitch(i));
			}
        } else {
			for (int k = tableStart; k < tableEnd; k += 8) {
				for (int i = 0; i < 4; i++) {
					t[i] = code[k + i];
				}
				int match = Util.byteCat(t[0], t[1], t[2], t[3]);
				upd.add(new DecisionAlternativeSwitch(match));
			}
        }
        this.time += System.currentTimeMillis() - start;
	}
}
