package muse.dec;

import java.io.IOException;
import java.io.PrintStream;
import java.util.SortedSet;

import muse.algo.Util;
import muse.apps.run.IO;
import muse.mem.Expression;
import muse.mem.Operator;
import muse.mem.Primitive;
import muse.mem.Simplex;
import muse.tree.DecisionAlternativeBoolean;
import muse.tree.DecisionAlternativeComparison;
import muse.tree.DecisionAlternativeSetFactory;
import muse.tree.DecisionAlternativeSwitch;


public abstract class DecisionProcedureExternal extends DecisionProcedure {
	private final DecisionProcedureAlwSat bs = new DecisionProcedureAlwSat();
	private final String className = this.getClass().getName().substring(this.getClass().getName().lastIndexOf('.') + 1);
	protected final PrintStream[] out;
	protected DecisionProcedureExternalInterface si;

	public DecisionProcedureExternal(PrintStream[] out) {
		super();
		if (out == null) {
			this.out = new PrintStream[0];
		} else {
			this.out = out;
		}
	}

	public DecisionProcedureExternal(DecisionAlternativeSetFactory c, PrintStream[] out) {
		super(c);
		if (out == null) {
			this.out = new PrintStream[0];
		} else {
			this.out = out;
		}
	}

	@Override
	public void pushClause(Primitive c) throws DecisionException {
		long start = System.currentTimeMillis();
	    try {
	        if (si.isWorking()) {    
				si.putCondition(c);
				si.appendCondToAssumptionsAs(true);
				bs.pushClause(c);
				IO.println(this.out, this.className + ": Pushed: " + c + ".");
	        } else {
	        	throw new DecisionException();
	        }
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public void resetClause() throws DecisionException {
		long start = System.currentTimeMillis();
	    try {
	        if (si.isWorking()) {
	        	si.clear();
	        	bs.resetClause();
	            IO.println(this.out, this.className + ": Cleared.");
	        } else
	        	throw new DecisionException();
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public Primitive getAccumulatedClause() {
		return bs.getAccumulatedClause();
	}

	@Override
	public void simplifyDecision(Expression exp, SortedSet<DecisionAlternativeBoolean> upd) throws DecisionException {
		long start = System.currentTimeMillis();
	    try {
	        if (si.isWorking()) {    
	            IO.print(this.out, this.className + ": Simplified: " + this.getAccumulatedClause() + " |- " + exp + ". Result: ");
	        	si.putCondition(exp);
	            if (si.curAssumptionsImplyCondIs(true)) { 
	            	upd.add(DecisionAlternativeBoolean.TT);
	            } else if (si.curAssumptionsImplyCondIs(false)) { 
	            	upd.add(DecisionAlternativeBoolean.FF);
	            } else {
	            	upd.add(DecisionAlternativeBoolean.TT);
	            	upd.add(DecisionAlternativeBoolean.FF);	            	
	            }
	            si.discardFormula();
	            IO.println(this.out, upd.toString());
	        } else {
	        	throw new DecisionException();
	        }
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public void simplifyComparison(Primitive val1, Primitive val2, SortedSet<DecisionAlternativeComparison> upd) throws DecisionException {
		Expression expGT = new Expression(val1, Operator.GT, val2);
		Expression expEQ = new Expression(val1, Operator.EQ, val2);
		Expression expLT = new Expression(val1, Operator.LT, val2);
	
		long start = System.currentTimeMillis();
		try {
			if (si.isWorking()) {
	            IO.print(this.out, this.className + ": Simplified: " + this.getAccumulatedClause() + " |- " + val1 + ">=<" + val2 + ". Result: ");
	
	            //tries to disprove every possible outcome, and  
				//adds to retval all the possible outcomes that
				//cannot be disproved
				si.putCondition(expGT);
	            if (!si.curAssumptionsImplyCondIs(false)) {
	            	upd.add(DecisionAlternativeComparison.GT);
	            }
	        	si.discardFormula();
				si.putCondition(expEQ);
	            if (!si.curAssumptionsImplyCondIs(false)) { 
	            	upd.add(DecisionAlternativeComparison.EQ);
	            }
	        	si.discardFormula();
	            si.putCondition(expLT);
			    if (!si.curAssumptionsImplyCondIs(false)) { 
			    	upd.add(DecisionAlternativeComparison.LT);
			    }
	            si.discardFormula();                
	            IO.println(this.out, upd.toString());
	        } else
	        	throw new DecisionException();
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public void simplifySwitch(Primitive val, boolean isTableSwitch, int low, int high, int tableStart, int tableEnd, byte[] code, SortedSet<DecisionAlternativeSwitch> upd) throws DecisionException {
		long start = System.currentTimeMillis();
		try {
			if (si.isWorking()) {
				IO.print(this.out, this.className + ": Simplified switch. Result: "); //TODO improve switch simplification feedback.
				
				Expression exp;
				if (isTableSwitch) {
					for (int i = low; i <= high; i++) {
						exp = new Expression(val, Operator.EQ, new Simplex(i));
						si.putCondition(exp);
			            if (!si.curAssumptionsImplyCondIs(false)) {
			            	upd.add(new DecisionAlternativeSwitch(i));
			            }
			            si.discardFormula();
					}
					exp = new Expression(new Expression(val, Operator.LT, new Simplex(low)), Operator.OR, 
							new Expression(val, Operator.GT, new Simplex(high)));
					si.putCondition(exp);
		            if (!si.curAssumptionsImplyCondIs(false)) { 
		            	upd.add(DecisionAlternativeSwitch.alternativeDefault());
		            }
		            si.discardFormula();
				} else {  //(!isTableSwitch)
					byte[] t = new byte[4];
					int match = 0; //initialization just to avoid compiler complaints 
					Expression expNeg = null;
					for (int k = tableStart; k < tableEnd; k += 8) {
						for (int i = 0; i < 4; i++) {
							t[i] = code[k + i];
						}
						match = Util.byteCat(t[0], t[1], t[2], t[3]);
						exp = new Expression(val, Operator.EQ, new Simplex(match));
						si.putCondition(exp);
			            if (!si.curAssumptionsImplyCondIs(false)) { 
			            	upd.add(new DecisionAlternativeSwitch(match));
	
			            	//adds clause to expNeg
				            if (expNeg == null) {
								expNeg = new Expression(Operator.NOT, exp);
				            } else {
								expNeg = new Expression(expNeg, Operator.AND, new Expression(Operator.NOT, exp));
				            }
			            }
			            si.discardFormula();			            
					}
					si.putCondition(expNeg);
		            if (!si.curAssumptionsImplyCondIs(false)) { 
		            	upd.add(DecisionAlternativeSwitch.alternativeDefault());
		            }
		            si.discardFormula();					
				}
				
				IO.println(this.out, upd.toString());
			} else {
				throw new DecisionException();
			}
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public boolean isSat(Expression exp) throws DecisionException {
		boolean retVal;
		long start = System.currentTimeMillis();
	    try {
	        if (si.isWorking()) {    
	            IO.print(this.out, this.className + ": Simplified: " + this.getAccumulatedClause() + " |-SAT- " + exp + ". Result: ");
	        	si.putCondition(exp);
	        	//exp is satisfiable iff it is not contradictory
	        	retVal = ! (si.curAssumptionsImplyCondIs(false)); 
	            si.discardFormula();
	            IO.println(this.out, Boolean.toString(retVal));
	            return retVal;
	        } else {
	        	throw new DecisionException();
	        }
		} catch (DecisionExternalProtocolException e) {
			throw new DecisionException();
		} catch (IOException e) {
			throw new DecisionException();
		} finally {
			this.time += System.currentTimeMillis() - start;
		}
	}

	@Override
	public void quit() throws DecisionException {
		long start = System.currentTimeMillis();
		if (si.isWorking()) {
			try {
				si.quit();
			} catch (DecisionExternalProtocolException e) {
				si.fail();
				throw new DecisionException();
			} catch (IOException e) {
				si.fail();
				throw new DecisionException();
			} finally {
				this.time += System.currentTimeMillis() - start;
			}
		}
	}

}