package muse.dec;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import muse.mem.Primitive;
import muse.mem.Term;


public abstract class DecisionProcedureExternalInterface {
    protected ExpressionMangler m = new ExpressionMangler();

    protected static class ExpressionMangler {
    	private ArrayList<String> symToExpr;
    	private HashMap<String, Integer> exprToSym;
    	private int symId;
    	
    	ExpressionMangler() { 
        	this.exprToSym = new HashMap<String, Integer>();
        	this.symToExpr = new ArrayList<String>();
        	this.symId = 0;
    	}

    	public Term mangle(Primitive p) {
    		String primToString = p.toString();
        	Integer retVal = this.exprToSym.get(primToString);
        	if (retVal == null) {
        		retVal = new Integer(this.symId);
        		this.exprToSym.put(primToString, retVal);
        		this.symToExpr.add(primToString);
        		symId++;
        	}
        	return new Term(p.getType(), symbol(retVal));
        }
        
        String symbol(int i) {
        	return "{X" + i + "}";
        }
    }
    
	public abstract boolean isWorking();

	/**
	 * Puts a condition without assuming it.
	 * 
	 * @param cond The condition to put
	 * @throws DecisionExternalProtocolException
	 * @throws IOException
	 */
	public abstract void putCondition(Primitive cond)
			throws DecisionExternalProtocolException, IOException;

	/**
	 * Verifies whether the current assumption implies a 
	 * (perhaps negated) condition put with a previous call 
	 * to putCondition.
	 * 
	 * @param value if false the condition is negated before verification
	 * @return true if it can be proved that the current assumption implies the condition,
	 * false otherwise (i.e., no logical relationship between the two is provable) 
	 * @throws DecisionExternalProtocolException
	 * @throws IOException
	 */
	public abstract boolean curAssumptionsImplyCondIs(boolean value)
			throws DecisionExternalProtocolException, IOException;

	/**
	 * Updates the state of the simplifier client, by appending 
	 * to the current assumption a condition previously put 
	 * with a call to putCondition. 
	 * 
	 * @param value When false the condition is negated before is 
	 * appended to the current assumption.
	 * @throws DecisionExternalProtocolException
	 * @throws IOException
	 */
	public abstract void appendCondToAssumptionsAs(boolean value)
			throws DecisionExternalProtocolException, IOException;

	public abstract void discardFormula() throws DecisionExternalProtocolException,
			IOException;

	/**
	 * Deletes the whole assumption set.
	 * @throws DecisionExternalProtocolException
	 * @throws IOException
	 */
	public abstract void clear() throws DecisionExternalProtocolException,
			IOException;

	/**
	 * Quits the simplifier.
	 * @throws DecisionExternalProtocolException
	 * @throws IOException
	 */
	public abstract void quit() throws DecisionExternalProtocolException, IOException;

	public abstract void fail();

}