package muse.mem;

import java.util.Map;

/**
 * Class denoting a reference to an array not yet available 
 * in the heap. It is meant to be stored in an <code>Array</code> to
 * lazily create layers of multidimensional arrays with
 * symbolic length upon access. It must never escape to the 
 * operand stack.
 * 
 * @author pietro
 */
public class ArrayConstraint extends Reference {
	private Primitive[] length;
	
	public ArrayConstraint(Primitive[] length) {
		this.length = length;
	}
	
	public Primitive getLength() {
		return this.length[0];
	}
	
	public ArrayConstraint next() {
		if (this.length.length > 1) {
			Primitive[] newLength = new Primitive[this.length.length - 1];
			boolean first = true;
			int i = 0;
			for (Primitive p : this.length) {
				if (first) {
					first = false;
				} else {
					newLength[i] = p;
				}
				i++;
			}
			return new ArrayConstraint(newLength);
		} else {
			return null;
		}
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof ArrayConstraint) {
			ArrayConstraint c = (ArrayConstraint) o;
			if (c.length.length == this.length.length) {
				boolean equals = true;
				for (int i = 0; i < this.length.length && equals; i++) {
					if (this.length[i] == null && c.length[i] != null) {
						equals = false;
					} else if (this.length[i] != null && c.length[i] == null) {
						equals = false;
					} else if (this.length[i] != null && c.length[i] != null) {
						equals = equals && this.length[i].equals(c.length[i]);
					}
				}
				return equals;
			}
		}
		return false;
	}

	@Override
	public Object getValueForNative() throws ValueDoesNotSupportNativeException {
		throw new ValueDoesNotSupportNativeException();
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		if (this.length == null) {
			return;
		} else {
			for (Primitive p : this.length) {
				if (p != null) {
					p.refine(refs);
				}
			}
		}
	}

	@Override
	public String toString() {
		String retVal = "{R<";
		for (Primitive p : this.length) {
			if (p == null) {
				retVal += "*";
			} else {
				retVal += p.toString();
			}
		}
		retVal += ">}";
		return retVal;
	}

	@Override
	public int getHeapPosition() {
		return 0;
	}

	@Override
	public boolean isNull() {
		return false;
	}

	@Override
	public boolean isSymbolic() {
		// TODO Auto-generated method stub
		return false;
	}
}
