package muse.mem;

import java.util.Map;

/**
 * Class representing a default value with unknown type. 
 * Used to initialize the local variable memory area to
 * cope with absence of type information.
 * 
 * @author pietro
 */
public class DefaultValue extends Value {
	
    private static DefaultValue instance = new DefaultValue();
	
	private DefaultValue() { 
		this.setType(Type.UNKNOWN); 
	}
	
    public static DefaultValue getInstance() {
        return DefaultValue.instance;
    }
	
	@Override
	public boolean equals(Object o) {
		return (o == instance);
	}
	
	@Override
	public DefaultValue clone() {
		return DefaultValue.getInstance();
	}
	
	@Override
	public String toString() {
		return "<DEFAULT>";
	}

	@Override
	public Object getValueForNative() throws ValueDoesNotSupportNativeException {
		throw new ValueDoesNotSupportNativeException();
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		return; //does nothing
	}
}
