package muse.mem;

import java.io.IOException;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import muse.dec.DecisionExternalProtocolException;


/**
 * Class that represent a unary or binary expression.
 */
public class Expression extends Primitive {
    /*Fields*/
    private String operation;
    private Primitive firstOp;
    private Primitive secondOp;

	@Override
	public Object getValueForNative() throws ValueDoesNotSupportNativeException {
		throw new ValueDoesNotSupportNativeException();
	}

	/**
     *Given the parameters, build a binary expression
     */
    public Expression(Primitive firstOperand, String operator, Primitive secondOperand) {
        firstOp = firstOperand;
        secondOp = secondOperand;
        operation = operator;
        this.setType(this.dominance());
    }
    
    /**
     *Given the parameters, build a unary expression with operator
     */
    public Expression(String operator, Primitive operand) {
        firstOp = null;
        secondOp = operand;
        operation = operator;
        this.setType(this.dominance());
    }
    
    /**
     *Returns the first operand of expression if exists
     */
    public Primitive getFirstOperand()
    {
        return(firstOp);
    }
    /**
     *Returns the operator of expression if exists
     */
    public String getOperator() {
        return(operation);
    }
    
    /**
     * Returns the second operand of expression if exists
     */
    public Primitive getSecondOperand() {
        return(secondOp);
    }
    
    public Primitive normalize() {
        if (secondOp == null) { 
        	return null; //all exp must have secondOp!=null
        }
        
        //recursively normalizes operands
        if (secondOp instanceof Expression) {
        	secondOp = ((Expression) secondOp).normalize();
        }
        
        boolean binary = (firstOp != null);
        if (binary) {
            if (firstOp instanceof Expression) {
            	firstOp = ((Expression) firstOp).normalize();
            }
        }
        
        if ((firstOp instanceof Simplex || !binary) && (secondOp instanceof Simplex)) {
        	//both operands are Simplex: performs the operation
        	int sOpType = secondOp.getType();
        	Object sOp = ((Simplex)secondOp).getActualValue();
        	if (!binary) {
        		if (operation == Operator.NOT) {
        			//return this;
        			return new Simplex(!((Boolean)sOp).booleanValue());
        			//if(sOpType==Type.INT){return(new Simplex(!((Integer)sOp).intValue()));}
        			//if(sOpType==Type.LONG){return(new Simplex(!((Long)sOp).longValue()));}
        		} else if (operation == Operator.NEG) {
        			if(sOpType==Type.DOUBLE){return(new Simplex(-((Double)sOp).doubleValue()));}
        			if(sOpType==Type.FLOAT){return(new Simplex(-((Float)sOp).floatValue()));}
        			if(sOpType==Type.LONG){return(new Simplex(-((Long)sOp).longValue()));}
        			if(sOpType==Type.INT){return(new Simplex(-((Integer)sOp).intValue()));}
        			if(sOpType==Type.SHORT){return(new Simplex(-((Short)sOp).shortValue()));}
        			if(sOpType==Type.BYTE){return(new Simplex(-((Byte)sOp).byteValue()));}
        			if(sOpType==Type.CHAR){return(new Simplex(-((Character)sOp).charValue()));}
        		}
        	} else {
        		int fOpType=firstOp.getType();
        		Object fOp=((Simplex)firstOp).getActualValue();
        		if (operation == Operator.ADD){
        			if(fOpType == Type.DOUBLE){
        				if(sOpType == Type.DOUBLE){return(new Simplex(((Double)fOp).doubleValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Double)fOp).doubleValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Double)fOp).doubleValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Double)fOp).doubleValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Double)fOp).doubleValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Double)fOp).doubleValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Double)fOp).doubleValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.FLOAT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Float)fOp).floatValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Float)fOp).floatValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Float)fOp).floatValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Float)fOp).floatValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Float)fOp).floatValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Float)fOp).floatValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Float)fOp).floatValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.LONG) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Long)fOp).longValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Long)fOp).longValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Long)fOp).longValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Long)fOp).longValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Long)fOp).longValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Long)fOp).longValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.INT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Integer)fOp).intValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Integer)fOp).intValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Integer)fOp).intValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Integer)fOp).intValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Integer)fOp).intValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Integer)fOp).intValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.SHORT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Short)fOp).shortValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Short)fOp).shortValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Short)fOp).shortValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Short)fOp).shortValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Short)fOp).shortValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Short)fOp).shortValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Short)fOp).shortValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.BYTE) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Byte)fOp).byteValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Byte)fOp).byteValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Byte)fOp).byteValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Byte)fOp).byteValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Byte)fOp).byteValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Byte)fOp).byteValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Byte)fOp).byteValue()+((Character)sOp).charValue()));}
        			} else if(fOpType==Type.CHAR) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Character)fOp).charValue()+((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Character)fOp).charValue()+((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Character)fOp).charValue()+((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Character)fOp).charValue()+((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Character)fOp).charValue()+((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Character)fOp).charValue()+((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Character)fOp).charValue()+((Character)sOp).charValue()));}
        			}
        		} else if(operation==Operator.MUL){
        			if(fOpType==Type.DOUBLE){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Double)fOp).doubleValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Double)fOp).doubleValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Double)fOp).doubleValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Double)fOp).doubleValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Double)fOp).doubleValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Double)fOp).doubleValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Double)fOp).doubleValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.FLOAT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Float)fOp).floatValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Float)fOp).floatValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Float)fOp).floatValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Float)fOp).floatValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Float)fOp).floatValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Float)fOp).floatValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Float)fOp).floatValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.LONG) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Long)fOp).longValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Long)fOp).longValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Long)fOp).longValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Long)fOp).longValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Long)fOp).longValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Long)fOp).longValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.INT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Integer)fOp).intValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Integer)fOp).intValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Integer)fOp).intValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Integer)fOp).intValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Integer)fOp).intValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Integer)fOp).intValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.SHORT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Short)fOp).shortValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Short)fOp).shortValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Short)fOp).shortValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Short)fOp).shortValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Short)fOp).shortValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Short)fOp).shortValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Short)fOp).shortValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.BYTE) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Byte)fOp).byteValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Byte)fOp).byteValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Byte)fOp).byteValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Byte)fOp).byteValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Byte)fOp).byteValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Byte)fOp).byteValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Byte)fOp).byteValue()*((Character)sOp).charValue()));}
        			} else if(fOpType==Type.CHAR) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Character)fOp).charValue()*((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Character)fOp).charValue()*((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Character)fOp).charValue()*((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Character)fOp).charValue()*((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Character)fOp).charValue()*((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Character)fOp).charValue()*((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Character)fOp).charValue()*((Character)sOp).charValue()));}
        			}
        		} else if(operation==Operator.SUB) {
        			if(fOpType==Type.DOUBLE){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Double)fOp).doubleValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Double)fOp).doubleValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Double)fOp).doubleValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Double)fOp).doubleValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Double)fOp).doubleValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Double)fOp).doubleValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Double)fOp).doubleValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.FLOAT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Float)fOp).floatValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Float)fOp).floatValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Float)fOp).floatValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Float)fOp).floatValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Float)fOp).floatValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Float)fOp).floatValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Float)fOp).floatValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.LONG) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Long)fOp).longValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Long)fOp).longValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Long)fOp).longValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Long)fOp).longValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Long)fOp).longValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Long)fOp).longValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.INT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Integer)fOp).intValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Integer)fOp).intValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Integer)fOp).intValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Integer)fOp).intValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Integer)fOp).intValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Integer)fOp).intValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.SHORT) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Short)fOp).shortValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Short)fOp).shortValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Short)fOp).shortValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Short)fOp).shortValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Short)fOp).shortValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Short)fOp).shortValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Short)fOp).shortValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.BYTE) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Byte)fOp).byteValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Byte)fOp).byteValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Byte)fOp).byteValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Byte)fOp).byteValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Byte)fOp).byteValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Byte)fOp).byteValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Byte)fOp).byteValue()-((Character)sOp).charValue()));}
        			} else if(fOpType==Type.CHAR) {
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Character)fOp).charValue()-((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Character)fOp).charValue()-((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Character)fOp).charValue()-((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Character)fOp).charValue()-((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Character)fOp).charValue()-((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Character)fOp).charValue()-((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Character)fOp).charValue()-((Character)sOp).charValue()));}
        			}
        		}
        		else if(operation==Operator.DIV){
        			if(fOpType==Type.DOUBLE){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Double)fOp).doubleValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Double)fOp).doubleValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Double)fOp).doubleValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Double)fOp).doubleValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Double)fOp).doubleValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Double)fOp).doubleValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Double)fOp).doubleValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.FLOAT){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Float)fOp).floatValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Float)fOp).floatValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Float)fOp).floatValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Float)fOp).floatValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Float)fOp).floatValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Float)fOp).floatValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Float)fOp).floatValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.LONG){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Long)fOp).longValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Long)fOp).longValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Long)fOp).longValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Long)fOp).longValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Long)fOp).longValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Long)fOp).longValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.INT){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Integer)fOp).intValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Integer)fOp).intValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Integer)fOp).intValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Integer)fOp).intValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Integer)fOp).intValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Integer)fOp).intValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.SHORT){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Short)fOp).shortValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Short)fOp).shortValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Short)fOp).shortValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Short)fOp).shortValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Short)fOp).shortValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Short)fOp).shortValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Short)fOp).shortValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.BYTE){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Byte)fOp).byteValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Byte)fOp).byteValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Byte)fOp).byteValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Byte)fOp).byteValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Byte)fOp).byteValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Byte)fOp).byteValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Byte)fOp).byteValue()/((Character)sOp).charValue()));}
        			}
        			else if(fOpType==Type.CHAR){
        				if(sOpType==Type.DOUBLE){return(new Simplex(((Character)fOp).charValue()/((Double)sOp).doubleValue()));}
        				if(sOpType==Type.FLOAT){return(new Simplex(((Character)fOp).charValue()/((Float)sOp).floatValue()));}
        				if(sOpType==Type.LONG){return(new Simplex(((Character)fOp).charValue()/((Long)sOp).longValue()));}
        				if(sOpType==Type.INT){return(new Simplex(((Character)fOp).charValue()/((Integer)sOp).intValue()));}
        				if(sOpType==Type.SHORT){return(new Simplex(((Character)fOp).charValue()/((Short)sOp).shortValue()));}
        				if(sOpType==Type.BYTE){return(new Simplex(((Character)fOp).charValue()/((Byte)sOp).byteValue()));}
        				if(sOpType==Type.CHAR){return(new Simplex(((Character)fOp).charValue()/((Character)sOp).charValue()));}
        			}
        		}
        		else if (operation==Operator.ANDBW){
        			if(fOpType==Type.INT){
        				if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()&((Integer)sOp).intValue()));}
        			}
        			else if(fOpType==Type.LONG)
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()&((Long)sOp).longValue()));}
        		}
        		else if(operation==Operator.ORBW){
        			if(fOpType==Type.INT)
        			{if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()|((Integer)sOp).intValue()));}}
        			else if(fOpType==Type.LONG)
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()|((Long)sOp).longValue()));}
        		}
        		else if(operation==Operator.XOR){
        			if(fOpType==Type.INT)
        			{if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()^((Integer)sOp).intValue()));}}
        			else if(fOpType==Type.LONG)
        				if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()^((Long)sOp).longValue()));}
        		}

        		else if(operation==Operator.REM){
        			//ho assunto che il remainder sia fattibile solo con valori dello stesso tipo
        			if(sOpType==Type.DOUBLE){return(new Simplex((((Double)fOp).doubleValue())%((Double)sOp).doubleValue()));}
        			if(sOpType==Type.FLOAT){return(new Simplex(((Float)fOp).floatValue()%((Float)sOp).floatValue()));}
        			if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()%((Long)sOp).longValue()));}
        			if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()%((Integer)sOp).intValue()));}
        			if(sOpType==Type.SHORT){return(new Simplex(((Short)fOp).shortValue()%((Short)sOp).shortValue()));}
        			if(sOpType==Type.BYTE){return(new Simplex(((Byte)fOp).byteValue()%((Byte)sOp).byteValue()));}
        			if(sOpType==Type.CHAR){return(new Simplex(((Character)fOp).charValue()%((Character)sOp).charValue()));}
        		}
        		//non sono sicurissimo che l'operazione di shift sia giusta!
        		else if(operation==Operator.SHL){
        			if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()<<((Long)sOp).longValue()));}
        			if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()<<((Integer)sOp).intValue()));}
        		}
        		else if(operation==Operator.SHR){
        			if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()>>((Long)sOp).longValue()));}
        			if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()>>((Integer)sOp).intValue()));}
        		}
        		else if(operation==Operator.USHR){
        			if(sOpType==Type.LONG){return(new Simplex(((Long)fOp).longValue()>>>((Long)sOp).longValue()));}
        			if(sOpType==Type.INT){return(new Simplex(((Integer)fOp).intValue()>>>((Integer)sOp).intValue()));}
        		}
        		else if(operation==Operator.EQ){
        			return(new Simplex((boolean)(((Number)fOp).intValue()==((Number)sOp).intValue())));
        		}
        		else if(operation==Operator.NE) {
        			return(new Simplex(((Number)fOp).intValue()!=((Number)sOp).intValue()));
        		}
        		else if(operation==Operator.GT) {
        			return(new Simplex(((Number)fOp).intValue()>((Number)sOp).intValue()));
        		}
        		else if(operation==Operator.GE) {
        			return(new Simplex(((Number)fOp).intValue()>=((Number)sOp).intValue()));
        		}
        		else if(operation==Operator.LT) {
        			return(new Simplex(((Number)fOp).intValue()<((Number)sOp).intValue()));
        		}
        		else if(operation==Operator.LE) {
        			return(new Simplex(((Number)fOp).intValue()<=((Number)sOp).intValue()));
        		}
        		else if(operation==Operator.AND) {
        			return(new Simplex(((Boolean)fOp).booleanValue()&&((Boolean)sOp).booleanValue()));
        		}
        		else if(operation==Operator.OR) {
        			return(new Simplex(((Boolean)fOp).booleanValue()||((Boolean)sOp).booleanValue()));
        		}
        	}
        } 
        
        //here start some less trivial simplifications
        else if (binary && (firstOp instanceof Simplex || secondOp instanceof Simplex)) {
        	boolean firstIsSimplex = (firstOp instanceof Simplex);
        	Simplex simplexOp = (Simplex) (firstIsSimplex ? firstOp : secondOp);
        	Primitive otherOp = (firstIsSimplex ? secondOp : firstOp);
        	if (operation == Operator.AND) {
        		if ((Boolean) simplexOp.getActualValue()) {
        			return otherOp;
        		} else {
        			return simplexOp;
        		}
        	} 
        	else if (operation == Operator.OR) {
        		if ((Boolean) simplexOp.getActualValue()) {
        			return simplexOp;
        		} else {
        			return otherOp;
        		}
        	}
        	else if (operation == Operator.ADD) {
        		if (isZeroOne(simplexOp, true)) {
        			return otherOp;
        		}
        	}
        	else if (operation == Operator.MUL) {
        		if (isZeroOne(simplexOp, false)) {
        			return otherOp;
        		}
        	} 
        	else if (operation == Operator.SUB) {
        		if (isZeroOne(simplexOp, true)) {
        			if (firstIsSimplex) {
        				return new Expression(null, Operator.NEG, otherOp).normalize();
        			} else {
        				return otherOp;
        			}
        		}
        	} 
        	else if (operation == Operator.DIV) {
        		if (isZeroOne(simplexOp, false)) {
        			if (!firstIsSimplex) {
        				return otherOp;
        			}
        		}
        	}

        }
        else if (!binary && operation == Operator.NOT && secondOp instanceof Expression) {
        	Expression secondExp = (Expression) secondOp;
        	if (secondExp.operation == Operator.EQ) {
        		return new Expression(secondExp.firstOp, Operator.NE, secondExp.secondOp).normalize();
        	}
        	else if (secondExp.operation == Operator.NE) {
        		return new Expression(secondExp.firstOp, Operator.EQ, secondExp.secondOp).normalize();
        	}
        	else if (secondExp.operation == Operator.GT) {
        		return new Expression(secondExp.firstOp, Operator.LE, secondExp.secondOp).normalize();
        	}
        	else if (secondExp.operation == Operator.GE) {
        		return new Expression(secondExp.firstOp, Operator.LT, secondExp.secondOp).normalize();
        	}
        	else if (secondExp.operation == Operator.LT) {
        		return new Expression(secondExp.firstOp, Operator.GE, secondExp.secondOp).normalize();
        	}
        	else if (secondExp.operation == Operator.LE) {
        		return new Expression(secondExp.firstOp, Operator.GT, secondExp.secondOp).normalize();
        	}
        }
        
        
        //no simplification: returns this
        return this;
    }

    private boolean isZeroOne(Simplex s, boolean zero) {
    	byte b;
    	char c;
    	int i;
    	long l;
    	float f;
    	double d;
    	
    	if (zero) {
    		b = 0;
    		c = 0;
    		i = 0;
    		l = 0L;
    		f = 0F;
    		d = 0D;
    	} else {
    		b = 1;
    		c = 1;
    		i = 1;
    		l = 1L;
    		f = 1F;
    		d = 1D;
    	}
    	
		boolean retVal = false;
		if (s.getType() == Type.BYTE) {
			retVal = (((Byte) s.getActualValue()) == b);
		} else if (s.getType() == Type.CHAR) {
			retVal = (((Character) s.getActualValue()) == c);
		} else if (s.getType() == Type.INT) {
			retVal = (((Integer) s.getActualValue()) == i);
		} else if (s.getType() == Type.LONG) {
			retVal = (((Long) s.getActualValue()) == l);
		} else if (s.getType() == Type.FLOAT) {
			retVal = (((Float) s.getActualValue()) == f);
		} else if (s.getType() == Type.DOUBLE) {
			retVal = (((Double) s.getActualValue()) == d);
		}
		
		return retVal;
    }
    
    private char dominance() {
        //unary operators have the type of their only argument
        if (firstOp==null) {
            return(secondOp.getType());
        }
        
        char fOpType=firstOp.getType();
        char sOpType=secondOp.getType();
        if(fOpType==Type.DOUBLE||sOpType==Type.DOUBLE)
            return(Type.DOUBLE);
        else if(fOpType==Type.FLOAT||sOpType==Type.FLOAT)
            return(Type.FLOAT);
        else if(fOpType==Type.LONG||sOpType==Type.LONG)
            return(Type.FLOAT);
        else if(fOpType==Type.INT||sOpType==Type.INT)
            return(Type.INT);
        else if(fOpType==Type.SHORT||sOpType==Type.SHORT)
            return(Type.SHORT);
        else if(fOpType==Type.BYTE||sOpType==Type.BYTE)
            return(Type.BYTE);
        else if(fOpType==Type.CHAR||sOpType==Type.CHAR)
            return(Type.CHAR);
        else if(fOpType==Type.BOOLEAN||sOpType==Type.BOOLEAN)
            return(Type.BOOLEAN);
        else if (fOpType == Type.UNKNOWN)
        	return sOpType;
        else if (sOpType == Type.UNKNOWN)
        	return fOpType;
        else
            return Type.NULLREF;
    }

    @Override
    public String toString() {
		String retVal = "(";
        if (firstOp != null) {
            retVal += firstOp.toString() + " ";
        }
        retVal += operation + " " + secondOp.toString() + ")";
        return retVal;
    }
    
    @Override
    public boolean equals(Object o) {
        boolean retVal = false;
        
        if (o instanceof Expression) {
            Expression e = (Expression) o;
            
            retVal = this.firstOp.equals(e.firstOp);
            retVal = retVal && (this.operation.equals(e.operation));
            retVal = retVal && (this.secondOp.equals(e.secondOp));
        }
        
        return retVal;
    }

    @Override
	public void accept(ExpressionVisitor v) throws IOException, DecisionExternalProtocolException {
		v.visitExpression(this);
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		return; //does nothing
	}
	
	@Override
	public Expression clone() {
		//TODO is it right? If we assume that Primitives are immutable it is; check it. 
		return (Expression) super.clone();
	}
	
	/**
	 * Returns a new <code>SEExpression</code> built by substitution of 
	 * a subexpression with another one.
	 * 
	 * @param from a <code>Primitive</code>, to be replaced. 
	 * @param to a <code>Primitive</code>, replacing <code>from</code>.
	 * @return a <code>SEExpression</code> representing the expression 
	 *         obtained by replacing <code>from</code> with <code>to</code>
	 *         into <code>this</code>.
	 */
	public Expression replace(Primitive from, Primitive to) {
		Primitive first;
		if (this.firstOp == null) {
			first = null;
		} else if (this.firstOp.equals(from)) {
			first = to;
		} else if (this.firstOp instanceof Expression) {
			first = ((Expression) this.firstOp).replace(from, to);
		} else {
			first = this.firstOp;
		}
		
		Primitive second;
		if (this.secondOp.equals(from)) {
			second = to;
		} else if (this.secondOp instanceof Expression) {
			second = ((Expression) this.secondOp).replace(from, to);
		} else {
			second = this.secondOp;
		}
		
		return new Expression(first, this.operation, second);
	}
	
	public Set<Integer> symIds() {
		HashSet<Integer> retVal = new HashSet<Integer>();

		if (this.firstOp == null) {
			//nothing to do
		} else if (this.firstOp instanceof PrimitiveSymbolic) {
			retVal.add(((PrimitiveSymbolic) this.firstOp).getId());
		} else if (this.firstOp instanceof Expression) {
			retVal.addAll(((Expression) this.firstOp).symIds());
		} else if (this.firstOp instanceof FunctionApplication) {
			retVal.addAll(((FunctionApplication) this.firstOp).symIds());
		}
		
		if (this.secondOp instanceof PrimitiveSymbolic) {
			retVal.add(((PrimitiveSymbolic) this.secondOp).getId());
		} else if (this.secondOp instanceof Expression) {
			retVal.addAll(((Expression) this.secondOp).symIds());
		} else if (this.secondOp instanceof FunctionApplication) {
			retVal.addAll(((FunctionApplication) this.secondOp).symIds());
		}
		
		return retVal;
	}
}