package muse.mem;

import java.io.IOException;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import muse.dec.DecisionExternalProtocolException;


public class FunctionApplication extends Primitive {
	private String operator;
	private Primitive[] args;

	public FunctionApplication(String type, String operator, Primitive[] args) {
		this.setType(type.charAt(0));
		this.operator = operator;
		this.args = args;
	}

	public String getOperator() {
		return this.operator;
	}

	public Primitive[] getArgs() {
		return this.args;
	}

	@Override
	public void accept(ExpressionVisitor v) throws IOException,
	DecisionExternalProtocolException {
		v.visitFunctionApplication(this);

	}

	public String getValue() {
		String retVal = this.operator + "(";
		boolean first = true;
		for (Primitive p : this.args) {
			retVal += (first ? "" : ",") + p.toString();
			first = false;
		}
		retVal += ")";
		return retVal;
	}

	@Override
	public String toString() {
		return this.getValue();
	}

	public Primitive normalize() {
		Object[] args = new Object[this.args.length];
		int[] argsType = new int[this.args.length];
		int i = 0;
		for (Primitive p : this.args) {
			if (p instanceof Simplex) {
				args[i] = ((Simplex) p).getActualValue(); 
				argsType[i] = p.getType();
			} else
				return this;
			i++;
		}

		if (this.operator.equals(Operator.ABS)) {
			if(argsType[0] == Type.DOUBLE){return(new Simplex(Math.abs(((Double)args[0]).doubleValue())));}
			if(argsType[0] == Type.FLOAT) {return(new Simplex(Math.abs(((Float)args[0]).floatValue())));}
			if(argsType[0] == Type.LONG)  {return(new Simplex(Math.abs(((Long)args[0]).longValue())));}
			if(argsType[0] == Type.INT)   {return(new Simplex(Math.abs(((Integer)args[0]).intValue())));}
			if(argsType[0] == Type.SHORT) {return(new Simplex(Math.abs(((Short)args[0]).shortValue())));}
			if(argsType[0] == Type.BYTE)  {return(new Simplex(Math.abs(((Byte)args[0]).byteValue())));}
			if(argsType[0] == Type.CHAR)  {return(new Simplex(Math.abs(((Character)args[0]).charValue())));}
		}

		if (this.operator.equals(Operator.SIN)) {
			if(argsType[0] == Type.DOUBLE){return(new Simplex(Math.sin(((Double)args[0]).doubleValue())));}
			if(argsType[0] == Type.FLOAT) {return(new Simplex(Math.sin(((Float)args[0]).floatValue())));}
			if(argsType[0] == Type.LONG)  {return(new Simplex(Math.sin(((Long)args[0]).longValue())));}
			if(argsType[0] == Type.INT)   {return(new Simplex(Math.sin(((Integer)args[0]).intValue())));}
			if(argsType[0] == Type.SHORT) {return(new Simplex(Math.sin(((Short)args[0]).shortValue())));}
			if(argsType[0] == Type.BYTE)  {return(new Simplex(Math.sin(((Byte)args[0]).byteValue())));}
			if(argsType[0] == Type.CHAR)  {return(new Simplex(Math.sin(((Character)args[0]).charValue())));}
		}

		if (this.operator.equals(Operator.COS)) {
			if(argsType[0] == Type.DOUBLE){return(new Simplex(Math.cos(((Double)args[0]).doubleValue())));}
			if(argsType[0] == Type.FLOAT) {return(new Simplex(Math.cos(((Float)args[0]).floatValue())));}
			if(argsType[0] == Type.LONG)  {return(new Simplex(Math.cos(((Long)args[0]).longValue())));}
			if(argsType[0] == Type.INT)   {return(new Simplex(Math.cos(((Integer)args[0]).intValue())));}
			if(argsType[0] == Type.SHORT) {return(new Simplex(Math.cos(((Short)args[0]).shortValue())));}
			if(argsType[0] == Type.BYTE)  {return(new Simplex(Math.cos(((Byte)args[0]).byteValue())));}
			if(argsType[0] == Type.CHAR)  {return(new Simplex(Math.cos(((Character)args[0]).charValue())));}
		}

		if (this.operator.equals(Operator.TAN)) {
			if(argsType[0] == Type.DOUBLE){return(new Simplex(Math.tan(((Double)args[0]).doubleValue())));}
			if(argsType[0] == Type.FLOAT) {return(new Simplex(Math.tan(((Float)args[0]).floatValue())));}
			if(argsType[0] == Type.LONG)  {return(new Simplex(Math.tan(((Long)args[0]).longValue())));}
			if(argsType[0] == Type.INT)   {return(new Simplex(Math.tan(((Integer)args[0]).intValue())));}
			if(argsType[0] == Type.SHORT) {return(new Simplex(Math.tan(((Short)args[0]).shortValue())));}
			if(argsType[0] == Type.BYTE)  {return(new Simplex(Math.tan(((Byte)args[0]).byteValue())));}
			if(argsType[0] == Type.CHAR)  {return(new Simplex(Math.tan(((Character)args[0]).charValue())));}
		}

		if (this.operator.equals(Operator.POW)) {
			if(argsType[0] == Type.DOUBLE){
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Double)args[0]).doubleValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]== Type.FLOAT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Float)args[0]).floatValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.LONG) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Long)args[0]).longValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.INT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Integer)args[0]).intValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.SHORT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Short)args[0]).shortValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.BYTE) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Byte)args[0]).byteValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.CHAR) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.pow(((Character)args[0]).charValue(), ((Character)args[1]).charValue())));}
			}
		}

		if (this.operator.equals(Operator.MIN)) {
			if(argsType[0] == Type.DOUBLE){
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Double)args[0]).doubleValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]== Type.FLOAT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Float)args[0]).floatValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.LONG) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Long)args[0]).longValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.INT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Integer)args[0]).intValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.SHORT) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Short)args[0]).shortValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.BYTE) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Byte)args[0]).byteValue(), ((Character)args[1]).charValue())));}
			} else if(argsType[0]==Type.CHAR) {
				if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Double)args[1]).doubleValue())));}
				if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Float)args[1]).floatValue())));}
				if(argsType[1] == Type.LONG)   {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Long)args[1]).longValue())));}
				if(argsType[1] == Type.INT)    {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Integer)args[1]).intValue())));}
				if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Short)args[1]).shortValue())));}
				if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Byte)args[1]).byteValue())));}
				if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.min(((Character)args[0]).charValue(), ((Character)args[1]).charValue())));}
			}
		}

		//TODO EXP?

				if (this.operator.equals(Operator.MAX)) {
					if(argsType[0] == Type.DOUBLE){
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Double)args[0]).doubleValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]== Type.FLOAT) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Float)args[0]).floatValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]==Type.LONG) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Long)args[0]).longValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]==Type.INT) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Integer)args[0]).intValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]==Type.SHORT) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Short)args[0]).shortValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]==Type.BYTE) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Byte)args[0]).byteValue(), ((Character)args[1]).charValue())));}
					} else if(argsType[0]==Type.CHAR) {
						if(argsType[1] == Type.DOUBLE) {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Double)args[1]).doubleValue())));}
						if(argsType[1] == Type.FLOAT)  {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Float)args[1]).floatValue())));}
						if(argsType[1] == Type.LONG)   {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Long)args[1]).longValue())));}
						if(argsType[1] == Type.INT)    {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Integer)args[1]).intValue())));}
						if(argsType[1] == Type.SHORT)  {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Short)args[1]).shortValue())));}
						if(argsType[1] == Type.BYTE)   {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Byte)args[1]).byteValue())));}
						if(argsType[1] == Type.CHAR)   {return(new Simplex(Math.max(((Character)args[0]).charValue(), ((Character)args[1]).charValue())));}
					}
				}

				return this;
	}

	@Override
	public boolean equals(Object o) {
		boolean retVal = false;

		if (o instanceof FunctionApplication) {
			FunctionApplication e = (FunctionApplication) o;

			retVal = this.operator.equals(e.operator);
			int i = 0;
			for (Primitive p : this.args) {
				retVal = retVal && (p.equals(e.args[i]));
				i++;
			}
		}

		return retVal;
	}

	@Override
	public Object getValueForNative() throws ValueDoesNotSupportNativeException {
		throw new ValueDoesNotSupportNativeException();
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		return; //does nothing
	}

	public Set<Integer> symIds() {
		HashSet<Integer> retVal = new HashSet<Integer>();

		for (Primitive p : this.args) {
			if (p == null) {
				//nothing to do
			} else if (p instanceof PrimitiveSymbolic) {
				retVal.add(((PrimitiveSymbolic) p).getId());
			} else if (p instanceof Expression) {
				retVal.addAll(((Expression) p).symIds());
			} else if (p instanceof FunctionApplication) {
				retVal.addAll(((FunctionApplication) p).symIds());
			}
		}

		return retVal;
	}
}
