package muse.mem;

import java.util.Map;
import java.util.Set;

/**
 * Interface for instance class and and array class.
 */
public abstract class Objekt implements Cloneable {
	public enum Epoch { 
		EPOCH_UNKNOWN, EPOCH_BEFORE_START, EPOCH_AFTER_START
	}
	
    /**
     * Static Type identifier.
     */
	protected String type;

	/**
	 * The origin of the object in the case it is created by 
	 * lazy initialization.
	 */
	protected String origin;

    /**
     * The creation epoch of the Java object denoted by this <code>ObjDef</code>.
     */
    Epoch epoch = Epoch.EPOCH_UNKNOWN;

	/**
     * Returns the Static Type identifier.
     */
    public abstract String getType();
    
    public Objekt(String origin) {
    	this.origin = origin;
    }
    
    public String getOrigin() {
    	return this.origin;
    }
    
    /**
     * Returns the <code>Epoch</code> of the symbolic 
     * execution when the Java object denoted 
     * by this <code>ObjDef</code> has been created.
     *  
     * @return the creation epoch of the <code>Instance</code>.
     */
    public Epoch creationEpoch() { return this.epoch; }
    
    /**
     * Sets the creation <code>Epoch</code> of the 
     * Java object denoted by this <code>ObjDef</code>.
     * @param e
     */
    public void setCreationEpoch(Epoch e) { this.epoch = e; } 

    /**
     * Refines the object.
     * @param refs
     */
	public abstract void refine(Map<String, ReferenceSymbolic> refs);
    
	/**
	 * Returns the set of all the identifiers of symbolic primitive 
	 * values stored in the object.
	 * 
	 * @return a <code>Set&lt;Integer&gt;</code> of all and only the 
	 *         identifiers of symbolic primitive values stored in 
	 *         the object (empty when none).
	 */
	public abstract Set<Integer> symIdsPrimitive();
	
	/**
	 * Returns the set of all the identifiers of symbolic reference 
	 * values stored in the object.
	 * 
	 * @return a <code>Set&lt;Integer&gt;</code> of all and only the 
	 *         identifiers of symbolic reference values stored in 
	 *         the object (empty when none).
	 */
	public abstract Set<Integer> symIdsReference(); 

	@Override
    public Objekt clone() {
    	try {
    		return (Objekt) super.clone();
    	} catch (CloneNotSupportedException e) {
    		return null;
    	}
    }
}
