package muse.mem;

import java.util.List;
import java.util.Map;
import java.util.Stack;

/**
 * Class that represent the JVM's operand stack.
 */
public class OperandStack implements Cloneable {
    private Stack<Value> valueStack;
    
    /**
     * Constructor of empty operand stack.
     * 
     */
    public OperandStack() {
        this.valueStack = new Stack<Value>();
    }
    
    /**
     * Put the argument on the top of the operand stack.
     * @param item : Value to put on the top of operand stack.*/
    public void push(Value item) {
        valueStack.push(item);
    }
    
    /**
     * Return and delete the element from the top of the operand stack.
     * @return Value: value on the top of the operand stack.
     */
    public Value pop() {
        return(valueStack.pop());
    }
    
    public List<Value> values() {
    	return this.valueStack;
    }
    
    /**
     * Returns a string representation for the operand stack
     */
    public String toString() {
        String tmp = "{";
        int i = 0;
        for (Value v : this.valueStack) {
            tmp += v.toString();
            if (i < this.valueStack.size() - 1) 
            	tmp += ", ";
            i++;
        }
        tmp += "}";
        return(tmp);
    }
    
    /**
     * Returns the topmost stack element without removing it
     * 
     * @return the topmost stack element
     */
    public Value returnLast() {
        return(valueStack.peek());
    }
    
    public Object clone() {
        OperandStack o = null;
        try {
            o = (OperandStack) super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        
        o.valueStack = new Stack<Value>();
        for (Value v : this.valueStack) {
        	o.valueStack.push((Value) v.clone());
        }
        return o;
    }

	public void refine(Map<String, ReferenceSymbolic> refs) {
		for (Value v : this.valueStack)
			v.refine(refs);
	}


}

//TODO Non viene gestito l'overflow dello stack che non dovrebbe neanche avvenire con una JVM corretta
