package muse.mem;

import java.io.IOException;
import java.util.Map;

import muse.dec.DecisionExternalProtocolException;


/**
 * Class that represent a symbolic value. 
 */
public class PrimitiveSymbolic extends Primitive implements Symbolic {
    //Fields
    private int id;
    private String origin;

    /**
     * Constructor.
     * 
     * @param value 
     *        <code>String</code> representing the symbolic value.
     *        It can be an arbitrary text.
     * @param type the type of the represented value.
     */
    public PrimitiveSymbolic(int id, String type, String origin) {
    	this.id = id;
        this.setType(type.charAt(0));
        this.origin = origin;
    }
    
    public int getId() {
    	return this.id;
    }
    
    public String getOrigin() {
    	return this.origin;
    }

	public void setOrigin(String origin) {
		this.origin = origin;
	}
    
    /**
     * Returns the value of symbolic type
     */
    public String getValue() {
        return "{V" + this.id + "}";
    }

    /**
     * Returns a string representation of the symbolic value.
     */
    @Override
    public String toString() {
        return this.getValue();
    }

    @Override
    public boolean equals(Object o) {
        boolean retVal = false;
        
        if (o instanceof PrimitiveSymbolic) {
            PrimitiveSymbolic s = (PrimitiveSymbolic) o;
            
            retVal = (s.id == this.id);
        }
        return retVal;
    }

	@Override
	public void accept(ExpressionVisitor v) throws DecisionExternalProtocolException, IOException {
		v.visitPrimitiveSymbolic(this);
	}

	@Override
	public Object getValueForNative() throws ValueDoesNotSupportNativeException {
		throw new ValueDoesNotSupportNativeException();
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		return; //does nothing
	}

}