package muse.mem;

import java.util.Map;

public class ReferenceSymbolic extends Reference implements Symbolic {
	private static final int NULL_VALUE = -1;
	private static final int NOT_INITIALIZED = -2;

	/** An identifier for the value, in order to track lazy initialization. */
	private int id;
	
	/** The position of the instance in the heap if the reference is initialized, otherwise NOT_INITIALIZED */
	private int value;
	
    /** 
     * The static type of the symbolic reference, or <code>null</code> 
     * iff it is a concrete reference. 
     */
    private String staticType;

    /** The origin of the reference. */
	private String origin;

    /**
     * Constructor returning an uninitialized symbolic reference.
     */
    public ReferenceSymbolic(int id, String staticType, String origin) {
        this.value = NOT_INITIALIZED;
        this.id = id;
        this.staticType = staticType;
        this.origin = origin;
    }
    
    @Override
    public int getHeapPosition() {
        return this.value;
    }
    
    public String getOrigin() {
    	return this.origin;
    }

	public void setOrigin(String origin) {
		this.origin = origin;
	}
    public String getTypeNotInitialized() {
    	return this.staticType;
    }

    public boolean isInitialized() {
        return (this.value != NOT_INITIALIZED); 
    }
    
    public void initialize(int i) {
        this.value = i;
    }
    
    public void initializeNull() {
        this.value = NULL_VALUE;
    }
    
    public int getId() {
    	return this.id;
    }

    public String getValue() {
    	return "{R" + this.id + "}";
    }

    @Override
    public boolean isSymbolic() {
    	return true;
    }
    
    @Override
    public boolean isNull() {
        return this.value == NULL_VALUE;
    }

    @Override
    public String toString() {
    	return this.getValue();
    }

    @Override
    public boolean equals(Object o) {
        boolean retVal = false;
        
        if (o instanceof ReferenceSymbolic) {
        	ReferenceSymbolic r = (ReferenceSymbolic) o;
            
           	retVal = (this.id == r.id);
        }
        return retVal;
    }

	@Override
	public ReferenceSymbolic clone() {
		return (ReferenceSymbolic) super.clone();
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		ReferenceSymbolic r = refs.get(this.getValue());
		if (r != null) {
			this.value = r.value;
		}
	}
}
