package muse.mem;

import java.io.IOException;
import java.util.Map;

import muse.dec.DecisionExternalProtocolException;


/**
 * Class that represent a simple primitive type. Implements the 
 * operation between value
 */
public class Simplex extends Primitive implements Cloneable {
	/*Fields*/
    private Object obj;

	@Override
	public Object getValueForNative() {
		return this.obj;
	}

	@Override
    public Simplex clone() {
        return (Simplex) super.clone();
    }
    
    /**
     * Constructor for boolean value
     */
    public Simplex(boolean value) {
        obj=new Boolean(value);
        this.setType(Type.BOOLEAN);
    }
    
    /**
     * Constructor for byte value
     */
    public Simplex(byte value){
        obj=new Byte(value);
        this.setType(Type.BYTE);
    }
    
    /**
     * Constructor for short value
     */
    public Simplex(short value){
        obj=new Short(value);
        this.setType(Type.SHORT);
    }
    
    /**
     * Constructor for int value
     */
    public Simplex(int value){
        obj=new Integer(value);
        this.setType(Type.INT);
    }
    
    /**
     *Constructor for long value
     */
    public Simplex(long value){
        obj=new Long(value);
        this.setType(Type.LONG);
    }
    
    /**
     *Constructor for float value
     */
    public Simplex(float value){
        obj=new Float(value);
        this.setType(Type.FLOAT);
    }
    
    /**
     *Constructor for double value
     */
    public Simplex(double value){
        obj=new Double(value);
        this.setType(Type.DOUBLE);
    }
    
    /**
     *Constructor for char value
     */
    public Simplex(char value){
        obj=new Character(value);
        this.setType(Type.CHAR);
    }
    
    /**
     *Returns the value of the simplex value
     */
    public Object getActualValue(){
        return(obj);
    }
    
    /**
     *Returns a string rapresentation of a simplex value
     */
    public String toString(){
        return(obj.toString());
    }
    
    @Override
    public boolean equals(Object o) {
        boolean retVal = false;
        
        if (o instanceof Simplex) {
            Simplex s = (Simplex) o;
            
            if (this.getType() == s.getType()) {            
                if (this.getType() == Type.BOOLEAN) {
                    retVal = (((Boolean) this.obj).booleanValue() == ((Boolean) s.obj).booleanValue());
                } else if (this.getType() == Type.BYTE) {
                    retVal = (((Byte) this.obj).byteValue() == ((Byte) s.obj).byteValue());
                } else if (this.getType() == Type.CHAR) {
                    retVal = (((Character) this.obj).charValue() == ((Character) s.obj).charValue());
                } else if (this.getType() == Type.DOUBLE) {
                    retVal = (((Double) this.obj).doubleValue() == ((Double) s.obj).doubleValue());
                } else if (this.getType() == Type.FLOAT) {
                    retVal = (((Float) this.obj).floatValue() == ((Float) s.obj).floatValue());
                } else if (this.getType() == Type.INT) {
                    retVal = (((Integer) this.obj).intValue() == ((Integer) s.obj).intValue());
                } else if (this.getType() == Type.LONG) {
                    retVal = (((Long) this.obj).longValue() == ((Long) s.obj).longValue());
                } else if (this.getType() == Type.SHORT) {
                    retVal = (((Short) this.obj).shortValue() == ((Short) s.obj).shortValue());
                }
            }
        }
        return retVal;
    }

	@Override
	public void accept(ExpressionVisitor v) throws IOException, DecisionExternalProtocolException {
		v.visitSimplex(this);
	}

	@Override
	public void refine(Map<String, ReferenceSymbolic> refs) {
		return; //does nothing
	}
}