package muse.mem;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Class representing JVM thread stacks.
 */
public class ThreadStack implements Cloneable {
    private ArrayList<Frame> frameStack;

    /**
     * Constructor of stack.
     */
    public ThreadStack() {
        this.frameStack = new ArrayList<Frame>(1);
    }
    
    /**
     * Put a frame on the top of the stack.
     * @param item: Frame that must be put on the top of the stack, it is the frame that it will become the active frame, it must be initialized before being passed.*/
    public void push(Frame item){
    	this.frameStack.add(item);
    }
    
    /**
     * Return the frame on the top of the stack and delete it.
     * @return Frame: Frame deallocated from the stack.
     */
    public Frame pop() {
        if (this.empty()) {
            throw new StackLastFrameException();
        }
        return this.frameStack.remove(this.frameStack.size() - 1);
    }
    
    public boolean empty() {
        return (this.frameStack.size() == 0);
    }
    
    /**
     * Returns the current frame.
     * @return the current <code>Frame</code>, i.e., the 
     *         <code>Frame</code> on the top of the stack.
     */
    public Frame currentFrame() {
        if (this.empty()) {
            throw new StackLastFrameException();
        }
        return((Frame) frameStack.get(frameStack.size() - 1));
    }
    
    public List<Frame> frames() {
    	return this.frameStack;
    }
    
    /**
     * Returns a <code>String</code> representation of frame stack.
     */
    public String toString() {
    	String tmpRet = "[";
        int j = 0;
        for (Frame f : this.frameStack) {
            tmpRet +=  j + ":" + f.toString(); 
            if (j < this.frameStack.size() - 1) 
            	tmpRet += ", ";
            j++;
        }
        tmpRet += "]";
        return(tmpRet);
    }
    
    @Override
    public ThreadStack clone() {
        ThreadStack o = null;
        try {
            o = (ThreadStack) super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        
        ArrayList<Frame> frameStackClone = new ArrayList<Frame>();
        
        for (int i = 0; i < frameStack.size(); i++) {
            frameStackClone.add((Frame)(frameStack.get(i)).clone());
        }
        o.frameStack = frameStackClone;
        //TODO is it just the same as o.frameStack = (ArrayList) o.frameStack.clone()?
        return o;
    }   
    
	public void refine(Map<String, ReferenceSymbolic> refs) {
		for (Frame f : this.frameStack) 
			f.refine(refs);
	}
}