package muse.mem;

import java.util.Map;

/**
 * Class that represent data of reference type, primitive type and symbolic 
 * type in memory.
 */
public abstract class Value implements Cloneable {
    /** The type of this <code>Value</code>. */
    private char type;

    public abstract Object getValueForNative() throws ValueDoesNotSupportNativeException;
    
    @Override
    public Value clone() {
      Value o = null;
      try {
        o = (Value) super.clone();
      } catch (CloneNotSupportedException e) {
      }
      return o;
    }


    /**
     *Return the type of the Value
     */
    public char getType() {
		return this.type;
    }
    
    /**
     * Used only in the package to set the type of a value
     */
    public void setType(char type) {
	    this.type = type;
    }
    
    /**
     * Returns String representation of a value
     */
    @Override
    public abstract String toString();
    
    @Override
    public abstract boolean equals(Object o);

    /**
     * Refines the object.
     * 
     * @param refs 
     *         a <code>Map&lt;String, SESymbolicReference&gt;</code> 
     *         associating symbolic reference names with their respective 
     *         concrete values.
     */
	public abstract void refine(Map<String, ReferenceSymbolic> refs);
}
