package muse.mem;

/**
 * Class representing a variable into the memory. It gathers the name, 
 * the declaration type and the runtime value of the variable. 
 */
public class Variable implements Cloneable {
    /** Type of the variable. */
    private String type;

    /** Name of the variable. */
    private String name;

    /** <code>Value</code> stored in the variable. */
    private Value lnkValue;

    @Override
    public Object clone() {
        Variable o = null;
        try {
            o = (Variable) super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        o.lnkValue = (Value) o.lnkValue.clone();
        return o;
    }

    /**
     * Constructor setting the variable to its default value.
     * 
     * @param type type of variable
     * @param name name of variable
     */
    public Variable(String type, String name) {
    	this(type, name, null);
    }

    /**
     * Constructor setting the variable to a prescribed value.
     * 
     * @param type type of variable
     * @param name name of variable
     * @param value value of variable
     */
    public Variable(String type, String name, Value value) {
        if (value != null)
        	this.lnkValue = value;
        else
        	this.lnkValue = Type.createDefault(type.charAt(0));
        this.type = type;
        this.name = name;
    }
    
    /**
     * Returns the value of variable
     */
    public Value getValue() {
        return this.lnkValue;
    }

    /**
     * Set the value of variable
     */
    public void setValue(Value value) {
    	this.lnkValue = value;
    }

    /**
     * Returns the name of variable
     */
    public String getName() {
        return this.name;
    }

    /**
     * Returns the type of variable
     */
    public String getType() {
        return this.type;
    }

    public String toString() {
        String tmp;
        if (this.lnkValue == null)
            tmp = "<UNASSIGNED>";
        else
            tmp = lnkValue.toString();
        return "[Name:" + this.name + ", Type:" + this.type + ", Value:" + tmp + "]";
    }
}