package muse.algo;

import static org.junit.Assert.*;

import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.Value;

import org.junit.*;

import util.Util;

public class BytecodesConstTest 
{
	public static char intType = 'I';
	public static char doubleType = 'D';
	public static char floatType = 'F';
	public static char longType = 'J';
	public static char referenceType = 'L';
	
	
	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataConst";
		Util.setUp();
	}
	
	/*
	 * Bytecode: 	iConstM1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConstM1()
     * Expected result: operand stack at the top of the stack has int type and value -1
     *
	 * */
	@Test
	public void IConstM1() throws EngineStuckException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, DecisionRunningException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConstM1");
		Util.stepUntil(Opcodes.OP_ICONST_M1);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("-1", topStack.toString());
		assertEquals(topStack.getType(), intType);
		
	}
	
	/*
	 * Bytecode: 	iConst_0
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_0()
     * Expected result: operand stack at the top of the stack has int type and value 0
     *
	 * */
	@Test
	public void IConst_0() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_0");
		Util.stepUntil(Opcodes.OP_ICONST_0);
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("0", topStack.toString());
		assertEquals(topStack.getType(), intType);
		
	}
	
	/*
	 * Bytecode: 	iConst_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_1()
     * Expected result: operand stack at the top of the stack has int type and value 1
     *
	 * */
	@Test
	public void IConst_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_1");
		Util.stepUntil(Opcodes.OP_ICONST_1);
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("1", topStack.toString());
		assertEquals(topStack.getType(), intType);
		
	}
	
	/*
	 * Bytecode: 	iConst_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_2()
     * Expected result: operand stack at the top of the stack has int type and value 2
     *
	 * */
	@Test
	public void IConst_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_2");
		Util.stepUntil(Opcodes.OP_ICONST_2);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("2", topStack.toString());
		assertEquals(topStack.getType(), intType);		
	}
	/*
	 * Bytecode: 	iConst_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_3()
     * Expected result: operand stack at the top of the stack has int type and value 3
     *
	 * */
	@Test
	public void IConst_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_3");
		Util.stepUntil(Opcodes.OP_ICONST_3);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("3", topStack.toString());
		assertEquals(topStack.getType(), intType);		
	}
	
	/*
	 * Bytecode: 	iConst_4
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_4()
     * Expected result: operand stack at the top of the stack has int type and value 4
     *
	 * */
	@Test
	public void IConst_4() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_4");
		Util.stepUntil(Opcodes.OP_ICONST_4);
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("4", topStack.toString());
		assertEquals(topStack.getType(), intType);		
	}
	
	/*
	 * Bytecode: 	iConst_5
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testIConst_5()
     * Expected result: operand stack at the top of the stack has int type and value 5
     *
	 * */
	@Test
	public void IConst_5() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testIConst_5");
		Util.stepUntil(Opcodes.OP_ICONST_5);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("5", topStack.toString());
		assertEquals(topStack.getType(), intType);
		
	}
	
	/*
	 * Bytecode: 	dConst_0
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testDConst_0()
     * Expected result: operand stack at the top of the stack has double type and value 0.0
     *
	 * */
	@Test
	public void DConst_0() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testDConst_0");
		Util.stepUntil(Opcodes.OP_DCONST_0);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("0.0", topStack.toString());
		assertEquals(topStack.getType(), doubleType);		
	}
	
	/*
	 * Bytecode: 	dConst_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testDConst_1()
     * Expected result: operand stack at the top of the stack has double type and value 1.0
     *
	 * */
	@Test
	public void DConst_1() throws EngineStuckException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, DecisionRunningException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testDConst_1");
		Util.stepUntil(Opcodes.OP_DCONST_1);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("1.0", topStack.toString());
		assertEquals(topStack.getType(), doubleType);
	}
	
	/*
	 * Bytecode: 	fConst_0
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testFConst_0()
     * Expected result: operand stack at the top of the stack has float type and value 0.0
     *
	 * */
	@Test
	public void FConst_0() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testFConst_0");
		Util.stepUntil(Opcodes.OP_FCONST_0);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("0.0", topStack.toString());
		assertEquals(topStack.getType(), floatType);
	}
	
	/*
	 * Bytecode: 	fConst_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testFConst_1()
     * Expected result: operand stack at the top of the stack has float type and value 1.0
     *
	 * */
	@Test
	public void FConst_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testFConst_1");
		Util.stepUntil(Opcodes.OP_FCONST_1);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("1.0", topStack.toString());
		assertEquals(topStack.getType(), floatType);
		
	}
	
	/*
	 * Bytecode: 	iConst_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testFConst_2()
     * Expected result: operand stack at the top of the stack has float type and value 2.0
     *
	 * */
	@Test
	public void FConst_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testFConst_2");
		Util.stepUntil(Opcodes.OP_FCONST_2);
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("2.0", topStack.toString());
		assertEquals(topStack.getType(), floatType);
	}
	
	/*
	 * Bytecode: 	lConst_0
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testLConst_0()
     * Expected result: operand stack at the top of the stack has long type and value 0
     *
	 * */
	@Test
	public void LConst_0() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testLConst_0");
		Util.stepUntil(Opcodes.OP_LCONST_0);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("0", topStack.toString());
		assertEquals(topStack.getType(), longType);
		
	}
	
	/*
	 * Bytecode: 	lConst_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testLConst_1()
     * Expected result: operand stack at the top of the stack has long type and value 1
     *
	 * */
	@Test
	public void LConst_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testLConst_1");
		Util.stepUntil(Opcodes.OP_LCONST_1);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("1", topStack.toString());
		assertEquals(topStack.getType(), longType);
		
	}
	
	/*
	 * Bytecode: 	aconst_null
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataConst.testAConst_Null()
     * Expected result: operand stack at the top of the stack has null value
     *
	 * */
	@Test
	public void AConst_Null() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V", "testAConst_Null");
		Util.stepUntil(Opcodes.OP_ACONST_NULL);			
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals("null", topStack.toString());		
	}
}
