package muse.algo;

import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.LocalVariablesArea;
import muse.mem.Simplex;
import muse.mem.Value;

import org.junit.*;

import util.Util;

import static org.junit.Assert.*;

public class BytecodesLoadTest 
{
	public static char intType = 'I';
	public static char doubleType = 'D';
	public static char floatType = 'F';
	public static char longType = 'J';
	public static char referenceType = 'L';


	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataLoad";
		Util.setUp();
	}


	/*
	 * Bytecode: 	iload_1
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_1()
	 * Expected result: top of operand stack has int type and value 5
	 *
	 * */
	@Test
	public void ILoad_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testILoad_1");
		Util.stepUntil(Opcodes.OP_ILOAD_1);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(1);
		assertEquals(valLocal.getType(),intType);
		
		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	iload_2
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_2()
	 * Expected result: top of operand stack has int type and value 6
	 *
	 * */
	@Test
	public void ILoad_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testILoad_2");
		Util.stepUntil(Opcodes.OP_ILOAD_2);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(2);
		assertEquals(valLocal.getType(),intType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);

	}

	/*
	 * Bytecode: 	iload_3
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_3()
	 * Expected result: top of operand stackhas int type and value 7
	 *
	 * */
	@Test
	public void ILoad_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testILoad_3");
		Util.stepUntil(Opcodes.OP_ILOAD_3);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(3);
		assertEquals(valLocal.getType(),intType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);

	}

	/*
	 * Bytecode: 	iload
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoadN()
	 * Expected result: top of operand stackhas int type and value 8
	 *
	 * */
	@Test
	public void ILoad_N() throws EngineStuckException, DecisionRunningException,CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testILoadN");
		Util.stepUntil(Opcodes.OP_ILOAD);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value indexVal = new Simplex((int)Util.s.getInstruction(1));
		int index = Integer.parseInt(indexVal.toString());
		Value valLocal = local.get(index);
		assertEquals(valLocal.getType(),intType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);

	}

	/*
	 * Bytecode: 	dload_1
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testDLoad_1()
	 * Expected result: top of operand stack has double type and value 50.1
	 *
	 * */
	@Test
	public void DLoad_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testDLoad_1");
		Util.stepUntil(Opcodes.OP_DLOAD_1);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(1);
		assertEquals(valLocal.getType(),doubleType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	dload_2
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testDLoad_2()
	 * Expected result: top of operand stackhas double type and value 50.2
	 *
	 * */
	@Test
	public void DLoad_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testDLoad_2");
		Util.stepUntil(Opcodes.OP_DLOAD_2);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(2);
		assertEquals(valLocal.getType(),doubleType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	dload_3
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testDLoad_3()
	 * Expected result: top of operand stack has double type and value 50.3
	 *
	 * */
	@Test
	public void DLoad_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testDLoad_3");
		Util.stepUntil(Opcodes.OP_DLOAD_3);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(3);
		assertEquals(valLocal.getType(),doubleType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	dload
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testDLoadN()
	 * Expected result: top of operand stack has double type and value 50.4
	 *
	 * */
	@Test
	public void DLoad_N() throws EngineStuckException, DecisionRunningException,CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testDLoadN");
		Util.stepUntil(Opcodes.OP_DLOAD);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value indexVal = new Simplex((int)Util.s.getInstruction(1));
		int index = Integer.parseInt(indexVal.toString());
		Value valLocal = local.get(index);
		assertEquals(valLocal.getType(),doubleType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	fload_1
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testFLoad_1()
	 * Expected result: top of operand stack has float type and value 5.1
	 *
	 * */
	@Test
	public void FLoad_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testFLoad_1");
		Util.stepUntil(Opcodes.OP_FLOAD_1);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(1);
		assertEquals(valLocal.getType(),floatType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	fload_2
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_2()
	 * Expected result: top of operand stack has float type and value 5.2
	 *
	 * */
	@Test
	public void FLoad_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testFLoad_2");
		Util.stepUntil(Opcodes.OP_FLOAD_2);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(2);
		assertEquals(valLocal.getType(),floatType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	fload_3
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_3()
	 * Expected result: top of operand stack has float type and value 5.3
	 *
	 * */
	@Test
	public void FLoad_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testFLoad_3");
		Util.stepUntil(Opcodes.OP_FLOAD_3);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(3);
		assertEquals(valLocal.getType(),floatType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	fload
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testFLoadN()
	 * Expected result: top of operand stack has float type and value 5.4
	 *
	 * */
	@Test
	public void FLoad_N() throws EngineStuckException, DecisionRunningException,CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testFLoadN");
		Util.stepUntil(Opcodes.OP_FLOAD);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value indexVal = new Simplex((int)Util.s.getInstruction(1));
		int index = Integer.parseInt(indexVal.toString());
		Value valLocal = local.get(index);
		assertEquals(valLocal.getType(),floatType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	lload_1
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testLLoad_1()
	 * Expected result: top of operand stackhas long type and value 15
	 *
	 * */
	@Test
	public void LLoad_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testLLoad_1");
		Util.stepUntil(Opcodes.OP_LLOAD_1);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(1);
		assertEquals(valLocal.getType(),longType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	lload_2
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testLLoad_2()
	 * Expected result: top of operand stack has long type and value 16
	 *
	 * */
	@Test
	public void LLoad_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testLLoad_2");
		Util.stepUntil(Opcodes.OP_LLOAD_2);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(2);
		assertEquals(valLocal.getType(),longType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	lload_3
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testLLoad_3()
	 * Expected result: top of operand stack has long type and value 17
	 *
	 * */
	@Test
	public void LLoad_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testLLoad_3");
		Util.stepUntil(Opcodes.OP_LLOAD_3);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(3);
		assertEquals(valLocal.getType(),longType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	lload
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoadN()
	 * Expected result: top of operand stack has long type and value 18
	 *
	 * */
	@Test
	public void LLoad_N() throws EngineStuckException, DecisionRunningException,CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testLLoadN");
		Util.stepUntil(Opcodes.OP_LLOAD);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value indexVal = new Simplex((int)Util.s.getInstruction(1));
		int index = Integer.parseInt(indexVal.toString());
		Value valLocal = local.get(index);
		assertEquals(valLocal.getType(),longType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	aload_1
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testILoad_1()
	 * Expected result: top of operand stack has reference type and value the same as 1st local variable 
	 *
	 * */
	@Test
	public void ALoad_1() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testALoad_1");
		Util.stepUntil(Opcodes.OP_ALOAD_1);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(1);
		assertEquals(valLocal.getType(),referenceType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	aload_2
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testALoad_2()
	 * Expected result: top of operand stack has reference type and value the same as 2nd local variable 
	 *
	 * */
	@Test
	public void ALoad_2() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testALoad_2");
		Util.stepUntil(Opcodes.OP_ALOAD_2);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal = local.get(2);
		assertEquals(valLocal.getType(),referenceType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	aload_3
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testALoad_3()
	 * Expected result: top of operand stack has reference type and value the same as 3rd local variable 
	 *
	 * */
	@Test
	public void ALoad_3() throws EngineStuckException, DecisionRunningException,CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testALoad_3");
		Util.stepUntil(Opcodes.OP_ALOAD_3);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value valLocal= local.get(3);
		assertEquals(valLocal.getType(),referenceType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}

	/*
	 * Bytecode: 	aload
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataLoad.testALoadN()
	 * Expected result: top of operand stack has reference type and value the same as 4th local variable 
	 *
	 * */
	@Test
	public void ALoad_N() throws EngineStuckException, DecisionRunningException,CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Util.begin("()V","testALoadN");
		Util.stepUntil(Opcodes.OP_ALOAD);
		LocalVariablesArea local = Util.s.getStack().currentFrame().getLocalVariableTable();
		Value indexVal = new Simplex((int)Util.s.getInstruction(1));
		int index = Integer.parseInt(indexVal.toString());
		Value valLocal = local.get(index);
		assertEquals(valLocal.getType(),referenceType);

		Util.engine.step();
		Value topStack = Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(valLocal,topStack);
	}
}
