package muse.algo;

import static org.junit.Assert.*;

import java.util.List;

import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.Value;

import org.junit.*;

import util.Util;

public class BytecodesOpStackManagementTest 
{
	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataStack";
		Util.setUp();
	}


	/*
	 * Bytecode: 	dup
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup()
     * Expected result: duplicate the value 6 on the top of the operand stack
     *
	 * */
	@Test
	public void Dup() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, val2, val3;
			List<Value> stackList;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup");
			Util.stepUntil(Opcodes.OP_DUP);
			topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
			stackBefore =  Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			Util.engine.step();
			stackList = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackList.size();
		    
		    val2= stackList.get(stackList.size()-1); //the last element
		    val3= stackList.get(stackList.size()-2); //the element before the last
			
		    assertEquals(topStack1,val2);
			assertEquals(val2,val3);
			assertEquals(stackBefore+1,stackAfter);
		
	}
	
	/*
	 * Bytecode: 	dup2
     * Coverage:    Form1
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2()
     * Expected result: (20 10) -> (20 10 20 10) All value is category 1
     *
	 * */
	@Test
	public void Dup2_Form1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2, val3,val4;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_Form1");
			Util.stepUntil(Opcodes.OP_DUP2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			
			topStack1 = stackListBefore.get(stackListBefore.size()-2);
			topStack2 = stackListBefore.get(stackListBefore.size()-1);
			stackBefore =  stackListBefore.size();
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		    val1= stackListAfter.get(stackListAfter.size()-4); 
		    val2= stackListAfter.get(stackListAfter.size()-3); 
		    val3= stackListAfter.get(stackListAfter.size()-2);
		    val4= stackListAfter.get(stackListAfter.size()-1); //the last element
		    
		    
		    assertEquals(topStack1,val1);
		    assertEquals(topStack2,val2);
		    assertEquals(topStack1,val3);
		    assertEquals(topStack2,val4);
		
			assertEquals(stackBefore*2,stackAfter);  //size test
		
	}
	
	/*
	 * Bytecode: 	dup2
     * Coverage:    Form2
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2()
     * Expected result: Duplicate 3000000000 on the operand stack
     *
	 * */
	@Test
	public void Dup2_Form2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack, val1,val2;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_Form2");
			Util.stepUntil(Opcodes.OP_DUP2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			
			topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
			stackBefore =  stackListBefore.size();
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
			
		    val1= stackListAfter.get(stackListAfter.size()-2);
		    val2= stackListAfter.get(stackListAfter.size()-1); //the last element
		    
		    assertEquals(topStack,val1);
		    assertEquals(topStack,val2);
		
		
			assertEquals(stackBefore*2,stackAfter);  //size test
		
	}
	
	/*
	 * Bytecode: 	pop
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataStack.testPop()
     * Expected result: Pop the value from the operand stack
     *
	 * */
	@Test
	public void Pop() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			
			int stackBefore, stackAfter;
			
			Util.begin("()V", "testPop");
			Util.stepUntil(Opcodes.OP_POP);
			stackBefore =  Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			Util.engine.step();
			stackAfter = Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			assertEquals(stackBefore,stackAfter+1);
		
	}
	
	/*
	 * Bytecode: 	pop2
     * Coverage:    Form1
     * Test data:   Test.bernardini.jasminClass.DataStack.testPop2_Form1()
     * Expected result: Pop two values from the top of the operand stack (values is both category 1)
     *
	 * */
	@Test
	public void Pop2_Form1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			
			int stackBefore, stackAfter;
			
			Util.begin("()V", "testPop2_Form1");
			Util.stepUntil(Opcodes.OP_POP2);
			stackBefore =  Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			Util.engine.step();
			stackAfter = Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			assertEquals(stackBefore,stackAfter+2);
		
	}
	
	/*
	 * Bytecode: 	pop2
     * Coverage:    Form2
     * Test data:   Test.bernardini.jasminClass.DataStack.testPop2_Form2()
     * Expected result: Pop a category 2 value from the top of the operand stack
     *
	 * */
	@Test
	public void Pop2_Form2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			
			int stackBefore, stackAfter;
			
			Util.begin("()V", "testPop2_Form2");
			Util.stepUntil(Opcodes.OP_POP2);
			stackBefore =  Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			Util.engine.step();
			stackAfter = Util.s.getStack().currentFrame().getOperandStack().values().size();
			
			assertEquals(stackBefore,stackAfter+1);
		
	}
	
	/*
	 * Bytecode: 	swap
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataStack.testSwap()
     * Expected result: Swap the top two values on the operand stack (5,6 -> 6,5)
     *
	 * */
	@Test
	public void Swap() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testSwap");
			Util.stepUntil(Opcodes.OP_SWAP);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			
			topStack1 = stackListBefore.get(stackListBefore.size()-2);
			topStack2 = stackListBefore.get(stackListBefore.size()-1);
			stackBefore =  stackListBefore.size();
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		   
		    val1= stackListAfter.get(stackListAfter.size()-2);
		    val2= stackListAfter.get(stackListAfter.size()-1); //the last element
		    
		    
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val1);
		  
		
			assertEquals(stackBefore,stackAfter);  //size test
		
	}
	
	/*
	 * Bytecode: 	dup_x1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup_X1()
     * Expected result: (20 10) -> (10 20 10)
     *
	 * */
	@Test
	public void Dup_X1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2, val3;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup_X1");
			Util.stepUntil(Opcodes.OP_DUP_X1);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-2);
			topStack2 = stackListBefore.get(stackBefore-1);
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		    val1= stackListAfter.get(stackAfter - 3); 
		    val2= stackListAfter.get(stackAfter - 2); 
		    val3= stackListAfter.get(stackAfter - 1);
		   
		    
		    assertEquals(topStack2,val1);
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val3);		
	}
	
	/*
	 * Bytecode: 	dup_x2
     * Coverage:    Form1
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup_X2_Form1()
     * Expected result: (30 20 10) -> (10 30 20 10)
     *
	 * */
	@Test
	public void Dup_X2_Form1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, topStack3, val1,val2, val3,val4;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup_X2_Form1");
			Util.stepUntil(Opcodes.OP_DUP_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-3);
			topStack2 = stackListBefore.get(stackBefore-2);
			topStack3 = stackListBefore.get(stackBefore-1);			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		    val1= stackListAfter.get(stackAfter - 4); 
		    val2= stackListAfter.get(stackAfter - 3); 
		    val3= stackListAfter.get(stackAfter - 2);
		    val4= stackListAfter.get(stackAfter - 1); // last element
			   
		    
		    assertEquals(topStack3,val1);
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val3);
		    assertEquals(topStack3,val4);
		
		
	}
	
	/*
	 * Bytecode: 	dup_x2
     * Coverage:    Form2
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup_X2_Form2()
     * Expected result: (3000000000L 5) -> (5 3000000000L 5)
     *
	 * */
	@Test
	public void Dup_X2_Form2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2, val3;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup_X2_Form2");
			Util.stepUntil(Opcodes.OP_DUP_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();			
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-2);
			topStack2 = stackListBefore.get(stackBefore-1);
			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		   
		    val1= stackListAfter.get(stackAfter-3); 
		    val2= stackListAfter.get(stackAfter-2);
		    val3= stackListAfter.get(stackAfter-1); // last element
			
		   		    
		   assertEquals(topStack2,val1);
		   assertEquals(topStack1,val2);
		   assertEquals(topStack2,val3);
		
		
	}
	
	/*
	 * Bytecode: 	dup2_x1
     * Coverage:    Form1
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X1_Form1()
     * Expected result: (30 20 10) -> (20 10 30 20 10) all value is category 1
     *
	 * */
	@Test
	public void Dup2_X1_Form1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, topStack3, val1,val2, val3,val4, val5;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X1_Form1");
			Util.stepUntil(Opcodes.OP_DUP2_X1);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-3);
			topStack2 = stackListBefore.get(stackBefore-2);
			topStack3 = stackListBefore.get(stackBefore-1);			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
		    val1= stackListAfter.get(stackAfter-5); 
		    val2= stackListAfter.get(stackAfter-4); 
		    val3= stackListAfter.get(stackAfter-3);
		    val4= stackListAfter.get(stackAfter-2);
			val5= stackListAfter.get(stackAfter-1); // last element
			   
		    assertEquals(topStack2,val1);
		    assertEquals(topStack3,val2);
		    assertEquals(topStack1,val3);
		    assertEquals(topStack2,val4);
		    assertEquals(topStack3,val5);
	}
	
	/*
	 * Bytecode: 	dup2_x1
     * Coverage:    Form2
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X1_Form2()
     * Expected result: (5 3000000000L) -> (3000000000L 5 3000000000L)
     *
	 * */
	@Test
	public void Dup2_X1_Form2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2, val3;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X1_Form2");
			Util.stepUntil(Opcodes.OP_DUP2_X1);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-2);
			topStack2 = stackListBefore.get(stackBefore-1);
			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		   
		    val1= stackListAfter.get(stackAfter-3);
		    val2= stackListAfter.get(stackAfter-2);
			val3= stackListAfter.get(stackAfter-1); // last element
			   
		    
		    assertEquals(topStack2,val1);
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val3);
		
	}
	
	/*
	 * Bytecode: 	dup2_x2
     * Coverage:    Form1
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X2_Form1()
     * Expected result: (40 30 20 10) -> (20 10 40 30 20 10) all value is category 1
     *
	 * */
	@Test
	public void Dup2_X2_Form1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, topStack3,topStack4, val1,val2, val3,val4, val5,val6;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X2_Form1");
			Util.stepUntil(Opcodes.OP_DUP2_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-4);
			topStack2 = stackListBefore.get(stackBefore-3);
			topStack3 = stackListBefore.get(stackBefore-2);
			topStack4 = stackListBefore.get(stackBefore-1);
			
			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
			val1= stackListAfter.get(stackAfter-6); 
			val2= stackListAfter.get(stackAfter-5); 
		    val3= stackListAfter.get(stackAfter-4); 
		    val4= stackListAfter.get(stackAfter-3); 
		    val5= stackListAfter.get(stackAfter-2);
		    val6= stackListAfter.get(stackAfter-1); // last element
			   
		    
		    assertEquals(topStack3,val1);
		    assertEquals(topStack4,val2);
		    assertEquals(topStack1,val3);
		    assertEquals(topStack2,val4);
		    assertEquals(topStack3,val5);
		    assertEquals(topStack4,val6);
			
		
	}
	
	/*
	 * Bytecode: 	dup2_x2
     * Coverage:    Form2
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X2_Form2()
     * Expected result: (30 20 3000000000L) -> (3000000000L 30 20 3000000000L)
     *
	 * */
	@Test
	public void Dup2_X2_Form2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, topStack3,val1,val2, val3,val4;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X2_Form2");
			Util.stepUntil(Opcodes.OP_DUP2_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			
			topStack1 = stackListBefore.get(stackBefore-3);
			topStack2 = stackListBefore.get(stackBefore-2);
			topStack3 = stackListBefore.get(stackBefore-1);
			
			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
			
			
		    val1= stackListAfter.get(stackAfter-4); 
		    val2= stackListAfter.get(stackAfter-3); 
		    val3= stackListAfter.get(stackAfter-2);
		    val4= stackListAfter.get(stackAfter-1); // last element
			   
		    
		    assertEquals(topStack3,val1);
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val3);
		    assertEquals(topStack3,val4);
		 
		
	}
	
	/*
	 * Bytecode: 	dup2_x2
     * Coverage:    Form3
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X2_Form3()
     * Expected result: (3000000000L 20 10) -> (20 10 3000000000L 20 10)
     *
	 * */
	@Test
	public void Dup2_X2_Form3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, topStack3, val1,val2, val3,val4, val5;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X2_Form3");
			Util.stepUntil(Opcodes.OP_DUP2_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			
			topStack1 = stackListBefore.get(stackBefore-3);
			topStack2 = stackListBefore.get(stackBefore-2);
			topStack3 = stackListBefore.get(stackBefore-1);
			
			
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
			
			val1= stackListAfter.get(stackAfter-5); 
		    val2= stackListAfter.get(stackAfter-4); 
		    val3= stackListAfter.get(stackAfter-3); 
		    val4= stackListAfter.get(stackAfter-2);
		    val5= stackListAfter.get(stackAfter-1); // last element
			   
		    
		    assertEquals(topStack2,val1);
		    assertEquals(topStack3,val2);
		    assertEquals(topStack1,val3);
		    assertEquals(topStack2,val4);
		    assertEquals(topStack3,val5);
		
	}
	
	/*
	 * Bytecode: 	dup2_x2
     * Coverage:    Form4
     * Test data:   Test.bernardini.jasminClass.DataStack.testDup2_X2_Form4()
     * Expected result: (4000000000L 3000000000L) -> (3000000000L 4000000000L 3000000000L)
     *
	 * */
	@Test
	public void Dup2_X2_Form4() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
			Value topStack1, topStack2, val1,val2, val3;
			List<Value> stackListBefore,stackListAfter;
			int stackBefore, stackAfter;
		
			Util.begin("()V", "testDup2_X2_Form4");
			Util.stepUntil(Opcodes.OP_DUP2_X2);
			stackListBefore = Util.s.getStack().currentFrame().getOperandStack().values();
			stackBefore =  stackListBefore.size();
			
			topStack1 = stackListBefore.get(stackBefore-2);
			topStack2 = stackListBefore.get(stackBefore-1);
						
			
			Util.engine.step();
			stackListAfter = Util.s.getStack().currentFrame().getOperandStack().values();
			stackAfter = stackListAfter.size();
		    
			
		    val1= stackListAfter.get(stackAfter-3); 
		    val2= stackListAfter.get(stackAfter-2);
		    val3= stackListAfter.get(stackAfter-1); // last element
			   
		    
		    assertEquals(topStack2,val1);
		    assertEquals(topStack1,val2);
		    assertEquals(topStack2,val3);
		   
		
	}
}
