package muse.algo;

import static org.junit.Assert.*;

import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Frame;
import muse.mem.Simplex;
import muse.mem.Value;

import org.junit.*;

import util.Util;

public class BytecodesPushTest 
{
	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataPush";
		Util.setUp();
	}
	
	
	/*
	 * Bytecode: 	bipush
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataPush.testBipush()
     * Expected result: operand stack at the top of the stack has value 50
     *
	 * */
	@Test
	public void Bipush() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException 
	{
		
			Value topStack,val;
		
			Util.begin("()V","testBipush");
			
			Util.stepUntil(Opcodes.OP_BIPUSH);
			val = (Value) new Simplex((int)Util.s.getInstruction(1));
			Util.engine.step();
		
			topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
			assertEquals(val,topStack);
			
		
	}
	
	/*
	 * Bytecode: 	ldc
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataPush.testLDC()
     * Expected result: operand stack at the top of the stack has value 50.1
     *
	 * */
	@Test
	public void LDC() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException 
	{
		Value val, topStack;
		int index;
		String currentClassName;
		Frame f;
		
		
		Util.begin("()V","testLDC");
		Util.stepUntil(Opcodes.OP_LDC);
		f = Util.s.getStack().currentFrame();
		byte tmp1 = Util.s.getInstruction(1);
		index = muse.algo.Util.byteCat((byte) 0, tmp1);
		currentClassName = f.getName().getClassName();
		val = Util.s.getClassFileInterface().getClassFile(currentClassName).getValueFromConstantPool(index);
		
		Util.engine.step();
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(val,topStack);
		
	}
	
	
	/*
	 * Bytecode: 	ldc2_w
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataPush.testLDC2_W()
     * Expected result: operand stack at the top of the stack has value 50.2
     *
	 * */
	@Test
	public void LDC2_W() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException 
	{
			Value val, topStack;
			int index;
			String currentClassName;
			Frame f;
		
			Util.begin("()V","testLDC2_W");
			Util.stepUntil(Opcodes.OP_LDC2_W);
			
			f = Util.s.getStack().currentFrame();
			byte tmp1 = Util.s.getInstruction(1);
			byte tmp2 = Util.s.getInstruction(2);
			index = muse.algo.Util.byteCat(tmp1, tmp2);
			currentClassName = f.getName().getClassName();
			val = Util.s.getClassFileInterface().getClassFile(currentClassName).getValueFromConstantPool(index);
		
			Util.engine.step();
			topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
			assertEquals(val,topStack);
			
		
		
	}
	
	/*
	 * Bytecode: 	ldc_w
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataPush.testLDC_W()
     * Expected result: operand stack at the top of the stack has value 20.1
     *
	 * */
	@Test 
	public void LDC_W() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException 
	{
		Value val, topStack;
		int index;
		String currentClassName;
		Frame f;
	
		Util.begin("()V","testLDC_W");
		Util.stepUntil(Opcodes.OP_LDC_W);
		
		f = Util.s.getStack().currentFrame();
		byte tmp1 = Util.s.getInstruction(1);
		byte tmp2 = Util.s.getInstruction(2);
		index = muse.algo.Util.byteCat(tmp1, tmp2);
		currentClassName = f.getName().getClassName();
		val = Util.s.getClassFileInterface().getClassFile(currentClassName).getValueFromConstantPool(index);
	
		Util.engine.step();
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(val,topStack);
		
	}
	
	/*
	 * Bytecode: 	sipush
     * Coverage:    full requirement
     * Test data:   Test.bernardini.jasminClass.DataPush.testSipush()
     * Expected result: operand stack at the top of the stack has value 6
     *
	 * */
	@Test 
	public void Sipush() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException 
	{
		Value val, topStack;
		
		
		Util.begin("()V","testSipush");
		Util.stepUntil(Opcodes.OP_SIPUSH);
		
		
		byte tmp1 = Util.s.getInstruction(1);
		byte tmp2 = Util.s.getInstruction(2);
		val = (Value) new Simplex((int) muse.algo.Util.byteCatShort(tmp1, tmp2));
		
		Util.engine.step();
		
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		assertEquals(val,topStack);
		
	}
}
