package muse.algo;

import util.Util;

import java.util.Map;


import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.CodeOutOfIndexException;
import muse.mem.Heap;
import muse.mem.Instance;
import muse.mem.LocalVariablesArea;
import muse.mem.Objekt;
import muse.mem.Simplex;
import muse.mem.Value;

import org.junit.*;
import static org.junit.Assert.*;


public class BytecodesStoreTest 
{
	public static char intType = 'I';
	public static char doubleType = 'D';
	public static char floatType = 'F';
	public static char longType = 'J';
	public static char referenceType = 'L';
	
	
	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataStore";
		Util.setUp();
	}
	
	
	/*
	 * Bytecode: 	istore_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testIStore_1()
     * Expected result: local variable 1 has int type and value the max integer
     *
	 * */
	@Test
	public void IStore_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testIStore_1");
		
		Util.stepUntil(Opcodes.OP_ISTORE_1);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(1);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),intType);
		
		
	}
	
	/*
	 * Bytecode: 	istore_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testIStore_2()
     * Expected result: local variable 2 has int type and value 16
     *
	 * */
	@Test
	public void IStore_2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testIStore_2");
		
		Util.stepUntil(Opcodes.OP_ISTORE_2);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(2);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),intType);
		
		
	}
	
	/*
	 * Bytecode: 	istore_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testIStore_3()
     * Expected result: local variable 3 has int type and value 18
     *
	 * */
	@Test
	public void IStore_3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testIStore_3");
		
		Util.stepUntil(Opcodes.OP_ISTORE_3);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(3);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),intType);
	}
	
	/*
	 * Bytecode: 	istore
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testIStoreN()
     * Expected result: local variable 4 has int type and value 20
     *
	 * */
	@Test
	public void IStore_N() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value indexVal;
		int index;
		
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
				
		Util.begin("()V","testIStoreN");
		
		Util.stepUntil(Opcodes.OP_ISTORE);
		indexVal = (Value) new Simplex((int)Util.s.getInstruction(1));
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		index = Integer.parseInt(indexVal.toString());
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(index);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),intType);
		
	}
	
	
	
	/*
	 * Bytecode: 	dstore_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testDStore_1()
     * Expected result: local variable 1 has double type and value the max double
     *
	 * */
	@Test
	public void DStore_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testDStore_1");
		
		Util.stepUntil(Opcodes.OP_DSTORE_1);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(1);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),doubleType);
		
	}
	
	/*
	 * Bytecode: 	dstore_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testDStore_2()
     * Expected result: local variable 2 has double type and value 20.2
     *
	 * */
	@Test
	public void DStore_2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testDStore_2");
		
		Util.stepUntil(Opcodes.OP_DSTORE_2);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(2);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),doubleType);
		
	}
	
	/*
	 * Bytecode: 	dstore_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testDStore_3()
     * Expected result: local variable 3 has double type and value 20.3
     *
	 * */
	@Test
	public void DStore_3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testDStore_3");
		
		Util.stepUntil(Opcodes.OP_DSTORE_3);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(3);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),doubleType);
		
	}
	
	/*
	 * Bytecode: 	dstore
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testDStoreN()
     * Expected result: local variable 4 has double type and value 20.4
     *
	 * */
	@Test
	public void DStore_N() throws EngineStuckException, DecisionRunningException, CannotManageStateException,CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value indexVal;
		int index;
		
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
				
		Util.begin("()V","testDStoreN");
		
		Util.stepUntil(Opcodes.OP_DSTORE);
		indexVal = (Value) new Simplex((int)Util.s.getInstruction(1));
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		index = Integer.parseInt(indexVal.toString());
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(index);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),doubleType);
		
	}
	
	/*
	 * Bytecode: 	fstore_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testFStore_1()
     * Expected result: local variable 1 has float type and value the max float
     *
	 * */
	@Test
	public void FStore_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testFStore_1");
		
		Util.stepUntil(Opcodes.OP_FSTORE_1);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(1);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),floatType);
		
	}
	
	/*
	 * Bytecode: 	fstore_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testFStore_2()
     * Expected result: local variable 2 has float type and value 30.2
     *
	 * */
	@Test
	public void FStore_2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testFStore_2");
		
		Util.stepUntil(Opcodes.OP_FSTORE_2);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(2);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),floatType);
		
	}
	
	/*
	 * Bytecode: 	fstore_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testFStore_3()
     * Expected result: local variable 3 has float type and value 30.3
     *
	 * */
	@Test
	public void FStore_3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testFStore_3");
		
		Util.stepUntil(Opcodes.OP_FSTORE_3);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(3);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),floatType);
		
	}
	
	/*
	 * Bytecode: 	fstore
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testFStoreN()
     * Expected result: local variable 4 has float type and value 30.4
     *
	 * */
	@Test
	public void FStore_N() throws EngineStuckException, DecisionRunningException, CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value indexVal;
		int index;
		
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
				
		Util.begin("()V","testFStoreN");
		
		Util.stepUntil(Opcodes.OP_FSTORE);
		indexVal = (Value) new Simplex((int)Util.s.getInstruction(1));
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		index = Integer.parseInt(indexVal.toString());
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(index);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),floatType);
	}
	
	/*
	 * Bytecode: 	lstore_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testLStore_1()
     * Expected result: local variable 1 has long type and value the min of long
     *
	 * */
	@Test
	public void LStore_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testLStore_1");
		
		Util.stepUntil(Opcodes.OP_LSTORE_1);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(1);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),longType);
		
	}
	
	/*
	 * Bytecode: 	lstore_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testLStore_2()
     * Expected result: local variable 2 has long type and value the max long
     *
	 * */
	@Test
	public void LStore_2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testLStore_2");
		
		Util.stepUntil(Opcodes.OP_LSTORE_2);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(2);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),longType);
		
	}
	
	/*
	 * Bytecode: 	lstore_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testLStore_3()
     * Expected result: local variable 3 has long type and value the max of long
     *
	 * */
	@Test
	public void LStore_3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
	
		Util.begin("()V","testLStore_3");
		
		Util.stepUntil(Opcodes.OP_LSTORE_3);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(3);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),longType);
		
	}
	
	/*
	 * Bytecode: 	lstore
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testLStoreN()
     * Expected result: local variable 4 has long type and value the max of long
     *
	 * */
	@Test
	public void LStore_N() throws EngineStuckException, DecisionRunningException, CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value indexVal;
		int index;
		
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
				
		Util.begin("()V","testLStoreN");
		
		Util.stepUntil(Opcodes.OP_LSTORE);
		indexVal = (Value) new Simplex((int)Util.s.getInstruction(1));
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		index = Integer.parseInt(indexVal.toString());
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(index);
		assertEquals(topStack,valLocal);
		assertEquals(valLocal.getType(),longType);
		
	}
	
		
	/*
	 * Bytecode: 	astore_1
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testAStore_1()
     * Expected result: local variable 1 has "java/lang/String" type and value is a Reference
     *
	 * */
	@Test
	public void AStore_1() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
		Heap heap;
		Util.begin("()V","testAStore_1");
		
		Util.stepUntil(Opcodes.OP_ASTORE_1);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(1);
		assertEquals(valLocal.getType(),referenceType);
		assertEquals(valLocal,topStack);
		
		//additional test
		//look if object[0] is java/lang/String
		heap = Util.s.getHeap();
		String localString = valLocal.toString();
		String objStr;
		
		
		for ( Map.Entry<Integer, Objekt> e: heap.getObjects().entrySet())
		{
			objStr = "Object["+e.getKey()+"]";
			if(objStr.equals(localString))
			{
				Objekt o = e.getValue();
				Instance a = (Instance) o;
				assertEquals("java/lang/String",a.getType());
				
			}
		}
		
	}
	
	/*
	 * Bytecode: 	astore_2
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testAStore_2()
     * Expected result: local variable 2 has "java/lang/String" type and value is a Reference
     *
	 * */
	@Test
	public void AStore_2() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
		Heap heap;
		Util.begin("()V","testAStore_2");
		
		Util.stepUntil(Opcodes.OP_ASTORE_2);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(2);
		assertEquals(valLocal.getType(),referenceType);
		assertEquals(valLocal,topStack);
		
		//additional test
		//look if object[0] is java/lang/String
		heap = Util.s.getHeap();
		String localString = valLocal.toString();
		String objStr;
		
		
		for ( Map.Entry<Integer, Objekt> e: heap.getObjects().entrySet())
		{
			objStr = "Object["+e.getKey()+"]";
			if(objStr.equals(localString))
			{
				Objekt o = e.getValue();
				Instance a = (Instance) o;
				assertEquals("java/lang/String",a.getType());
				
			}
		}
		
	}
	
	/*
	 * Bytecode: 	astore_3
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testAStore_3()
     * Expected result: local variable 3 has "java/lang/String" type and value is a Reference
     *
	 * */
	@Test
	public void AStore_3() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
		Heap heap;
		Util.begin("()V","testAStore_3");
		Util.stepUntil(Opcodes.OP_ASTORE_3);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		valLocal = local.get(3);
		assertEquals(valLocal.getType(),referenceType);
		assertEquals(valLocal,topStack);
		
		//additional test
		//look if object[0] is java/lang/String
		heap = Util.s.getHeap();
		String localString = valLocal.toString();
		String objStr;
		
		
		for ( Map.Entry<Integer, Objekt> e: heap.getObjects().entrySet())
		{
			objStr = "Object["+e.getKey()+"]";
			if(objStr.equals(localString))
			{
				Objekt o = e.getValue();
				Instance a = (Instance) o;
				assertEquals("java/lang/String",a.getType());
				
			}
		}
		
	}
	
	/*
	 * Bytecode: 	astore
     * Coverage:    full requirement
     * Test data:   Test.bernardini.Data.DataStore.testAStoreN()
     * Expected result: local variable 4 has "java/lang/String" type and value is a Reference
     *
	 * */
	@Test
	public void AStore_N() throws EngineStuckException, DecisionRunningException, CannotManageStateException, CodeOutOfIndexException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
	
		Value indexVal;
		int index;
		
		LocalVariablesArea local;
		Value valLocal;
		Value topStack;
		Heap heap;
		Util.begin("()V","testAStoreN");
		Util.stepUntil(Opcodes.OP_ASTORE);
		topStack = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		indexVal = (Value) new Simplex((int)Util.s.getInstruction(1));
		index = Integer.parseInt(indexVal.toString());
		
		
		Util.engine.step();
		local = Util.s.getStack().currentFrame().getLocalVariableTable();
		
		valLocal = local.get(index);
		assertEquals(valLocal.getType(),referenceType);
		assertEquals(valLocal,topStack);
		
		//additional test
		//look if object[0] is java/lang/String
		heap = Util.s.getHeap();
		String localString = valLocal.toString();
		String objStr;
		
		
		for ( Map.Entry<Integer, Objekt> e: heap.getObjects().entrySet())
		{
			objStr = "Object["+e.getKey()+"]";
			if(objStr.equals(localString))
			{
				Objekt o = e.getValue();
				Instance a = (Instance) o;
				assertEquals("java/lang/String",a.getType());
				
			}
		}
		
	}
	

}
