package muse.algo;

import static org.junit.Assert.*;
import muse.algo.CannotManageStateException;
import muse.algo.PartialReferenceResolutionException;
import muse.bc.Opcodes;
import muse.dec.DecisionInitException;
import muse.dec.DecisionRunningException;
import muse.jvm.EngineStuckException;
import muse.jvm.InitializationException;
import muse.jvm.NonexistingObservedVariablesException;
import muse.mem.Value;

import org.junit.*;

import util.Util;

public class BytecodesTypeConversionTest 
{
	public static char intType = 'I';
	public static char doubleType = 'D';
	public static char floatType = 'F';
	public static char longType = 'J';
	public static char referenceType = 'L';

	@Before
	public void setUp()
	{
		Util.path = "muse/algo/testdata/DataTypeConversion";
		Util.setUp();
	}

	/*
	 * Bytecode: 	i2d
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testI2D()
	 * Expected result: operand stack at the top of the stack has double type and value maximum of Int Type
	 *
	 */
	@Test
	public void I2D() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		int itop1;
		double dtop2, dcast;

		Util.begin("()V","testI2D");
		Util.stepUntil(Opcodes.OP_I2D);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		itop1 = Integer.parseInt(topStack1.toString());
		dcast = (double)itop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		dtop2 = Double.parseDouble(topStack2.toString());

		assertEquals(dcast,dtop2);
		assertEquals(topStack2.getType(),doubleType);

	}

	/*
	 * Bytecode: 	i2f
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testI2F()
	 * Expected result: operand stack at the top of the stack has float type and value maximum of Int Type
	 *
	 */
	@Test
	public void I2F() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		int itop1;
		float ftop2, fcast;


		Util.begin("()V","testI2F");
		Util.stepUntil(Opcodes.OP_I2F);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		itop1 = Integer.parseInt(topStack1.toString());
		fcast = (float)itop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ftop2 = Float.parseFloat(topStack2.toString());
		assertEquals(fcast,ftop2);
		assertEquals(topStack2.getType(),floatType);

	}

	/*
	 * Bytecode: 	i2l
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testI2L()
	 * Expected result: operand stack at the top of the stack has long type and value maximum of Int Type
	 *
	 */
	@Test
	public void I2L() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		int itop1;
		long ltop2, lcast;


		Util.begin("()V","testI2L");
		Util.stepUntil(Opcodes.OP_I2L);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		itop1 = Integer.parseInt(topStack1.toString());
		lcast = (long)itop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ltop2 = Long.parseLong(topStack2.toString());
		assertEquals(lcast,ltop2);
		assertEquals(topStack2.getType(),longType);

	}


	/*
	 * Bytecode: 	l2d
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testL2D()
	 * Expected result: operand stack at the top of the stack has double type and value maximum of Long Type
	 *
	 */
	@Test
	public void L2D() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		long ltop1;
		double dtop2, dcast;


		Util.begin("()V","testL2D");
		Util.stepUntil(Opcodes.OP_L2D);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ltop1 = Long.parseLong(topStack1.toString());
		dcast = (double)ltop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		dtop2 = Double.parseDouble(topStack2.toString());
		assertEquals(dcast,dtop2);
		assertEquals(topStack2.getType(),doubleType);

	}

	/*
	 * Bytecode: 	l2f
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testL2F()
	 * Expected result: operand stack at the top of the stack has float type and value maximum of Long Type
	 *
	 */
	@Test
	public void L2F() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		long ltop1;
		float ftop2, fcast;

		Util.begin("()V","testL2F");
		Util.stepUntil(Opcodes.OP_L2F);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ltop1 = Long.parseLong(topStack1.toString());
		fcast = (float)ltop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ftop2 = Float.parseFloat(topStack2.toString());
		assertEquals(fcast,ftop2);
		assertEquals(topStack2.getType(),floatType);


	}

	/*
	 * Bytecode: 	f2d
	 * Coverage:    full requirement
	 * Test data:   Test.bernardini.Data.DataTypeConversion.testF2D()
	 * Expected result: operand stack at the top of the stack has double type and value maximum of Float Type
	 *
	 */
	@Test
	public void F2D() throws EngineStuckException, DecisionRunningException, CannotManageStateException, DecisionInitException, InitializationException, NonexistingObservedVariablesException, PartialReferenceResolutionException
	{
		Value topStack1, topStack2;
		float ftop1;
		double dtop2, dcast;


		Util.begin("()V","testF2D");
		Util.stepUntil(Opcodes.OP_F2D);
		topStack1 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		ftop1 = Float.parseFloat(topStack1.toString());
		dcast = (double) ftop1;

		Util.engine.step();
		topStack2 = (Value)Util.s.getStack().currentFrame().getOperandStack().returnLast();
		dtop2 = Double.parseDouble(topStack2.toString());
		assertEquals(dcast,dtop2);
		assertEquals(topStack2.getType(),doubleType);


	}
}
